/*************************************
  
g++ -O3 CG.cpp rational.o longint.o ratutil.o thread_manager.o mempool.o gettimeofday.o

n   size of problem
which =0 Frank matrix,  
      =1 Hilbert matrix,  
      =2 uniform random number by multiplcative congruential method of U(i+1)=16807*U(i) % 2147483647
      =3 Thermal conduction matrix (derogatory)

#define RHSRAND  uniform random number by multiplcative congruential method of U(i+1)=16807*U(i) % 2147483647
        otherwise RHS equals to unit vector

 ***************************************************************************/

//#define RHSRAND
//#define RBLAS

#include <iostream>
#include <math.h>
#include <algorithm>
#include <stdexcept>
#include "rational.h"
#include "ratutil.h"   // 2015oct18
#include "rblas.h"
#include "prblas.h"
//#include "interval.h"
#include "rvector.h"
#include "rmatrix.h"
#include "archive.h"

long long gettime();
void cg(const rat::matrix<rational>& a, const rat::vector<rational>& b, rat::vector<rational>& x, const int n, int& nit);


int main()
{
	int n, m, nit, which;
	long long iseed=1;

	long long tstart = gettime();

	std::cout << "enter n, which(0-3): "; std::flush(std::cout);
	std::cin >> m >> which;
	std::cout << "m=" << m << " which=" << which << std::endl;

	n=m;
	if(which==3) {
		n=m*m;
		std::cout << "n=" << n << std::endl;
	}

	rat::matrix<rational> a(n,n);
	rat::vector<rational> b(n),x(n);

	if (which == 0) SetFrank(a,n);
	if (which == 1) SetHilbert(a,n);
	if (which == 2) SetRand(a,n,iseed);
	if (which == 3) SetFtherm(a,m,0);

	matprn(a,n,n);

#ifdef RHSRAND
	for (int i=0; i < n; ++i) {
		double t=urand(); b[i] = Rdset(&t); x[i] = 0;
	}
#else
	for (int i=0; i < n; ++i) { b[i] = 1; x[i] = 0; }
#endif

	tstart = gettime();
	cg(a,b,x,n,nit);
	std::cout << "cg time=" << (double)(gettime()-tstart)/1000000.0 << "(sec)" << std::endl;

#ifdef EPROF2
	std::cout << "lgcd2time=" << (double)longint::lgcd2time/1000000.0 << " lgcd2count=" << longint::lgcd2count  << std::endl;
	std::cout << "ldividetime=" << (double)longint::ldividetime/1000000.0 << " ldividecount=" << longint::ldividecount  << std::endl;
	std::cout << "lmultime=" << (double)longint::lmultime/1000000.0 << " lmulcount=" << longint::lmulcount  << std::endl;
#endif
}

void cg(const rat::matrix<rational>& a, const rat::vector<rational>& b, rat::vector<rational>& x, const int n, int& nit)
{
	rat::vector<rational> r(n), z(n), p(n), q(n);
	rational alpha, beta, pq, bnrm2, rr, resid, rho, rho1;
	int i, k;

#ifdef RBLAS
	rat::rgemv(rational::ONE,a,x,rational::ZERO,r);
#else
	rat::prgemv(rational::ONE,a,x,rational::ZERO,r);
#endif
	for (i=0; i < n; ++i) r[i] = b[i]-r[i];
#ifdef RBLAS
	bnrm2 = rat::rdot(b,b);
#else
	bnrm2 = rat::prdot(b,b);
#endif
	rr = bnrm2;

	for (k=1; k <= n+1; ++k) {
		resid=RatAbs(rr/bnrm2);
		std::cout  << "iteration " << k << ", resid=" << (double)resid << std::endl;
		rho = rr;
		if (k > 1) {
			beta = rho/rho1;
#ifdef RBLAS
			rat::rcopy(r,z);
			rat::raxpy(beta,p,z);
			rat::rcopy(z,p);
#else
			rat::prcopy(r,z);
			rat::praxpy(beta,p,z);
			rat::prcopy(z,p);
#endif
		} else {
			beta = rational::ZERO;
#ifdef RBLAS
			rat::rcopy(r,p);
#else
			rat::prcopy(r,p);
#endif
		}
#ifdef RBLAS
		rat::rgemv(rational::ONE,a,p,rational::ZERO,q);
		pq = rat::rdot(p,q);
		alpha = rho/pq;
		rat::raxpy(alpha,p,x);   //  current solution vector
		rat::raxpy(-alpha,q,r);  //  residual vector
		rr = rat::rdot(r,r);              //   ScalVec
#else
		rat::prgemv(rational::ONE,a,p,rational::ZERO,q);
		pq = rat::prdot(p,q);
		alpha = rho/pq;
		rat::praxpy(alpha,p,x);   //  current solution vector
		rat::praxpy(-alpha,q,r);  //  residual vector
		rr = rat::prdot(r,r);              //   ScalVec
#endif
//		std::cout <<   r << std::endl;
//		std::cout << "rdot=" << rr <<  std::endl;
		resid = rr/bnrm2;
		longint rsd = resid.denominator();
		longint rsn = resid.numerator();
		std::cout << " Num of Dgts (resid)=" << numdgts(rsd)+numdgts(rsn) <<  std::endl;
		if (resid <= rational::ZERO) break;
		rho1 = rho;
	}
	if (resid > rational::ZERO) {
		std::cerr << "Iteration fail in subroutine cg" << std::endl;
		exit(1);
	}
	std::cout << "Converged CG k=" << k << std::endl;
	nit = k;
}
