/********************************************************************
  CGsc.cpp
  
  created on: 2013.Dec.08
      author: Hikaru Samukawa
 

n   size of problem
which =0 Frank matrix,  =1  Hilbert matrix,  =2  rand, =3  thermal conduction
scal =0  No scaling,    =1   scaling 

#define RHSRAND  uniform random number
        otherwise RHS equals to unit vector


g++ -O3 CGsc.cpp -c 
g++ CGsc.o rational.o longint.o ratutil.o mempool.o gettimeofday.o

 **************************************************************************/

//#define RHSRAND

#include <iostream>
#include <math.h>
#include <algorithm>
#include <stdexcept>
#include "rational.h"
#include "ratutil.h"    //  2015oct18
#include "rblas.h"
#include "prblas.h"
#include "rvector.h"
#include "rmatrix.h"
#include "archive.h"

//using namespace std;

long long gettime();

void cgsc(const rat::matrix<rational>& a, const rat::vector<rational>& b, rat::vector<rational>& x, const int n, int& nit, const int& scal);

int main()
{
	int n, m, nit, which, scal;
	long long iseed=1;

	long long tstart = gettime();

	std::cout << "enter n, which(0-3) scal(0-1): "; std::flush(std::cout);
	std::cin >> m >> which >> scal;
	std::cout << "m=" << m << " which=" << which << " scal=" << scal << std::endl;

	n=m;
	if(which==3) {
		n=m*m;
		std::cout << "n=" << n << std::endl;
	}

	rat::matrix<rational> a(n,n);
	rat::vector<rational> b(n), x(n);

	if (which == 0) SetFrank(a,n);
	if (which == 1) SetHilbert(a,n);
	if (which == 2) SetRand(a,n,iseed);
	if (which == 3) SetFtherm(a,m,0);

	matprn(a,n,n);

#ifdef RHSRAND
	for (int i=0; i < n; ++i) {
		double t=urand(); b[i] = Rdset(&t); x[i] = 0;
	}
#else
	for (int i=0; i < n; ++i) { b[i] = 1; x[i] = 0; }
#endif

	tstart = gettime();
	cgsc(a,b,x,n,nit,scal);
	std::cout << "cg time=" << (double)(gettime()-tstart)/1000000.0 << "(sec)" << std::endl;

#ifdef EPROF2
	std::cout << "lgcd2time=" << (double)longint::lgcd2time/1000000.0 << " lgcd2count=" << longint::lgcd2count  << std::endl;
	std::cout << "ldividetime=" << (double)longint::ldividetime/1000000.0 << " ldividecount=" << longint::ldividecount  << std::endl;
	std::cout << "lmultime=" << (double)longint::lmultime/1000000.0 << " lmulcount=" << longint::lmulcount  << std::endl;
#endif

}

void cgsc(const rat::matrix<rational>& a, const rat::vector<rational>& b, rat::vector<rational>& x, const int n, int& nit, const int& scal)
{
	rat::vector<rational> r(n), z(n), p(n), q(n);
	rational alpha, beta, pq, bnrm2, rr, resid, rho, rho1;
	rational sr, sp;                          //   ScalVec
	rat::vector<rational> rs(n),ps(n);        //   ScalVec
	int i, k;
	for (i=0; i < n; ++i) { r[i] = 0;}
#ifdef RBLAS
	rat::rgemv(rational::ONE,a,x,rational::ZERO,r);
#else
	rat::prgemv(rational::ONE,a,x,rational::ZERO,r);
#endif
	for (i=0; i < n; ++i) {
		r[i] = b[i]-r[i];
	}
	ScalVec(r,n,sr,rs,1);                      //   ScalVec
#ifdef RBLAS
	bnrm2 = rat::rdot(b,b);
#else
	bnrm2 = rat::prdot(b,b);
#endif
	rr = bnrm2;
	for (k=1; k <= n+1; ++k) {
		resid=RatAbs(rr/bnrm2);
		std::cout  << "iteration " << k << ", resid=" << (double)resid << std::endl;
		rho = rr;
		if (k > 1) {
			beta = rho/rho1;
#ifdef RBLAS
			rat::rcopy(r,z);
			rat::raxpy(beta,p,z);
			rat::rcopy(z,p);
#else
			rat::prcopy(r,z);
			rat::praxpy(beta,p,z);
			rat::prcopy(z,p);
#endif
		} else {
			beta = rational::ZERO;
			rat::rcopy(r,p);
		}
		if(scal==1){                                      //   ScalVec
			ScalVec(p,n,sp,ps,1);                     //   ScalVec
		}
		for (i=0; i < n; ++i) { q[i] = 0;}
		if(scal==0){                                      //   ScalVec
#ifdef RBLAS
			rat::rgemv(rational::ONE,a,p,rational::ZERO,q);
#else
			rat::prgemv(rational::ONE,a,p,rational::ZERO,q);
#endif
		} else {                                          //   ScalVec
#ifdef RBLAS
			rat::rgemv(rational::ONE,a,ps,rational::ZERO,q);
#else
			rat::prgemv(rational::ONE,a,ps,rational::ZERO,q);
#endif
		}                                                 //   ScalVec
		if(scal==0){                                      //   ScalVec
#ifdef RBLAS
			pq = rat::rdot(p,q);
#else
			pq = rat::prdot(p,q);
#endif
		} else {                                          //   ScalVec
#ifdef RBLAS
			pq = rat::rdot(ps,q) * sp * sp;
#else
			pq = rat::prdot(ps,q) * sp * sp;
#endif
		}                                                 //   ScalVec
		alpha = rho/pq;
		if(scal==0){                                      //   ScalVec
#ifdef RBLAS
			rat::raxpy(alpha,p,x);   //  current solution vector
			rat::raxpy(-alpha,q,r);  //  residual vector
#else
			rat::praxpy(alpha,p,x);   //  current solution vector
			rat::praxpy(-alpha,q,r);  //  residual vector
#endif
		} else {                                          //   ScalVec
#ifdef RBLAS
			rat::raxpy(alpha*sp,ps,x);  //  current solution vector
			rat::raxpy(-alpha*sp,q,r);  //  residual vector
#else
			rat::praxpy(alpha*sp,ps,x);  //  current solution vector
			rat::praxpy(-alpha*sp,q,r);  //  residual vector
#endif
			ScalVec(r,n,sr,rs,1);                     //   ScalVec
		}                                                 //   ScalVec
		if(scal==0){                                      //   ScalVec
#ifdef RBLAS
			rr = rat::rdot(r,r);              //   ScalVec
#else
			rr = rat::prdot(r,r);              //   ScalVec
#endif
		} else {                                          //   ScalVec
#ifdef RBLAS
			rr = rat::rdot(rs,rs) * sr * sr ; //   ScalVec;
#else
			rr = rat::prdot(rs,rs) * sr * sr ; //   ScalVec;
#endif
		}                                                 //   ScalVec
		resid = rr/bnrm2;
		longint rsd = resid.denominator();
		longint rsn = resid.numerator();
		std::cout << " Num of Dgts (resid)=" << numdgts(rsd)+numdgts(rsn) <<  std::endl;
		if (resid <= rational::ZERO) break;
		rho1 = rho;
	}
	if (resid > rational::ZERO) {
		std::cerr << "Iteration fail in subroutine cgsc" << std::endl;
		exit(1);
	}
	std::cout << "Converged CGsc k=" << k << std::endl;
	nit = k;
}
