/*
 *   g++ GauLeg.cpp longint.o rational.o interval.o ratutil.o thread_manager.o mempool.o gettimeofday.o
 *       2016 May 19 --> May 29 ==> Oct 29 ==> Nov 02
 *
 *
 *
 * */

#include <stdexcept>
#include <math.h>
#include <iostream>
#include <sstream>    // std::stringstream
#include <iomanip>
#include "rational.h"
#include "rvector.h"
#include "rmatrix.h"
#include "longint.h"
#include "interval.h"
#include "ratutil.h"
#include "rblas.h"
#include "rough.h"
#include "numeric_cast.h"
//#include "prblas.h"

#define SQRRATIONALB

using namespace rat;

long long gettime();

void SQRndchk(const rational& a)
{
	longint an=a.N; longint ad=a.D;
	uint32_t antz=an.longint::ctz(); uint32_t adtz=ad.longint::ctz();
	if(numbits(an)-antz == 1 || numbits(ad)-adtz == 1){  // either an or ad is power of 2
		if(numbits(ad)-adtz == 1){
			std::cout << " SQRndchk ad power two" << std::endl;
		}else if(numbits(an)-antz == 1){
			std::cout << " SQRndchk an power two" << std::endl;
		}
	}else{                                     // 
		std::cout << " SQRndchk both an and ad are not power of 2" << std::endl;
	}
}

int main() 
{
	rational a,b,p,t,x,y,sqrt2,by,zl,zu;
	interval z,Z;
	int i,tol, ttol;
	long long stime=0;            //  SQR timer
	long long xstart, ystart;       //  SQR timer
	extern long long timeSQR;     //timerratutil
	extern long long timeround;   //timerratutil
	extern long long timeSQR1;     //timerratutil
	extern long long timeSQR2;     //timerratutil
	extern long long timeSQR3;     //timerratutil
	extern long long timesqrt4g1; //timerratutil
	extern long long timesqrt4g2; //timerratutil
	extern long long timesqrt4g3; //timerratutil
	extern long long timesqrt4g4; //timerratutil
	extern long long timesqrt4g5; //timerratutil
	extern long long timesqrt4g6; //timerratutil

	std::cout << "enter tol: ";
	std::cin >> tol ;
	std::cout << tol << std::endl;
	ttol=tol+tol/2;  // ttol=1.5*tol  used for SQR
	if(tol>99)  ttol=tol+tol/10;// ttol=1.1*tol used for SQR
//	if(tol>999) ttol=tol+tol/100;// ttol=1.01*tol used for SQR
	if(tol>1000) ttol=tol+tol/100;// ttol=1.01*tol used for SQR
	std::cout << " ttol=" << ttol << std::endl;

	long long tstart=gettime();
	interval pii=setpi(ttol);
	std::cout << "setpi time=" << (double)(gettime()-tstart)/1000000.0 << "(sec)" << std::endl;
	std::cout << " pii=" << std::endl; formatprn(pii);
	rational pil=pii.lower(); rational piu=pii.upper(); rational pim=(pil+piu)/rational::TWO;
	std::cout << " (double)pim=" << (double)pim << std::endl;

//      select gcd algorithm
//	longint::lgcd_p=longint::lgcd;  // 0.14 sec --> 1.35 sec (n=2000) b=sqrt(2)/2

	tstart=gettime();
	a=rational::ONE;
	sqrt2=SQR(rational::TWO,ttol);
// 	b=rational::ONE/sqrt2;
	b=sqrt2/rational::TWO;
	t=RRset(longint::ONE,longint::FOUR);
	x=rational::ONE;
	int nit=(int)log2((double)tol);
	for(int i=0; i<=nit;++i){
		y=a;
		a=(a+b)/rational::TWO;
//		longint::lgcd_p=longint::lgcd2;
		by=b*y;
//		longint::lgcd_p=longint::lgcd1;
		/****************/
		long long sstart=gettime();
//		SQRndchk(by);
#ifdef SQRRATIONALB
		b=SQR(by,ttol);
#else
		z=SQRint(by,ttol);
//		std::cout << " log10int(z)=" <<  log10int(z) << std::endl;
//		std::cout << " z=" <<  std::endl; formatprn(z);
		zl=z.lower(); zu=z.upper(); b=(zl+zu)/rational::TWO;
		/****************************************
		rational c=SQR(by,ttol);
		std::cout << " numbits(b)=" <<  numbits(b.numerator()) << " " <<  numbits(b.denominator());
		std::cout << " numbits(c)=" <<  numbits(c.numerator()) << " " <<  numbits(c.denominator()) <<  std::endl;
		if(b>c){
			interval bc(c,b);
			std::cout << " b is upper" << std::endl; formatprn(bc);
		}else{
			interval bc(b,c);
			std::cout << " c is upper" << std::endl; formatprn(bc);
		}
		****************************************/
#endif
		stime+=gettime()-sstart;
		/****************/
		t=t-x*(y-a)*(y-a);
		x=x*rational::TWO;
	}
	t=t-x*(y-a)*(y-a);
	std::cout << " numbits(t)=" <<  numbits(t.numerator()) << " " <<  numbits(t.denominator()) <<  std::endl;
	p=((a+b)*(a+b))/(rational::FOUR*t);
	std::cout << "Gauss Legendre time to compute pi=" << (double)(gettime()-tstart)/1000000.0 << "(sec)" << std::endl;
	if(p>pim){
		interval ppi(pim,p); z=ppi;
		std::cout << " p is upper" << std::endl; formatprn(ppi);
	}else{
		interval ppi(p,pim); z=ppi;
		std::cout << " p is lower" << std::endl; formatprn(ppi);
	}
	int lz=(int)(-log10int(z));   // lz is where the difference begins in interval ab.
	std::cout << "lz=" << lz << std::endl;
	std::cout << "loop time=" << (double)(gettime()-tstart)/1000000.0 << "(sec)" << std::endl;
	std::cout << "SQR time=" << (double)(stime)/1000000.0 << "(sec)" << std::endl;
	std::cout << "timeSQR=" << (double)(timeSQR)/1000000.0 << "(sec)" << std::endl;      //timerratutil
	std::cout << "timeround=" << (double)(timeround)/1000000.0 << "(sec)" << std::endl;  //timerratutil
	std::cout << "timeSQR1=" << (double)(timeSQR1)/1000000.0 << "(sec)" << std::endl;  //timerratutil
	std::cout << "timeSQR2=" << (double)(timeSQR2)/1000000.0 << "(sec)" << std::endl;  //timerratutil
	std::cout << "timeSQR3=" << (double)(timeSQR3)/1000000.0 << "(sec)" << std::endl;  //timerratutil
	std::cout << "timesqrt4g1=" << (double)(timesqrt4g1)/1000000.0 << "(sec)" << std::endl;  //timerratutil
	std::cout << "timesqrt4g2=" << (double)(timesqrt4g2)/1000000.0 << "(sec)" << std::endl;  //timerratutil
	std::cout << "timesqrt4g3=" << (double)(timesqrt4g3)/1000000.0 << "(sec)" << std::endl;  //timerratutil
	std::cout << "timesqrt4g4=" << (double)(timesqrt4g4)/1000000.0 << "(sec)" << std::endl;  //timerratutil
	std::cout << "timesqrt4g5=" << (double)(timesqrt4g5)/1000000.0 << "(sec)" << std::endl;  //timerratutil
	std::cout << "timesqrt4g6=" << (double)(timesqrt4g6)/1000000.0 << "(sec)" << std::endl;  //timerratutil

#ifdef EPROF2
	std::cout << "lgcd2time=" << (double)longint::lgcd2time/1000000.0 << " lgcd2count=" << longint::lgcd2count  << std::endl;
	std::cout << "lgcdtime=" << (double)longint::lgcdtime/1000000.0 << " lgcdcount=" << longint::lgcdcount  << std::endl;
	std::cout << "ldividetime=" << (double)longint::ldividetime/1000000.0 << " ldividecount=" << longint::ldividecount  << std::endl;
//	std::cout << "opsubeqtime=" << (double)longint::opsubeqtime/1000000.0 << " opsubeqcount=" << longint::opsubeqcount  << std::endl;
	std::cout << "shrtime=" << (double)longint::shrtime/1000000.0 << " shrcount=" << longint::shrcount  << std::endl;
	std::cout << "lmultime=" << (double)longint::lmultime/1000000.0 << " lmulcount=" << longint::lmulcount  << std::endl;
	std::cout << "laddtime=" << (double)longint::laddtime/1000000.0 << " laddcount=" << longint::laddcount  << std::endl;
#endif

}
