/*
 * LDL.cpp
 * 
 *     author: Hikaru Samukawa
 * 
 *     2015 oct 17, 2016 mar 25 add SetRand2
 * 
g++ -O3 ratutil.cpp  -DNUMTHREADS=4 -c 
g++ -O3 LDL.cpp -c 
g++ LDL.o rational.o longint.o ratutil.o thread_manager.o mempool.o gettimeofday.o -lpthread
 * 
 */
#include <iostream>
#include <math.h>
#include <algorithm>
#include <stdexcept>
#include "longint.h"
#include "rational.h"
#include "ratutil.h"
#include "rblas.h"
#include "prblas.h"
#include "rvector.h"
#include "rmatrix.h"
#define LGCD1
long long gettime();

/*
 ./LDL.exe n which

             which = 0,1,2,3,4 

 ./LDL.exe  10 2 : solve n=10 problem, for matrix=2
*/
void LDLnaive(rat::matrix<rational>& a, const int n){
    rational s,t;
    int i,j,k;
    for (j=1; j < n; ++j) {
	std::cout << "LDLnaive j=" << j << std::endl;
        for (i=1; i < j; ++i) {
            s=0;
            for (k=0; k < i; ++k)  s = s +  a[k][i] * a[k][j];
	    std::cout << " i=" << i << " A(" << i << "," << j << ")=" << a[i][j];
            a[i][j] = a[i][j] - s;
	    std::cout <<  " dot s=" << s << " Aij=" << a[i][j] ;
        }
	std::cout << std::endl;
	std::cout << " diagonal loop  s=0" << std::endl;
        s=0;
        for (k=0; k <= j-1; ++k) {
            t = a[k][j]/a[k][k];
	    std::cout << " k=" << k << " A(" << k << "," << j << ")=" << a[k][j] <<  " / A(" << k << "," << k << ")=" <<a[k][k] << " t=" << t;
            s = s + t * a[k][j];
	    std::cout << " *A(" << k << "," << j << ")=" << a[k][j] << " becomes s=" << s;
            a[k][j] = t;
	    std::cout << " store t in A(" << k << "," << j <<")=" << a[k][j];
        }
	std::cout  << std::endl;
	std::cout  << " Diag A(" << i << "," << j << ")=" << a[i][j] << " - s=" << s;
        a[j][j] = a[j][j] - s;
	std::cout  << " =  A(" << i << "," << j << ")=" << a[i][j] <<  std::endl;
    }
}

int main(int argc, char *argv[])
{
	int n, m, which;
	long long iseed=1L;
	rational lcm;

	long long tstart = gettime();

	n = atoi(argv[1]); which = atoi(argv[2]);
	std::cout << "n=" << n << " which=" << which << std::endl;
	if(which==6){
		m = n; n = m * m;
		std::cout << "n is replaced with =" << n << std::endl;
	}
	rat::matrix<double> d(n,n);
	rat::matrix<rational> a(n,n),c(n,n);
	rat::vector<rational> b(n), x(n);

	if (which == 0) SetFrank(a,n);
	if (which == 1) SetHilbert(a,n);
	if (which == 2) {
		SetHilbert(d,n);
		cnvmat(d,a,n,n);    //  covert double D to rational A
	}
	if (which == 3) {
		SetHilbert(d,n);
		cnvmat(d,c,n,n);    //  covert double D to rational
//		SetHilbert(c,n);
		lcm=rat2int(c,n,a); //  multiply AA by its LCM
		std::cout << "LCM=" << lcm << std::endl;
	}
	if (which == 4) {
		SetRand (a,n,iseed);
	}
	if (which == 41) {
		SetRand (c,n,iseed);
		lcm=rat2int(c,n,a); //  multiply AA by its LCM
		std::cout << "LCM=" << lcm << std::endl;
	}
	if (which == 5) {
		SetRand2(a,n,iseed);
	}
	if (which == 51) {
		SetRand2(c,n,iseed);
		lcm=rat2int(c,n,a); //  multiply AA by its LCM
		std::cout << "LCM=" << lcm << std::endl;
	}
	if (which == 6) {
		SetFtherm(d,m);
		cnvmat(d,a,n,n);    //  covert double D to rational A
	}
	std::cout << "matrix A" << std::endl;
	matprn(a,n,n);
//	matprt(a,n,n);
//	MatDgtCountPrt(a,n,n);

	for (int i=0; i < n; ++i) { b[i] = 1; x[i] = 0; }
	std::cout << "b=" << b << std::endl;
	std::cout << "x=" << x << std::endl;

	tstart = gettime();
//	rat::rgemv(rational::ONE,a,b,rational::ZERO,x);
//	std::cout << "rgemv time=" << (double)(gettime()-tstart)/1000000.0 << "(sec)" << std::endl;
	rat::prgemv(rational::ONE,a,b,rational::ZERO,x);
	std::cout << "prgemv time=" << (double)(gettime()-tstart)/1000000.0 << "(sec)" << std::endl;
//	std::cout << "righthand-side " << x << std::endl;

	std::cout << "LDL start" <<  std::endl;
#ifdef LGCD1
	longint::lgcd_p=longint::lgcd1;
#endif
	tstart = gettime();
	LDL(a,n);
//	LDLnaive(a,n);
	std::cout << "LDL time=" << (double)(gettime()-tstart)/1000000.0 << "(sec)" << std::endl;
//	matprn(a,n,n);
//	MtrDgtCountPrt(a,n);

	tstart = gettime();
	LDLsubst(a,n,x);
	std::cout << "LDLsubst time=" << (double)(gettime()-tstart)/1000000.0 << "(sec)" << std::endl;
//	std::cout << "x=" << x << std::endl;
	std::cout << "end ratLDL n=" << n << " matrix is "<< which << std::endl;
	int bad = 0;
	for (int i=0; i < n; ++i) {
		if(x[i]-b[i] != rational::ZERO) bad=i+1;
	}
	if(bad == 0){
		std::cout << "result OK" << std::endl;
	} else {
		std::cout << "result bad i=" << bad << std::endl;
	}
	
#ifdef EPROF2
	std::cout << "lgcd2time=" << (double)longint::lgcd2time/1000000.0 << " lgcd2count=" << longint::lgcd2count  << std::endl;
	std::cout << "ldividetime=" << (double)longint::ldividetime/1000000.0 << " ldividecount=" << longint::ldividecount  << std::endl;
	std::cout << "lmultime=" << (double)longint::lmultime/1000000.0 << " lmulcount=" << longint::lmulcount  << std::endl;
#endif

	std::cout << "see diagonal terms before compute determinant" << std::endl;
	SeeMatDiag(a,n);

	tstart = gettime();
	rational t=rational::ONE;
	for (int i=0; i < n; ++i) t = t * a[i][i];
	long long tend=gettime();
	std::cout << "determinant time=" << (double)(tend-tstart)/1000000.0 << "(sec)" << std::endl;
	std::cout << "tstart=" << tstart << " tend=" << tend << std::endl;
// 	std::cout << "determinant=" << t.format(80,60)  << " " << t << std::endl;
 	std::cout << "determinant=" << t.format(80,60)  << std::endl;
}
