/*
 * LLSschmidt.cpp
 * 
 * created on: 2013.Jul.26
 *     author: Hikaru Samukawa
 * 
g++ -O3 LLSschmidt.cpp -c 
g++ LLSschmidt.o rational.o longint.o rblas.o mempool.o gettimeofday.o
g++ LLSschmidt.o rational.o longint.o ratutil.o mempool.o gettimeofday.o
 
 ./LLSschmidt.exe 50 20 0   
                  m  n mtd, where mthd =0:MGS, 
                                       =1:CGS,
                                       =2: scale(A) then CGS,
                                       =3: CGS with scaling
 */

#include <iostream>
#include <math.h>
#include <algorithm>
#include <stdexcept>
#include "longint.h"
#include "rational.h"
#include "ratutil.h"
#include "rblas.h"
#include "prblas.h"
#include "rvector.h"   // MersenneTwister  2013.Sep.11
#include "rmatrix.h"   // MersenneTwister  2013.Sep.11
#include <fstream>
#include <sstream>
#include "archive.h"

long long gettime();

void MGS(rat::matrix<rational>& a, const int m, const int n, rat::matrix<rational>& r);
void CGS(rat::matrix<rational>& a, const int m, const int n, rat::matrix<rational>& r);
void CGSsc(rat::matrix<rational>& a, const int m, const int n, rat::matrix<rational>& r, rat::vector<rational>&);
void bwdgssbst(const rat::matrix<rational>& r, const int n, rat::vector<rational>& z);

int main(int argc, char *argv[])
{
	int m, n, mthd;            //  m >=  n
	long long iseed=1;
	long long tstart = gettime();

	m = atoi(argv[1]); n = atoi(argv[2]); mthd = atoi(argv[3]);
	std::cout << "m=" << m << " n=" << n <<  " mthd=" << mthd << std::endl;

	rat::matrix<rational> a(m,n), r(n,n);
	rat::vector<rational> b(m), x(n);
	rat::vector<rational> z(m), w(m);
	rat::matrix<rational> aa(m,n);
	rat::vector<rational> as(n);

	FloatUrandR(a,m,n);

	for (int i=0; i < n; ++i) x[i] = 1;
#ifdef RBLAS
	rat::rgemv(rational::ONE,a,x,rational::ZERO,b);
#else
	rat::prgemv(rational::ONE,a,x,rational::ZERO,b);
#endif

	MatDgtCount(a,m,n);

	tstart = gettime();

	if(mthd == 0){
		MGS(a,m,n,r);
		std::cout << "MGS time=" << (double)(gettime()-tstart)/1000000.0 << "(sec)" << std::endl;
		std::cout << "end MGS m=" << m << " n=" << n << std::endl;
	}else if(mthd == 1){
		CGS(a,m,n,r);
		std::cout << "CGS time=" << (double)(gettime()-tstart)/1000000.0 << "(sec)" << std::endl;
	}else if(mthd == 2){
		ScalMat(a,m,n,as,aa);
//		std::cout << "as=" << as << std::endl;
		CGS(aa,m,n,r);
		std::cout << "CGS with scaling time=" << (double)(gettime()-tstart)/1000000.0 << "(sec)" << std::endl;
	}else{
		CGSsc(a,m,n,r,as);
		std::cout << "CGSsc time=" << (double)(gettime()-tstart)/1000000.0 << "(sec)" << std::endl;
	}
	MtrDgtCount(r,n);


	for (int i=0; i < n; ++i) { x[i] = 0; }
	if(mthd == 2){
#ifdef RBLAS
		rat::rgemtv(rational::ONE,aa,b,rational::ZERO,x);
#else
		rat::prgemtv(rational::ONE,aa,b,rational::ZERO,x);
#endif
	}else{
#ifdef RBLAS
		rat::rgemtv(rational::ONE,a,b,rational::ZERO,x);
#else
		rat::prgemtv(rational::ONE,a,b,rational::ZERO,x);
#endif
	}  
	if(mthd > 2){
		for (int j=0; j < n; ++j) x[j]=x[j]*as[j];
	}  


	bwdgssbst (r,n,x);

	if(mthd == 2 ){
		for (int i=0; i < n; ++i) x[i] = x[i] / as[i];
	}  

//	std::cout << "x=" << x << std::endl;

	int bad = 0;
	for (int i=0; i < n; ++i) {
		if(x[i]-rational::ONE != rational::ZERO) bad=i+1;
	}
	if(bad == 0){
		std::cout << "result OK" << std::endl;
	} else {
		std::cout << "result bad i=" << bad << std::endl;
	}
//////////////////////////////////////////////////////////////
#ifdef EPROF2
	std::cout << "lgcd2time=" << (double)longint::lgcd2time/1000000.0 << " lgcd2count=" << longint::lgcd2count  << std::endl;
	std::cout << "lgcdtime=" << (double)longint::lgcdtime/1000000.0 << " lgcdcount=" << longint::lgcdcount  << std::endl;
	std::cout << "ldividetime=" << (double)longint::ldividetime/1000000.0 << " ldividecount=" << longint::ldividecount  << std::endl;
	std::cout << "lmultime=" << (double)longint::lmultime/1000000.0 << " lmulcount=" << longint::lmulcount  << std::endl;
#endif

}


void MGS(rat::matrix<rational>& a, const int m, const int n, rat::matrix<rational>& r)
{
	for (int j=0; j < n; ++j) {
		for (int k=0; k <= j-1; ++k) {  // since both 0 and j-1 are shifted.
			rational s = rational::ZERO;
			for(int i=0; i < m; i++) {
				s += a[i][k]*a[i][j];
//				rational t = a[i][k]*a[i][j];
//				longint td = t.denominator();
//				longint tn = t.numerator();
//				ndgtt = ndgtt + numdgts(td) + numdgts(tn);
//				s += t;
//				std::cout << " s=" << s << std::endl;
			}
			r[k][j] = s / r[k][k];
			for(int i=0; i < m; i++) {
				a[i][j] = a[i][j] - r[k][j]*a[i][k];
			}
		}
		rational s = rational::ZERO;
		for(int i=0; i < m; i++) {
			s += a[i][j]*a[i][j];
		}
		r[j][j] = s;
	}
}

void CGS(rat::matrix<rational>& a, const int m, const int n, rat::matrix<rational>& r)
{
	double timedots=0;
	double timeaxpy=0;
	long long tstart;
	int ndgts=0;
	int ndgtt=0;
	int ndgtu=0;
	for (int j=0; j < n; ++j) {
		tstart = gettime();
		for (int k=0; k <= j-1; ++k) {  // since both 0 and j-1 are shifted.
			rational s = rational::ZERO;
			for(int i=0; i < m; i++) {
				s += a[i][k]*a[i][j];
//				rational t = a[i][k]*a[i][j];
//				longint td = t.denominator();
//				longint tn = t.numerator();
//				ndgtt = ndgtt + numdgts(td) + numdgts(tn);
//				s += t;
			}
			longint sd = s.denominator();
			longint sn = s.numerator();
			ndgts = ndgts + numdgts(sd) + numdgts(sn);
			r[k][j] = s / r[k][k];
//			if(j==n-1 && k==j-1){
//				std::cout << "s=" << s << std::endl;
//				std::cout << "r[" << k << "," << k <<"]=" << r[k][k] << std::endl;
//				std::cout << "r[" << k << "," << j <<"]=" << r[k][j] << std::endl;
//			}
		}
		timedots=timedots+(double)(gettime()-tstart)/1000000.0;
		tstart = gettime();
		for (int k=0; k <= j-1; ++k) {  // since both 0 and j-1 are shifted.
			for(int i=0; i < m; i++) {
				a[i][j] = a[i][j] - r[k][j]*a[i][k];
//				rational u = r[k][j]*a[i][k];
//				longint ud = u.denominator();
//				longint un = u.numerator();
//				ndgtu = ndgtu + numdgts(ud) + numdgts(un);
//				a[i][j] = a[i][j] - u;
			}
		}
		timeaxpy=timeaxpy+(double)(gettime()-tstart)/1000000.0;
		rational s = rational::ZERO;
		for(int i=0; i < m; i++) {
			s += a[i][j]*a[i][j];
		}
		r[j][j] = s;
//	std::cout << "CGS sum of sd+sn =" << ndgts << std::endl;
//	std::cout << "CGS sum of td+tn =" << ndgtt << std::endl;
//	std::cout << "CGS sum of ud+un =" << ndgtu << std::endl;
	}
	std::cout << "CGS time for Dots=" << timedots << std::endl;
	std::cout << "CGS time for Axpy=" << timeaxpy << std::endl;
}

void CGSsc(rat::matrix<rational>& a, const int m, const int n, rat::matrix<rational>& r, rat::vector<rational>& sa)
{
	double timescal=0;
	double timedots=0;
	double timeaxpy=0;
	long long tstart;
	rat::matrix<rational> rs(n,n);    /*       rs(n,n) is needed because   AAA     */
	for (int i=0; i < n; ++i) { sa[i]=1; }   /*   sa  initialize             */

	for (int j=0; j < n; ++j) {
		tstart = gettime();
		for (int k=0; k <= j-1; ++k) {  // since both 0 and j-1 are shifted.
			rational s = rational::ZERO;
			for(int i=0; i < m; i++) {
				s += a[i][k]*a[i][j];
			}
			r[k][j] = (s * sa[k]) / r[k][k];
			rs[k][j] = r[k][j] * sa[k];
		}
		timedots=timedots+(double)(gettime()-tstart)/1000000.0;
		tstart = gettime();
		for (int k=0; k <= j-1; ++k) {
			for(int i=0; i < m; i++) {
//				a[i][j] = a[i][j] - r[k][j]*a[i][k];
//				a[i][j] = a[i][j] - r[k][j]*a[i][k]*sa[k];  /*   AAA   */
				a[i][j] = a[i][j] - rs[k][j]*a[i][k];       /*   AAA   */
			}
		}
		timeaxpy=timeaxpy+(double)(gettime()-tstart)/1000000.0;

		sa[j] = ScalMatCol(a,m,n,j,1);    /*   Scale p_j, i.e.,  a[0:n-1][j] is scaled  */

		rational s = rational::ZERO;
		for(int i=0; i < m; i++) {
			s += a[i][j]*a[i][j];
		}
//		r[j][j] = s;
		r[j][j] = s * sa[j] * sa[j];        /*   since  p_j is scaled  */
	}
	std::cout << "CGSsc time for Dots=" << timedots << std::endl;
	std::cout << "CGSsc time for Axpy=" << timeaxpy << std::endl;
	std::cout << "CGSsc time for ScalVec=" << timescal << std::endl;
}

void bwdgssbst(const rat::matrix<rational>& r, const int n, rat::vector<rational>& z)
{
	for (int i=0; i < n; ++i) {
		z[i] = z[i] / r[i][i];
//		std::cout << z[i]  << std::endl;
	}
	for (int i=n-1; i >= 0; --i) {
		for (int k=0; k <= i-1; ++k) {
			z[k] = z[k] - r[k][i]*z[i];
		}
	}
}
