/*
 *   g++ LU.cpp longint.o rational.o ratutil.o thread_manager.o mempool.o gettimeofday.o
 *
 *   ./a.exe
 *   6 5 4
 *
	  m = n = r   regular (non singular) matrix
	  m = n > r   singular matrix of rank r
	  m > n > r   non singular but pivoting occurs at n
 *
 * */

#include <iostream>    // setprecision
#include <iomanip>     // setprecision
#include <stdexcept>
#include "rational.h"
#include "rvector.h"
#include "rmatrix.h"
#include "ratutil.h"
#include "rblas.h"
#include "prblas.h"

long long gettime();

int main() 
{
	int m,n,r,rank;
	long long iseed=1;
	long long tstart = gettime();

	std::cout << "enter m n r: "; std::flush(std::cout);
	std::cin >> m ;
	std::cin >> n ;
	std::cin >> r ;
	std::cout <<  "m=" << m << " n=" << n << "  r=" << r << std::endl;

	rat::matrix<rational> a(m,m);
	rat::vector<rational> b(m), x(m);
	rat::vector<int> ipvt(m), jpvt(m);
				//  Generate m x m Matrix
	GenMat(a,m,n,r,iseed);	//  its n x n leading submatrix is rank r
				//  m = n = r   regular (non singular) matrix
	matprn(a,m,m);		//  m = n > r   singular matrix of rank r
				//  m > n > r   non singular but pivoting occurs at n
	for (int i=0; i < m; ++i) { b[i] = 1; x[i] = 0; }
//	rblas::rgemv(a,b,x,m,m);  // rhs is in x
//	rat::rgemv(rational::ONE,a,b,rational::ZERO,x);
	rat::prgemv(rational::ONE,a,b,rational::ZERO,x);

	std::cout << "b=" << b << std::endl;
	std::cout << "x=" << x << std::endl;

	tstart = gettime();
	LUdecomp(a,m,ipvt,jpvt,rank);
	std::cout << "LUdecomp time=" << (double)(gettime()-tstart)/1000000.0 << "(sec)" << std::endl;
	std::cout << "LUdecomp rank=" << rank <<  std::endl;
	/*****************************************************************
	rational p=a[n-1][n-1];
	for(int k=1; k<7; k++) p *= rational::MILLION;
	std::cout << " p=" << p << std::endl;
	std::cout << " (double)p=" << (double)p << std::endl;
	std::cout << " (double)p=" << std::setprecision(16) << (double)p << std::endl;
	std::cout << p.format(42,12) << std::endl;
	std::cout << p.format(41,11) << std::endl;
	std::cout << p.format(40,10) << std::endl;
	*****************************************************************/
//	matprn(a,n,n);
	if(rank < m){
		std::cout << "LU stop since rank is less than m" <<  std::endl;
        } else {
		tstart = gettime();
		LUsubst(a,m,x,ipvt,jpvt);
		std::cout << "LDLsubst time=" << (double)(gettime()-tstart)/1000000.0 << "(sec)" << std::endl;
		std::cout << "x=" << x << std::endl;

		int bad = 0;
		for (int i=0; i < m; ++i) {
		if(x[i]-b[i] != rational::ZERO) bad=i+1;
		}
		if(bad == 0){
			std::cout << "result OK" << std::endl;
		} else {
			std::cout << "result bad i=" << bad << std::endl;
		}
	}
}

