/******************************************************************************
 
g++ -O3 dvsrch.cpp longint.o mempool.o rational.o  rough.o interval.o
                   numeric_cast.o thread_manager.o ratutil.o gettimeofday.o

 ./a.{out,exe} {2,3,4,5,6,7,8,9,10,11,12} {0,1,2}
                 
----------------------------------------------------------------------------------------------------------------------
 ./a.exe  6 0 | grep % 
     show factorization of the characteristic polynomial associated each Frobenius submatrix.

 ./a.exe 8 0 | tee piyo8
 grep OK piyo8
 grep submatrix piyo8       shows H_x, H_y, H_z, .....

******************************************************************************/
#include <iostream>
#include <fstream>  
#include <iomanip>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <algorithm>
#include <stdexcept>
#include "rational.h"
#include "ratutil.h"    // 2015oct19
#include "rblas.h"
#include "rvector.h"
#include "rmatrix.h"
#include <sstream>

using namespace rat;

void eigvals(int);

void selectev(const vector<double>& a, int n, int& ngrp, vector<double>& eigvl, vector<double>& eigvh, rat::vector<int>& mult);
void nmhes(rat::matrix<rational>& a, int n, rat::vector<int>& ith, int& nmat);
void cpsubmat(rat::matrix<rational>& a, int n, rat::matrix<rational>& b, int i1, int i2);
int setecand(const vector<double>& eigvl, const int ncan, const int included[], double evcan[], int usev[]);
int chkonce(const int n, const rat::vector<rational>& p, const rational a, const rational b, rational& fa, rational& fb);
int chkinc(const int n, const rat::vector<rational>& p, const double e, const double h, const int mult);

const double eps=1e-9;
int chkprt;


#ifdef EPROF2
	long long longint::ldividetime;
	long longint::ldividecount;
	long long longint::lgcdtime;
	long longint::lgcdcount;
	long long longint::lgcd2time;
	long longint::lgcd2count;
	long long longint::opsubeqtime;
	long longint::opsubeqcount;
	long long longint::shrtime;
	long longint::shrcount;
	long long longint::lmultime;
	long longint::lmulcount;
	long long longint::laddtime;
	long longint::laddcount;
#endif

/*********************************************************************/
/*********************************************************************/
/*********************************************************************/
int main(int argc, char *argv[])
{
	long long tstart0=gettime();
	int m = atoi(argv[1]);
	chkprt = atoi(argv[2]);
	std::cout << " m=" << m << " chkprt=" << chkprt << std::endl;

	eigvals(m);

	std::cout << "dvsrch finish. total time=" << (double)(gettime()-tstart0)/1000000.0 << "(sec)" << std::endl;
	return 0;
}

void eigvals(int m){
// *******   Declare scalar variables *********************************************
	int i, j, k, n, i1, i2, itr[2], nexist;
	int ngrp, nmat, nsize, ngrp1, ncandi, d, noff, reset;
	double u, epsint=1e-2;
	long long nn, ii, tstart0;  /* nn used for number of combination  */
// *******   determine matrix size ***********************************
	n = m*m;
// *******   Declare array type variables *********************************************
	int usev[n+1], included[n+1], idx[n+1];
	double evcan[n+1], cof[n], figvl[n+1];
	rat::matrix<double> a(n,n), ev(n,n);
	vector<double> v(n+1), est(n+1), eigvl(n+1), eigvh(n+1);
	rat::vector<int> ith(n), mult(n+1);
	rat::matrix<rational> ar(n+1,n+1),b(n+1,n+1);
	rat::vector<rational> p(n+1), p0(n+1), p1(n+1), q(n+1);
	rat::vector<rational> urat(n+1), vrat(n+1), qrat(n+1), rrat(n+1);
	// ********* get, put polynomials   *********************************/
	int npolynomial=0;                /* number of polynomials found  */
	rat::vector<int> ppolynomial(n+2);     /* pointer to  polynomials      */
	rat::vector<rational> cpolynomial(n*2); /* coefficients of polynomials  */
	ppolynomial[0]=0;
	// *******   Matrix generation  *********************************************
	SetFtherm(a,m);        // Thermal coduction (rational.cpp)
	if(chkprt>0) std::cout << n << "x" << n << " A matrix" << std::endl;
	cnvmat(a,ar,n,n);      // convert double precision A to rational (rational.cpp)
	if(chkprt>0) matprn(ar,n,n);        // matrix print (rational.cpp)
	// *******   Transform A to Hessenberg form *****************************************
	tstart0=gettime();          /*********************************/
	elmhes(ar,n);               /***   ELMHES  *******************/
	std::cout << std::endl;     /*********************************/
	std::cout << " ELMHES complete, time=" << (double)(gettime()-tstart0)/1000000.0 << "(sec)" << std::endl << std::endl;
	if(chkprt>1) matprn(ar,n,n);
	//***********  double precision computation for eigenvalues ***********
	int maxit=n*4; double tol=1e-10;
	int irtn=jacobev(a,ev,tol,n,maxit);  // compute all eigenvalues and eigenvectors in double precision
	if(irtn<0){
		std::cout << " jacobev NOT converged stop" << std::endl;
		exit(3);
	}
	//**************** sort (bubble sort) eigenvalues *********************
	for (i=0; i<n; i++) v[i] = a[i][i];
	bsort(v,n);   //  bubble sort 
	//************************************************
	std::cout << n << " eigenvalues obtained by using double precision floating-point arithmetic" << std::endl;
	for (i=0; i<n; i++){
		est[i] = v[i];
		std::cout << i+1 << "-th estimate eigval = " << std::setprecision(16) << est[i] << std::endl;
	}
	//********************************************************************
	selectev(est,n,ngrp,eigvl,eigvh,mult);  //  grouping eigenvalues 
	//********************************************************************
	ngrp1=ngrp+1;
	std::cout << ngrp1 << " eigenvalues selected" << std::endl << "  group          eivenvalue "<< std::endl;
	for (i=1; i<=ngrp1; i++) std::cout << "      "  << i << "   " << eigvl[i]  << "   " << eigvh[i] << std::endl;
	std::cout << std::endl;
	//******  if double root exists, diagonal term becomes zero *************
	nmhes(ar,n,ith,nmat);	// count number of submatrices of Hessenberg form
	//***********************************************************************
	std::cout << "Hessenberg matrix size of " << n << " is divided into " << nmat+1 << " submatrices" << std::endl;
	i1=1;
	for (i=0; i<=nmat; i++){
		std::cout << i+1 << "-th submatrix = " << i1 << " to " << ith[i]+1 << std::endl;
		i1=ith[i]+2;
	}
	std::cout << std::endl;
	std::cout << "Characteristic polynomials for " << nmat+1 << " submatrices" << std::endl;
	//********************************************************************/
	/******* Loop over each Hessenberg submatrix  ************************/
	//********************************************************************/
	for (k=nmat; k>=0; k--){
		i2=ith[k];
		std::cout << std::endl;
		if(k>0) i1=ith[k-1]+1;  else  i1=0; 
		nsize = i2-i1+1;
		std::cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%"  << std::endl;
		std::cout << "Sub Matrix " << k+1 << " size=" << nsize << " i1=" << i1+1 << " i2=" << i2+1 << std::endl;
		std::cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%"  << std::endl;
		//********************************************************************************
		cpsubmat(ar,n,b,i1,i2);      // ----- copy submatrix from ar to b 
		//********************************************************************************
		if(chkprt>1) matprn(b,nsize,nsize);
		tstart0=gettime();
		//******************************************************************************
		hesfrb(b,nsize,p);  // compute characteristic polynomial of Hessenberg submatrix
		//******************************************************************************
		std::cout << std::endl;
		std::cout << "%HESFRB complete, time=" << (double)(gettime()-tstart0)/1000000.0 << "(sec)" << std::endl;
		std::cout << std::endl;
		std::cout << "%Characteristic polynomial = " ; polytexformint(p,nsize);
		//**************************************************************************************/
		/* (1)  divide by the known (found in the previous loop iterations) divisors  **********/
		//**************************************************************************************/
		if(chkprt>0) std::cout << "divide by the known (found in previous iterations) divisors=" << npolynomial << std::endl;
		for(int iod=1; iod<=npolynomial; iod++){
			int ksz=getpolynomial(iod,vrat,npolynomial,ppolynomial,cpolynomial);
			if(chkprt>0) polytexformr(vrat,ksz);
			for (i=0; i<=nsize; i++)  urat[nsize-i] =p[i];
			if(polydivchk(urat,nsize,vrat,ksz,qrat,rrat)){
				for (i=0; i<=nsize-ksz; i++) p[nsize-ksz-i]=qrat[i];   //  replace P by Q 
				nsize = nsize-ksz; 
				std::cout << "Divided up by the known divisor" << std::endl; 
				polytexformintr(vrat,ksz);
				if(chkprt>0) std::cout << " Replaced characteristic polynomial new polynomial follows;" << std::endl;
				if(chkprt>0) polytexform(p,nsize);
				npolynomial=putpolynomial(ksz,vrat,npolynomial,ppolynomial,cpolynomial);
			}
		}
		std::cout << "divide by the known polynomials end" << std::endl << std::endl;
		//********************************************************************/
		if(nsize <= 0) continue;  
		//********************************************************************/
		//*********** (2)  inclusion ******************************************
		//********************************************************************/
		nexist=0;
		for(i=1; i<=ngrp1;i++){
			included[i] = chkinc(nsize,p,eigvl[i],eigvh[i],mult[i]);
			nexist = nexist + included[i];
		}
		std::cout << std::endl << "found " << nexist << " intervals.  nsize=" << nsize << std::endl;
		if(nsize == nexist){
			if(chkprt>0) std::cout << " in each of intervals one root exists." << std::endl;
			for(i=1; i<=ngrp1;i++){
				if(included[i]){
					if(chkprt>0) std::cout << " group=" << i << " used[" << i << "]=0"  << " for candidate, eigvl=" << eigvl[i] << std::endl;
				}
			}
		}else{
			std::cout << "ERROR nsize != nexist  nsize=" << nsize << " nexist=" << nexist << "  STOP" << std::endl;
			exit(3);
		}
		//********************************************************************/
		/*************** (3)  divide by the first order polynomials **********/
		//********************************************************************/
		std::cout << "%divide by the first order polynomial ngrp1=" << ngrp1 << std::endl;
		for(i=1; i<=ngrp1;i++){
			if(included[i]){
				if(chkprt>0) std::cout << "check first order polynomial eigvl[" << i << "]=" << eigvl[i] << std::endl;
				if(askinty(eigvl[i],epsint,vrat[0])){
					if(chkprt>0) std::cout << "AskInty return val=1  for eigvl[i]=" << eigvl[i] << std::endl;
					for (j=0; j<=nsize; j++) urat[nsize-j]=p[j];
					vrat[1]=-rational::ONE;
					if(polydivchk(urat,nsize,vrat,1,qrat,rrat)){	// trial division by polydivchk (ratutil)
						std::cout << " Divided up by the first order polynomial = " << std::endl;
						polytexformintr(vrat,1);
						included[i]=0; nsize --;
						for (j=0; j<=nsize; j++) p[nsize-j]=qrat[j];   //  replace P by Q 
						if(chkprt>0) std::cout << " Replaced characteristic polynomial new polynomial as;" << std::endl;
						if(chkprt>0) polytexform(p,nsize);
						npolynomial=putpolynomial(1,vrat,npolynomial,ppolynomial,cpolynomial);
					}  // if(polydivchk)
				}         //  if(askinty)
			}                //   if(included[i])
		}	                //    for(i=1; i<=ngrp1;i++){
		if(nsize <= 1){
			if(nsize>0){
				for (i=0; i<=nsize; i++)  urat[nsize-i] =p[i];
				npolynomial=putpolynomial(nsize,urat,npolynomial,ppolynomial,cpolynomial);
				if(chkprt>0) std::cout << "added p() and replace npolynomial=" << npolynomial << std::endl;
			}
			continue;
		}
		/*************************************************************************/
		/************** (4) compose divisor candidates and make trial division ***/
		/*************************************************************************/
		std::cout << "Vieta's formula check  ngrp1=" << ngrp1 <<  std::endl;
		for(i=1; i<=ngrp1;i++){
			if(included[i] >= 1) std::cout << "included[" << i << "]=" << included[i] << " eigvl[i]=" << eigvl[i] << std::endl;
		}
		d=2; noff=0;
		while(d<=nsize){                  // search for polynomial of degree 2 to nsize
			ncandi=setecand(eigvl,ngrp1,included,evcan,usev); // set number of candidate eigenvalues
			for(i=1; i<d;i++) idx[i]=i+noff; idx[d]=d+noff-1; // initialize for nxtcmb
			nn=comb(ncandi,d); reset=0;       // since nn is type "long long", ngrp1-nuse should be "< 62"
			std::cout <<  std::endl  << "comb(" << ncandi << "," << d << ")=" << nn << std::endl << std::endl;
			if(chkprt>0) std::cout << "Vieta's formula check for d=" << d << " ngrp1=" << ngrp1 << " ncandi=" << ncandi << std::endl;
			for(ii=1; ii<=nn;ii++){               // loop over nn combinations of candidates
				nxtcmb(ncandi,d,idx);         // select one of _(ncan-nused) C_d candidates
				u=0;
				for(j=1; j<=d; j++){ figvl[j]=evcan[idx[j]]; u=u+figvl[j];}
				if(askinty(u,epsint,vrat[0])){  // pass the first cofficient check, then ...
					int ok=1;
					for(int r=1; r<=d; r++){
						cof[r]=vietaterm(figvl,d,r); // make coefficieants terms of Vieta's formula from figvr
						if(! askinty(cof[r],epsint,vrat[d-r])){ ok=0; break; }
					}
					if(ok){        // OK, try trial division
						std::cout << " OK polynomial of degree " << d << " all coefficients seem to be integer" << std::endl;
						if(chkprt>0) for(int r=1; r<=d; r++) std::cout << " cof[" << r << "]=" << cof[r];
						if(chkprt>0) std::cout << std::endl;
						vrat[d]=rational::ONE;
						if(chkprt>0) std::cout << " Trial division"  << std::endl;
						for (j=0; j<=nsize; j++) urat[nsize-j]=p[j];
						if(chkprt>0) polytexformintr(urat,nsize);
						int isit=polydivchk(urat,nsize,vrat,d,qrat,rrat);  // trial division
						std::cout << " Polydivchk rtnval=" << isit << std::endl;
						if(isit==0) std::cout << std::endl;
						if(isit){;                            // trial division divided up
							for (j=0; j<=nsize-d; j++) p[nsize-d-j]=qrat[j]; // set quatient for next p(\lambda)
							nsize = nsize-d;                                 // and decrease nsize
							std::cout << " ii=" << ii << " Divisor" << std::endl;
							polytexformintr(vrat,d);
							for (j=1; j<=d; j++) std::cout << " group is " << usev[idx[j]] << ", eigval=" << figvl[j] << std::endl; 
							npolynomial=putpolynomial(d,vrat,npolynomial,ppolynomial,cpolynomial);
							if(chkprt>0) std::cout << "replaced. npolynomial=" << npolynomial << std::endl << std::endl;
							for (j=1; j<=d; j++) included[usev[idx[j]]]=0;
							noff=idx[1]-1;  // number of OFFed indecies
							if(d+noff>ncandi) break;         // there are no degree d left, then break for loop
							reset=1; break;
						}    // if(divup)
					}           //  if(ok){
				}	           //   if(askinty(u,epsint,vrat[0]))
			}                         //    for(ii=1; ii<=nn;ii++){
			if(reset==0){ d++; noff=0;}
		}                               //      while(d<nsize){
// Loopend:
		if(nsize>0){
			for (i=0; i<=nsize; i++)  urat[nsize-i] =p[i];
			npolynomial=putpolynomial(nsize,urat,npolynomial,ppolynomial,cpolynomial);
			if(chkprt>0) std::cout << "added p() and replace npolynomial=" << npolynomial << std::endl;
		}
		std::cout << std::endl;
		std::cout << "END of k =" << k << std::endl;
		if(chkprt>0) std::cout << "PRINT current divisors found npolynomial=" << npolynomial << std::endl;
		for(int iod=1; iod<=npolynomial; iod++){
			int ksz=getpolynomial(iod,vrat,npolynomial,ppolynomial,cpolynomial);
			if(k==0){
				polytexformr2(vrat,ksz,rational::ONE);  //  add % 
			}else{
				if(chkprt>0) polytexformintr(vrat,ksz);
			}
		}
		if(chkprt>0) std::cout << "PRINT current divisors found END" << std::endl;
	}         //                                    for(k=nmat; k>=0; k--){
}

int setecand(const vector<double>& eigvl, const int ncan, const int included[], double evcan[], int usev[]){
	int j=0;
	for(int i=1; i<=ncan;i++){     
		if(included[i]){                           // exclude eigvl[i] according to included[i]
			j++; usev[j]=i; evcan[j]=eigvl[i];
		};
	}
	return j;        // return number of candidates set in array evcan[1] to [j]
}

void selectev(const vector<double>& a, int n, int& ngrp, vector<double>& eigvl, vector<double>& eigvh, rat::vector<int>& mult)
{
	int i,ii,j=0;
	double x,am;
	am=fabs((a[n-1]-a[0])/((double)n));  /* abs(mean) */
	std::cout << "selectev Amean=" << am  << std::endl;
	eigvl[j+1]=a[0]; eigvh[j+1]=0; x=a[0]/am; ii=0; mult[j+1]=0;
	for(i=1; i < n; i++){
		if(fabs(a[i]/am-x) > eps){
			j++; x = a[i]/am; eigvl[j+1]=a[i]; ii=i; mult[j+1]=0;
		}
		if(ii != i){
			eigvh[j+1]=a[i]; /* higher eigval of jth group */
			mult[j+1]++;
		}
	}
	ngrp = j;
}

void nmhes(rat::matrix<rational>& a, int n, rat::vector<int>& ith, int& nmat)
{
	rational x, y;
	int j=1;
	std::cout << "nmhes  start n=" << n  << std::endl;
	for (int i=2; i <= n; ++i) {
		if (a[i-1][i-2] == rational::ZERO) {
			ith[j-1] = i-2;
			j++;
		}
	}
	ith[j-1] = n-1;
	nmat = j-1;
}

void cpsubmat(rat::matrix<rational>& a, int n, rat::matrix<rational>& b, int i1, int i2)
{
	int nsize = i2-i1+1;
	for (int j=0; j < nsize; ++j) {
		for (int i=0; i < nsize; ++i) b[i][j] = a[i+i1][j+i1];
	}
}

int chkonce(const int n, const rat::vector<rational>& p, const rational a, const rational b, rational& fa, rational& fb)
{
	rational fda,fdb,fdda,fddb;
	int rtnval=1;
	Horner3(n,p,a,fa,fda,fdda);
	Horner3(n,p,b,fb,fdb,fddb);
	if(fa*fb > rational::ZERO){
		rtnval=0;
	}
	return rtnval;
}

int chkinc(const int n, const rat::vector<rational>& p, const double e, const double h, const int mult)
{
	double ta,tb; 
	int rtnval=0;
	rational a,b,fa,fb;

	if(fabs(e) > eps){
		ta=e-fabs(e)*eps; tb=h+fabs(h)*eps;
		if(mult==0) tb=e+fabs(e)*eps;
	}else{
		ta=e-eps; tb=h+eps;   // consider eigenvalues close to zero
		if(mult==0) tb=e+eps;
	}
	a = Rdset(&ta); fa=Horner(n,p,a);
	b = Rdset(&tb); fb=Horner(n,p,b);
	if(fa*fb<rational::ZERO) rtnval=1;
	return rtnval;
}
