/******************************************************************************
   Symmetric Eigenvalue Hessenberg Frobenius Vieta version 2
 
g++ -O3 dvsrch1.cpp longint.o mempool.o rational.o  rough.o interval.o
                   numeric_cast.o thread_manager.o ratutil.o gettimeofday.o

 ./a.{out,exe} {T,t,s} {2,3,4,5,6,7,8,9,10,11,12}
		 T thermal    2-12     n=m*m thermal conductivity matrix
		 t thermal    3-12     n=m*m thermal conductivity matrix, thermal conductivity = -0.1                 *
		 s thermal    3-12     n=m*m thermal conductivity matrix, thermal conductivity = -0.1 in double float *

		 *  denotes eigenvalues are multiplied by LCM of denominators of A. ---------------------------------
                                                                                                                       
 ./a.{out,exe} {G} {3,4,5,6,7,8,9,10,11}

 G    3           4               5                   6                    7                   
 n    5          12              21                  32                   45                   
 ---------------------------------------------------------------------------------------------
 H_x  3->(1),1,1  7->(1,2),1,1,2 12->(1,2,4),1,1,1,2 18->(1,2,2,7),1,2,3  25->(1,2,5,10),1,3,3
 H_y  1           2->2            6->2,4              7->7                15->5,10            
 H_z  1           1->(1)          1->(1)              4->2,2               1->(1)             
                  1->(1)          1->(1)              1->(1)               1->(1)             
                  1->1            1->1                1->(1)               2->2               
                                                      1->1                 1->1              
---------------------------------------------------------------------------------------------

 G    8                        9                    10                    11  
 n   60                       77                    96                   117
 ---------------------------------------------------------------------------------------------------------------------
 H_x 33->(1,2,3,5,14),1,1,3,3 42->(1,5,9,18),1,4,4  52->(1,9,9,23),1,4,5  63->(1,9,14,28),1,1,4,5
 H_y 14->14                   32->5,9,18            23->23                42->14,28 
 H_z 10->2,3,5                 1                    18->9,9                9
      1->(1)                   1                     1                     1
      1->(1)                   1                     1                     1
      1->1                                           1                     1
--------------------------------------------------------------------------------------------------------------------
 ./a.exe T 6 | grep % 
     show factorization of the characteristic polynomial associated each Frobenius submatrix.

 ./a.exe t 8 | tee piyot8
 grep OK piyot8

 ./a.exe s 8 | tee piyos8
 grep OK piyos8
 grep submatrix piyos8       shows H_x, H_y, H_z, .....

 s    2  3  4  5  6   7   8   9  10  11  12  13
 -----------------------------------------------
 H_x  3  5  9 13 19  25  33  41  51  55  73  85
 H_y  1  3  3  9 13  19  25  33  41  37  61  73
 H_z     1  3  1  1   1   1   1   1  15   1   1
            1  1  1   1   1   1   1   7   1   1
               1  1   1   1   1   1   1   1   1
                  1 2*1 3*1 4*1 5*1 6*1 7*1 8*1

 T,t,s=13 encounterd: Error stop since _72 C xx  > long long integer limit in comb()

******************************************************************************/
#define INTYDOUBLE
//#define FACTORIZEONLY
#define VIETAZ
//#define VIETACOUNT
// ******************************************************************************
#include <iostream>
#include <fstream>  
#include <iomanip>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <algorithm>
#include <stdexcept>
#include "rational.h"
#include "rblas.h"
#include "ratutil.h"
#include "rvector.h"
#include "rmatrix.h"
#include <sstream>

using namespace rat;

void eigvals(char, int);              //dvsrch1:    add the first argument

void selectev(const vector<double>& a, int n, int& ngrp, vector<double>& eigvl, vector<double>& eigvh, rat::vector<int>& mult);
void nmhes(rat::matrix<rational>& a, int n, rat::vector<int>& ith, int& nmat);
void cpsubmat(rat::matrix<rational>& a, int n, rat::matrix<rational>& b, int i1, int i2);
int chkonce(const int n, const rat::vector<rational>& p, const rational a, const rational b, rational& fa, rational& fb);
int chkinc(const int n, const rat::vector<rational>& p, const double e, const double h, const int mult, rational& a, rational& b);

int setecanint(const rat::vector<rational>& eigvr, const vector<interval>& eab, const int ncan, const int included[], rat::vector<rational>& ercan,  rat::vector<interval>& ercanint, vector<double>& dfcand, int usev[]); 
#ifdef VIETAZ
int vietatermschk(const vector<interval>& e, const int n, rat::vector<rational>& cor);
int mulvieta(rat::vector<int>& num,const interval& sp,const int n,const int r,const vector<interval>& e, vector<interval>& p,rat::vector<int>& last,int dir); 
#else
int vietaterm(const vector<interval>& e, const int n, const int r, rational& ta, rational& tb);
#endif

void RatFtherm(rat::matrix<rational>& a, int m);
void genDtherm(rat::matrix<double>& a, int m);

const double eps=1e-9;
int chkprt;
#ifdef VIETACOUNT
long long vietamul;
int vietamultime;
#endif
// *******************************************************************
// *******************************************************************
int main(int argc, char *argv[]) 
{
#ifdef  INTYDOUBLE
	std::cout << " INTYDOUBLE,";
#endif
#ifdef  FACTORIZEONLY
	std::cout << " FACTORIZEONLY,";
#endif
	long long tstart0=gettime();
	char cmat = *argv[1];
	int m = atoi(argv[2]);
	chkprt = atoi(argv[3]);
	std::cout << " cmat=" << cmat << " m=" << m << " chkprt=" << chkprt << std::endl;

	eigvals(cmat, m);

	std::cout << "%dvsrch1 finish. total time=" << (double)(gettime()-tstart0)/1000000.0 << "(sec)" << std::endl;
	return 0;
}

void eigvals(char cmat, int m){   //dvsrch1:
// *******   Declare scalar variables *********************************************
	int i, j, k, n, i1, i2, nexist, isit, rank;
	int ngrp, nmat, nsize, ngrp1, ncandi, d, noff, reset;
	double epsint=1e-5, eveps, accdouble=45, log2ev;
	long long nn, ii, tstart0;               // nn used for number of combination 
	double dlcm=1;
	rational ur, Clcm=rational::ONE, faeig, fbeig, ceig;//dvsrch1:
// *******   determine matrix size **************************************************
	if(cmat == 'G'){
		n=(m-2)*(m-2)+4*(m-2);
	}else{
		n = m*m;
	}
// *******   Declare array type variables *********************************************
	int usev[n+1], included[n+1], idx[n+1];
	int ind[n+1];
	rat::vector<rational>  ercan(n+1), cor(n+1);
	vector<interval> figv(n+1), eigint(n+1), ercanint(n+1);
	vector<double> dfcand(n+1), figdaf(n+1);
	rat::matrix<double> a(n,n), ev(n,n);
	vector<double> v(n+1), est(n+1), eigvl(n+1), eigvh(n+1);
	rat::vector<int> ith(n), mult(n+1);
	rat::matrix<rational> ar(n+1,n+1),b(n+1,n+1),as(n+1,n+1),bs(n+1,n+1),eigenvector(n+1,n+1);
	rat::vector<rational> p(n+1), urat(n+1), vrat(n+1), qrat(n+1), rrat(n+1);
	rat::vector<rational> eigvr(n+1), eiglow(n+1), eigup(n+1);
	rat::matrix<rational> h(n+1,n+1);
	// ******* get, put polynomials   ************************************
	int npolynomial=0;                      // number of polynomials found  
	rat::vector<int> ppolynomial(n+2);      // pointer to  polynomials 
	rat::vector<rational> cpolynomial(n*2); // coefficients of polynomials
	// *******************************************************************
	ppolynomial[0]=0;
	// *******   Matrix generation  **************************************
	if(cmat == 'T'){
		SetFtherm(a,m);        // Thermal coduction (rational.cpp)
	}else if(cmat == 't'){
		genDtherm(a,m);        // Thermal coduction /10,  D means double precision.....
		RatFtherm(h,m);        // Thermal coduction in Rational 0.4, i.e., without rounding error
		matprn(h,n,n);
		Clcm=rat2int(h,n,ar);  // multiply H by LCM of denominators to make integer matrix Ar
		dlcm=Clcm;
		std::cout << "%LCM of denominators of Thermal conduction matrix=" << Clcm  << std::endl;
	}else if(cmat == 's'){
		genDtherm(a,m);        // Thermal coduction in double floating-point diag=0.4
		cnvmat(a,h,n,n);       // h(n,n)  covert to rational matrix H, i.e., with rounding error
		matprn(h,n,n);
		Clcm=rat2int(h,n,ar);  // multiply H by LCM of denominators to make integer matrix Ar
		dlcm=Clcm;
		std::cout << "%LCM of denominators of double Ftherm matrix=" << Clcm  << " dlcm=" << dlcm << std::endl;
		longint mgcd=matngcd(ar,n);
		std::cout << "%GCD of numerators of integer matrix=" << mgcd  << std::endl;
	}else if(cmat == 'G'){
		SetGLmat(a,m);
	}
	if(cmat != 't' && cmat != 's'){
		cnvmat(a,ar,n,n);    // ar(n,n) <-- rational matrix
		std::cout << " rational Ar matrix converted from double a" << std::endl;
	}
	copymat(ar,n,n,as);    // matrix save in as for eigenvector computation
	std::cout << n << "x" << n << " A matrix" << std::endl;
	if(chkprt>1) matprn(ar,n,n);
	// *******   Transform A to Hessenberg form *****************************************
	tstart0=gettime();          // ******************************************************
	elmhes(ar,n);               // *******   ELMHES  ************************************
	std::cout << std::endl;     // ******************************************************
	std::cout << " ELMHES complete, time=" << (double)(gettime()-tstart0)/1000000.0 << "(sec)" << std::endl << std::endl;
	if(chkprt>1) matprn(ar,n,n);
	//*********************************************************************
	int maxit=n*4; double tol=1e-10;
	int irtn=jacobev(a,ev,tol,n,maxit);  // compute all eigenvalues and eigenvectors in double precision
	if(irtn<0){
		std::cout << " jacobev NOT converged stop" << std::endl;
		exit(3);
	}
	//*********************************************************************
	//  eigenvalues are multiplied by LCM of denominators of A.
	for (i=0; i<n; i++){     
		if(dlcm > 0){                   //  m * A * x = m * \lambda * x 
			v[i] = a[i][i] * dlcm;
		} else{
			v[i] = a[i][i];
		}
	}
	//************************************************
	bsort(v,n,ind);   //  sort (bubble sort) eigenvalues, ind includes order of sort
	//************************************************
	std::cout << n << " eigenvalues obtained by using double precision floating-point arithmetic" << std::endl;
	for (i=0; i<n; i++){
		est[i] = v[i];
		std::cout << std::endl << i+1 << "-th estimate eigval = " << std::setprecision(16) << est[i] << std::endl;
		for (j=0; j<n; j++){  
			double scal = 1.0;
			if((j+1)%8==0){
				printf(" %10.6f\n", scal * ev[j][ind[i]]);
			}else{
				printf(" %10.6f", scal * ev[j][ind[i]]);
			}
		}
		std::cout << std::endl << "Eigenvectors orthogonality" << std::endl;;
		for (j=i+1; j<n; j++){  
			double ss=0;
			for (int k=0; k<n; k++) ss+=ev[k][ind[i]]*ev[k][ind[j]];
			std::cout << " " << ss;
		}
		std::cout << std::endl;
	}
	std::cout << std::endl;
	//********************************************************************
	if(cmat == 'T' && m==3){
		std::cout << "Eigenvector check" << std::endl;;
		rat::matrix<rational> aar(n+1,n+1),evr(n+1,n+1);
		rat::vector<rational> evrat(n+1), rrrat(n+1);
		copymat(as,n,n,aar);    // matrix copy for eigenvector error analysis
		if(chkprt>1) matprn(aar,n,n);
		for (j=0; j<n; j++) aar[j][j]-=rational::FOUR*Clcm; 
		std::cout << "Eigenvector check for eigenvalue=4" << std::endl;;
		if(chkprt>1) matprn(aar,n,n);
		for (i=3; i<6; i++){
			for (j=0; j<n; j++) evrat[j]=Rdset(&ev[j][ind[i]]); 
			rat::rgemv(rational::ONE,aar,evrat,rational::ZERO,rrrat);
			rational evnrm2=rat::rdot(rrrat,rrrat);
			std::cout << i << "-th evector nrm2=" << (double)evnrm2 << std::endl;;
		}
	}
	//********************************************************************
	//********************************************************************
	selectev(est,n,ngrp,eigvl,eigvh,mult);  //  grouping eigenvalues 
	//********************************************************************
	ngrp1=ngrp+1;
	std::cout << ngrp1 << " eigenvalues selected" << std::endl << "  group          eivenvalue "<< std::endl;
	int ismal=0;
	for (i=1; i<=ngrp1; i++){
		if(i==1){
			eveps=epsint;
		}else{
			if(fabs(eigvl[i]-eigvl[i-1]) < eveps){ 
				eveps = fabs(eigvl[i]-eigvl[i-1]); 
				ismal=i;
			}
		}
		std::cout << "      "  << i << "   " << eigvl[i]  << "   " << eigvh[i] << " possible multiplicity " << mult[i]+1 <<  std::endl;
	}
	if(epsint > eveps) epsint=eveps;
	std::cout << " Smallest eigenvalue gap="  << eveps << "  epsint=" << epsint << " @ i=" << ismal << std::endl;
	std::cout << std::endl;
	//*************************************************************************
	nmhes(ar,n,ith,nmat);	// count number of submatrices of Hessenberg form
	//*************************************************************************
	std::cout << "Hessenberg matrix size of " << n << " is divided into " << nmat+1 << " submatrices" << std::endl;
	i1=1;
	for (i=0; i<=nmat; i++){
		std::cout << i+1 << "-th submatrix = " << i1 << " to " << ith[i]+1 << std::endl;
		i1=ith[i]+2;
	}
	std::cout << std::endl;
	std::cout << "Eigenvalue computaion for " << nmat+1 << " submatrices" << std::endl;
	// *******************************************************************
	// ***** Loop over each Hessenberg submatrix  ************************
	// *******************************************************************
	for (k=nmat; k>=0; k--){
		i2=ith[k];
		std::cout << std::endl; std::cout << std::endl;
		if(k>0) i1=ith[k-1]+1;  else  i1=0; 
		nsize = i2-i1+1;
		std::cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%"  << std::endl;
		std::cout << "Sub Matrix  " << k+1 << " size =" << nsize << " i1=" << i1+1 << " i2=" << i2+1 << std::endl; 
		std::cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%"  << std::endl;
		//********************************************************************************
		cpsubmat(ar,n,b,i1,i2);      // ----- copy submatrix from ar to b 
//		copymat(b,nsize,nsize,as);   // matrix save in as for eigenvector computation
		// *******************************************************************************
		if(chkprt>1) matprn(b,nsize,nsize);
		tstart0=gettime();
		// *****************************************************************************
		hesfrb(b,nsize,p);  // compute characteristic polynomial of Hessenberg submatrix
		// *****************************************************************************
		std::cout << std::endl;
		std::cout << "%HESFRB complete, time=" << (double)(gettime()-tstart0)/1000000.0 << "(sec)" << std::endl;
		std::cout << std::endl;
		std::cout << " " << std::endl;
		std::cout << "%Characteristic polynomial = " ; polytexformint(p,nsize);
		// *************************************************************************************
		// (1)  divide by the known (found in the previous loop iterations) divisors  **********
		// *************************************************************************************
		if(chkprt>0) std::cout << "divide by the known (found in previous iterations) divisors=" << npolynomial << std::endl;
		for(int iod=1; iod<=npolynomial; iod++){
			int ksz=getpolynomial(iod,vrat,npolynomial,ppolynomial,cpolynomial);
			if(chkprt>0) polytexformr(vrat,ksz);
			for (i=0; i<=nsize; i++)  urat[nsize-i] =p[i];
			if(polydivchk(urat,nsize,vrat,ksz,qrat,rrat)){
				for (i=0; i<=nsize-ksz; i++) p[nsize-ksz-i]=qrat[i];   //  replace P by Q 
				nsize = nsize-ksz; 
				std::cout << "Divided up by the known divisor = "  << std::endl;
				polytexformintr(vrat,ksz);
				if(chkprt>0) std::cout << " Replaced characteristic polynomial new polynomial follows;" << std::endl;
				if(chkprt>0) polytexform(p,nsize);
				npolynomial=putpolynomial(ksz,vrat,npolynomial,ppolynomial,cpolynomial);
			}
		}
		std::cout << "divide by the known polynomials end" << std::endl << std::endl;
		// *******************************************************************
		// ********** (2)  inclusion *****************************************
		// *******************************************************************
		nexist=0;
		for(i=1; i<=ngrp1;i++){
		included[i] = chkinc(nsize,p,eigvl[i],eigvh[i],mult[i],eiglow[i],eigup[i]); //dvsrch1:
		nexist = nexist + included[i];
		eigint[i]=interval(eiglow[i],eigup[i]);
		}
		std::cout << std::endl << "found " << nexist << " intervals.  nsize=" << nsize << std::endl;
		if(nsize == nexist){
			if(chkprt>0) std::cout << " in each of intervals one root exists. For the first order factors," << std::endl;
			for(i=1; i<=ngrp1;i++){
				if(included[i] > 0){
					eigvr[i]=Rdset(&eigvl[i]);
					std::cout << std::endl << " included[" << i << "]=" << included[i] << " for candidate, eigvl=" << (double)eigvl[i] << std::endl;
					faeig=Horner(nsize,p,eiglow[i]); fbeig=Horner(nsize,p,eigup[i]);
					if(faeig*fbeig > rational::ZERO){
						std::cout << " ERROR check Horner .....faeig=" << faeig << " fbeig=" << fbeig << std::endl;
						polytexform(p,nsize);
						exit(3);
					}
					// ******** bisect to get interval of eigenvalue estimation *****************
					int irtn = bisect(p,nsize,epsint,100,eigint[i],faeig,fbeig,eigvr[i]);
					if(irtn < 0){
						std::cout << " bisect not converged STOP" << std::endl; exit(3);
					}
					if(irtn==0){
						irtn=100; // eigvr[i] is solution, i.e., eigenvalue is rational number
						std::cout << " bisect encountered rational number solution" << std::endl;
					}
					included[i] = abs(irtn);
					formatprn(eigint[i]);
				}
			}
		}else{
			std::cout << "ERROR nsize != nexist  nsize=" << nsize << " nexist=" << nexist << "  STOP" << std::endl;
			exit(3);
		}
		// *******************************************************************
		// ************* (3)  divide by the first order polynomials **********
		// *******************************************************************
		std::cout << std::endl << "%divide by the first order polynomial ngrp1=" << ngrp1 << std::endl;
		for(i=1; i<=ngrp1;i++){
			if(included[i]){
				if(chkprt>0) std::cout << "check first order polynomial eigvr[" << i << "]=" << (double)eigvr[i] << std::endl;
				if(eigint[i].upper().floor() == eigint[i].lower().ceil()){
					std::cout << "this interval includes integer" << std::endl;
					for (j=0; j<=nsize; j++) urat[nsize-j]=p[j];
					vrat[0]=-eigint[i].upper().floor(); vrat[1]=rational::ONE;
					isit=polydivchk(urat,nsize,vrat,1,qrat,rrat);
					if(isit){
						std::cout << " OK Divided up by the first order polynomial = " << std::endl;
						polytexformintr(vrat,1);
						included[i]=0; nsize --;
						for (j=0; j<=nsize; j++) p[nsize-j]=qrat[j];   //  replace P by Q 
						if(chkprt>0) std::cout << " Replaced characteristic polynomial new polynomial as;" << std::endl;
						if(chkprt>0) polytexform(p,nsize);
						npolynomial=putpolynomial(1,vrat,npolynomial,ppolynomial,cpolynomial);
						copymat(as,n,n,bs);    // matrix save in as for eigenvector computation
						for (int l=0; l<n; l++) bs[l][l]=bs[l][l]+vrat[0];
//						matprn(bs,n,n);
						std::cout << " Eigenvector computation for lambda=" << -(double)vrat[0] << std::endl;
						solhmg(bs,n,rank,eigenvector);   //  solve homogenious equation

					}  // if(polydivchk)
				}         //  if(askinty)
			}                //   if(includea[i]) || if(included[i])
		}                       //    for(i=1; i<=ngrp1;i++){
		if(nsize <= 0){
			if(nsize>0){
				for (i=0; i<=nsize; i++)  urat[nsize-i] =p[i];
				npolynomial=putpolynomial(nsize,urat,npolynomial,ppolynomial,cpolynomial);
				if(chkprt>0) std::cout << "added p() and replace npolynomial=" << npolynomial << std::endl;
			}
			continue;
		}
		// ***********************************************************************
		// ************ (4) compose divisor candidates and make trial division ***
		// ***********************************************************************
		std::cout << "Vieta's formula check  ngrp1=" << ngrp1 <<  std::endl;
		for(i=1; i<=ngrp1;i++){
			if(included[i] >= 1) std::cout << "included[" << i << "], eigvr[i]=" << (double)eigvr[i] << std::endl;   // dvsrch1:
		}
		d=2; noff=0; reset=0;
#ifdef  FACTORIZEONLY
		while(d<=nsize/2){              // search for polynomial of degree 2 to nsize/2
#else
		while(d<=nsize){                // search for polynomial of degree 2 to nsize
#endif
			if(d>nsize/2 && d!=nsize){
				d++; reset=0; noff=0; continue;
			}
			tstart0=gettime();
			std::cout << std::endl << "Divisor search for degree " << d << " starts. reset=" << reset << std::endl;
			if(reset==0){
			// *****************************************************************************
			// (*) refine accuracy of eigenvalues according to d, i.e., degree of polynomial
			// *****************************************************************************
				for(i=1; i<=ngrp1; i++){
					if(included[i]>=1){
						log2ev=log2(eigvl[i])+1;
						int reqndeig=((int)log2ev+2)*d+accdouble; // Required Number of Digits for eigenvalue
						std::cout << std::endl << "required accuracy i=" << i << " for eigval=" << eigvl[i] << " reqndeig=" << reqndeig << std::endl;
						int iwas = chkonce(nsize,p,eiglow[i],eigup[i],faeig,fbeig); 
						if(iwas==0){ 
							std::cout << "chkonce ERROR then STOP" << std::endl;
							std::cout << "  refine accuracy of eigval" <<  " Width of interval=" << (double)(eigup[i]-eiglow[i]) << std::endl; 
							exit(3);
						}
						double stmp=pow(2,log2ev*d); double epstmp=1/stmp;
						int irtn=bisect(p,nsize,epstmp,reqndeig,eigint[i],faeig,fbeig,eigvr[i]);
						if(irtn < 0){
							std::cout << " bisect not converged STOP" << std::endl; exit(3);
						}
						if(irtn==0){
//							irtn=reqndeig;
							std::cout << " bisect encountered rational number solution" << std::endl;
						}
						formatprn(eigint[i]);
					}
				}
			}
			// *****  set candidates of eigenvalues in ercan from eigvr  ************* 
			ncandi=setecanint(eigvr,eigint,ngrp1,included,ercan,ercanint,dfcand,usev);
			for(i=1; i<d;i++) idx[i]=i+noff; idx[d]=d+noff-1; // initialize for nxtcmb
			nn=comb(ncandi,d); reset=0;       // since nn is type "long long", ngrp1-nuse should be "< 62"
			std::cout <<  std::endl  << "comb(" << ncandi << "," << d << ")=" << nn << std::endl << std::endl;
			if(chkprt>0) std::cout << "Vieta's formula check for d=" << d << " ngrp1=" << ngrp1 << " ncandi=" << ncandi << std::endl;
			for(ii=1; ii<=nn;ii++){              // loop over nn combinations of candidates
				nxtcmb(ncandi,d,idx);        // select one of _(ncan-nused) C_d candidates
#ifdef INTYDOUBLE
				double ud=0;
				for(j=1; j<=d; j++) ud+=dfcand[idx[j]];
				if(askinty(ud,epsint)){         //  pass the first cofficient check, then ...
#else
				ur=rational::ZERO;                                           //dvsrch1: u ==> ur 
				for(j=1; j<=d; j++) ur+=ercan[idx[j]];
				if(askinty(ur,epsint,cor[1])){  //  pass the first cofficient check, then ...
#endif
					int ok=1;
					for(j=1; j<=d; j++) figv[j]=ercanint[idx[j]]; // dvsrch1
#ifdef VIETAZ
					ok=vietatermschk(figv,d,cor);
#else
					for(int r=1; r<=d; r++){
						rational ca,cb;
						vietaterm(figv,d,r,ca,cb);      //interval:
						if(cb.floor() == ca.ceil()){
							cor[r]=cb.floor();
						}else{
							std::cout << "cof[" << r << "] != its a and b   exit loop" << std::endl;
							ok=0; break; 
						}
					}
#endif
					if(ok){    // OK, try trial division
						for(int r=1; r<=d; r++) vrat[d-r]=cor[r];
						vrat[d]=rational::ONE;
						std::cout << " OK polynomial of degree " << d << " all coefficients seem to be integer" << std::endl;
						if(chkprt>0) for(int r=1; r<=d; r++) std::cout << " cor[" << r << "]=" << (double)cor[r];  //dvsrch1: cof ==> cor 
						vrat[d]=rational::ONE;
						if(chkprt>0) std::cout << " Trial division"  << std::endl;
						for (j=0; j<=nsize; j++) urat[nsize-j]=p[j];
						isit=polydivchk(urat,nsize,vrat,d,qrat,rrat);  // trial division
						std::cout << " Polydivchk rtnval=" << isit << std::endl;
						if(isit==0) std::cout << std::endl;
						if(isit){;                            // trial division divided up
							for (j=0; j<=nsize-d; j++) p[nsize-d-j]=qrat[j]; // set quatient for next p(\lambda)
							nsize = nsize-d;                                 // and decrease nsize
							std::cout << " ii=" << ii << " Divisor = "  << std::endl;
							polytexformintr(vrat,d);
							for (j=1; j<=d; j++) std::cout << " group is " << usev[idx[j]] << ", eigval=" << (double)ercan[idx[j]] << std::endl; 
							npolynomial=putpolynomial(d,vrat,npolynomial,ppolynomial,cpolynomial);
							if(chkprt>0) std::cout << "replaced. npolynomial=" << npolynomial << std::endl << std::endl;
							for (j=1; j<=d; j++) included[usev[idx[j]]]=0;
							noff=idx[1]-1;  // number of OFFed indecies
							if(d+noff>ncandi) break;
							reset=1; break;
						}    // if(divup) 
					}           //  if(ok){
				}	           //   if(areyouint(u,epsint))
			}                         //    for(ii=1; ii<=nn;ii++){
			if(reset==0){
				std::cout << "%combination loop for " << d << " complete, time=" << (double)(gettime()-tstart0)/1000000.0 << "(sec)" << std::endl;
				d++; noff=0;
			}
		}                               //      while(d<nsize){

		if(nsize>0){
			for (i=0; i<=nsize; i++)  urat[nsize-i] =p[i];
			npolynomial=putpolynomial(nsize,urat,npolynomial,ppolynomial,cpolynomial);
			std::cout << "added p() and replace npolynomial=" << npolynomial << std::endl;
		}
		std::cout << std::endl;
		std::cout << "END of k =" << k << std::endl;
		if(chkprt>0) std::cout << "PRINT current divisors found npolynomial=" << npolynomial << std::endl;
		for(int iod=1; iod<=npolynomial; iod++){
			int ksz=getpolynomial(iod,vrat,npolynomial,ppolynomial,cpolynomial);
			if(k==0){
				polytexformr2(vrat,ksz,rational::ONE);  //  add % 
			}else{
				if(chkprt>0) polytexformintr(vrat,ksz);
			}
		}	
		if(chkprt>0) std::cout << "PRINT current divisors found END" << std::endl;
	}         //                              for(k=nmat; k>=0; k--){
}

int setecanint(const rat::vector<rational>& eigvr, const vector<interval>& eab, const int ncan, const int included[],
		rat::vector<rational>& ercan, rat::vector<interval>& ercanab, vector<double>& dfcand, int usev[])
{ 
	int j=0;
	for(int i=1; i<=ncan;i++){     
		if(included[i]){                           // exclude eigvl[i] according to included[i]
			j++; 
			usev[j]=i; 
			ercan[j]=eigvr[i];
			ercanab[j]=eab[i]; 
			rational tmp=eab[i].upper();
			dfcand[j]=(double)(tmp-tmp.floor()); // extract 53 bits under the point of eab[i].upper()
		};
	}
	return j;        // return number of candidates set in array evcan[1] to [j]
}
void selectev(const vector<double>& a, int n, int& ngrp, vector<double>& eigvl, vector<double>& eigvh, rat::vector<int>& mult)
{
	int i,ii,j=0;
	double x,am;
	am=fabs((a[n-1]-a[0])/((double)n));  // abs(mean)
	std::cout << "selectev Amean=" << am  << std::endl;
	eigvl[j+1]=a[0]; eigvh[j+1]=0; x=a[0]/am; ii=0; mult[j+1]=0;
	for(i=1; i < n; i++){
		if(fabs(a[i]/am-x) > eps){
			j++; x = a[i]/am; eigvl[j+1]=a[i]; ii=i; mult[j+1]=0;
		}
		if(ii != i){
			eigvh[j+1]=a[i]; // higher eigval of jth group
			mult[j+1]++;
		}
	}
	ngrp = j;
}

void nmhes(rat::matrix<rational>& a, int n, rat::vector<int>& ith, int& nmat)
{
	rational x, y;
	int j=1;
	std::cout << "nmhes  start n=" << n  << std::endl;
	for (int i=2; i <= n; ++i) {
		if (a[i-1][i-2] == rational::ZERO) {
			ith[j-1] = i-2;
			j++;
		}
	}
	ith[j-1] = n-1;
	nmat = j-1;
}

void cpsubmat(rat::matrix<rational>& a, int n, rat::matrix<rational>& b, int i1, int i2)
{
	int nsize = i2-i1+1;
	for (int j=0; j < nsize; ++j) {
		for (int i=0; i < nsize; ++i) b[i][j] = a[i+i1][j+i1];
	}
}

int chkonce(const int n, const rat::vector<rational>& p, const rational a, const rational b, rational& fa, rational& fb)
{
	int rtnval=1;
	fa=Horner(n,p,a);
	fb=Horner(n,p,b);
	if(fa*fb > rational::ZERO) rtnval=0;
	return rtnval;
}
int chkinc(const int n, const rat::vector<rational>& p, const double e, const double h, const int mult, rational& a, rational& b)
{
	double ta,tb; 
	int rtnval=0;
	rational fa,fb;
	
	if(fabs(e) > eps){
		ta=e-fabs(e)*eps; tb=h+fabs(h)*eps;
		if(mult==0) tb=e+fabs(e)*eps;
	}else{
		ta=e-eps; tb=h+eps;   // consider eigenvalues close to zero 
		if(mult==0) tb=e+eps;
	}
	a = Rdset(&ta); fa=Horner(n,p,a);
	b = Rdset(&tb); fb=Horner(n,p,b);
	if(fa*fb<rational::ZERO) rtnval=1;
	return rtnval;
}

#ifdef VIETAZ
int vietatermschk(const vector<interval>& e, const int n, rat::vector<rational>& cor)
{
	int i,r,inty,ok=1;
	rat::vector<int> last(n+1);
	rat::vector<int> num(n+2);
	vector<interval> p(n+1);               //interval
#ifdef VIETACOUNT
		vietamul=0;
		vietamultime=0;
#endif
	// ******************************************************************
//	std::cout << "vietatermschk called n=" << n << std::endl;
//	for(int j=1; j<=n; j++) formatprn(e[j]);
	// ******************************************************************
//	for(i=0; i<n+1; i++){ p[i]=interval(rational::ZERO,rational::ZERO);}
	for(i=1; i<n; i++){
		long long tstart0=gettime();
		num[2]=i;
		for(int j=1; j<n; j++) last[j]=0;
		inty=mulvieta(num,e[i],n,2,e,p,last,1);
		if(inty==0){ ok=0; break;}
		std::cout << "loop in vietatermschk i=" << i << " time=" << (double)(gettime()-tstart0)/1000000.0 << "(sec)" << std::endl;
#ifdef VIETACOUNT
		std::cout << "VIETACOUNT vietamul=" << vietamul << " vietamultime=" << vietamultime << std::endl;
#endif
	}
	if(ok==1){
		interval t=e[1];	
		for(r=2; r<=n; r++){ 
			interval er=e[r];	
			t=t+er;	
		}
		if((t.upper()).floor() == (t.lower()).ceil()){
			cor[1]=-(t.upper().floor());
		}else{
			ok=0;
		}
		std::cout << "first order term=" << (double)cor[1] << std::endl;
		formatprn(t);
	}
	if(ok==1){
		for(r=2; r<=n; r++){
			if(p[r].upper().floor() == p[r].lower().ceil()){
				if(r%2 == 1){
					cor[r]=-p[r].upper().floor();
				}else{
					cor[r]=p[r].upper().floor();
				}
			}else{
				ok=0; break; 
			}
			std::cout << r << "-th term=" << (double)cor[r] << std::endl;
			formatprn(p[r]);
		}
	}
	return ok;
}

int mulvieta(rat::vector<int>& num,const interval& sp,const int n,const int r,const vector<interval>& e, vector<interval>& p,rat::vector<int>& last,int dir) 
{
	int dgt, dgt1, rtnval=1;
	interval t;
	dgt=num[r];
	if(dir == 1){
		dgt1=dgt+1;
	}else{
		dgt1=last[r]+1;
	}
	t=sp*e[dgt1];
#ifdef VIETACOUNT
	rational tmp=sp.upper(); longint tn=tmp.numerator();
	tmp=e[dgt1].upper(); longint sn=tmp.numerator();
	vietamul+=numdgts(tn)+numdgts(sn);
	tmp=sp.lower(); tn=tmp.numerator();
	tmp=e[dgt1].lower(); sn=tmp.numerator();
	vietamul+=numdgts(tn)+numdgts(sn);
	vietamultime++;
#endif
	p[r]=p[r]+t;      // p[r] += t; is not yet
	last[r]=dgt1;

	if(r == n){	          // check if the final term 123...n case
		if(p[r].upper().floor() != p[r].lower().ceil()) return 0;
	}
	if(dgt1 < n){
		num[r+1]=dgt1;
		int inty=mulvieta(num,t,n,r+1,e,p,last,1); 
		if(inty == 1){
			if(dgt1 <n){
				mulvieta(num,sp,n,r,e,p,last,0);
			}
		}else{
			rtnval=0;
		}
	}
	return rtnval;
}
#else
int vietaterm(const vector<interval>& e, const int n, const int r, rational& ta, rational& tb)
{
	interval s;
	int ith[r+1];
	for(int i=1; i<=r;i++){ith[i]=i;}
	ith[r]=ith[r]-1;

	interval t=interval(rational::ZERO,rational::ZERO);
	long long nn=comb(n,r);
	for(long long i=1; i<=nn; i++){
		nxtcmb(n,r,ith);
		s=interval(rational::ONE,rational::ONE);
		for(int j=1; j<=r; j++) {
			if(e[ith[j]].lower().s * e[ith[j]].upper().s < 0){
				std::cout << "rational.vietaterm ERROR STOP" << std::endl;
				exit(3);
			}
			s=s*e[ith[j]];
		}
		t=t+s;
	}
	if(r%2==1) { t=t.neg(); }
	ta=t.lower(); tb=t.upper();
	return 1;
}
#endif
void RatFtherm(rat::matrix<rational>& a, int m) {
	int i, j;
	rational adiag = RRset(longint::FOUR,longint::TEN);
	rational offdg = RRset(longint::ONE,longint::TEN);
	offdg=-offdg;
//	std::cout << "RatFterm generates matrix m=" << m << " adiag=" << adiag << " offdg=" << offdg << std::endl;

	for (i=0; i < m*m; ++i) {
		for (j=0; j < m*m; ++j) {
			a[i][j] = 0;
		}
	}
	for (i=0; i < m; ++i) {
		if (i >= 1) a[i][i-1] = offdg;
		a[i][i] = adiag;
		if (((i+1) % m) != 0) a[i][i+1] = offdg;
		a[i][i+m] = offdg;
	}
	for (i=m; i < (m*(m-1)); ++i) {
		a[i][i-m] = offdg;
		if (((i+1) % m) != 1) a[i][i-1] = offdg;
		a[i][i] = adiag;
		if (((i+1) % m) != 0) a[i][i+1] = offdg;
		a[i][i+m] = offdg;
	}
	for (i=m*(m-1); i < m*m; ++i) {
		a[i][i-m] = offdg;
		if (((i+1) % m) != 1) a[i][i-1] = offdg;
		a[i][i] = adiag;
		if (((i+1) % m) != 0) a[i][i+1] = offdg;
	}
}

void genDtherm(rat::matrix<double>& a, int m) {
	int i, j;
	double adiag = 0.4;
//	double ediag = 0.4000000000000001;
	double offdg = -0.1;
//	double adiag = 4;
//	double ediag = 4.000000000000001;
//	double offdg = -1;

	for (i=0; i < m*m; ++i) {
		for (j=0; j < m*m; ++j) {
			a[i][j] = 0;
		}
	}
	for (i=0; i < m; ++i) {
		if (i >= 1) a[i][i-1] = offdg;
		a[i][i] = adiag;
		if (((i+1) % m) != 0) a[i][i+1] = offdg;
		a[i][i+m] = offdg;
	}
	for (i=m; i < (m*(m-1)); ++i) {
		a[i][i-m] = offdg;
		if (((i+1) % m) != 1) a[i][i-1] = offdg;
		a[i][i] = adiag;
		if (((i+1) % m) != 0) a[i][i+1] = offdg;
		a[i][i+m] = offdg;
	}
	for (i=m*(m-1); i < m*m; ++i) {
		a[i][i-m] = offdg;
		if (((i+1) % m) != 1) a[i][i-1] = offdg;
		a[i][i] = adiag;
		if (((i+1) % m) != 0) a[i][i+1] = offdg;
	}
//	a[0][0] = ediag;     //  contam error
}
