/******************************************************************************
 
g++ -O3 dvsrch2.cpp ratutil.o rational.o longint.o interval.o rough.o archive.o numeric_cast.o thread_manager.o mempool.o gettimeofday.o

to see source code after preprocess
g++ -E dvsrch2.cpp -c > dvsrch2C.cpp


  CT2D matrix

 2.1  checkpoint run:   ./a.exe 0 S13free 0 | tee S13free.txt
 2.2  restart run:      ./a.exe 1 S13free 0 | tee S13free1.txt  
                                  use S13freeA.chk to read H matrix
      restart run:      ./a.exe 2 S13free 0 | tee S13free1.txt  
                                  use S13freeA2.chk to read F matrix, i.e., characterestic polynomial
 2.1  checkpoint run:   ./a.exe 0 S13freeO3 0 | tee S13free.txt

******************************************************************************/
#define FACTORIZEONLY
//#define FACTORCOMP
//#define BISECT   // BISECT or Regura Falsa
#include <iostream>
#include <fstream>  
#include <iomanip>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <algorithm>
#include <stdexcept>
#include <sstream>
//#include "longint.h" 
#include "rational.h"
#include "rvector.h"
#include "rmatrix.h"
#include "interval.h"  
#include "rblas.h"
#include "ratutil.h"
#include "rough.h"
#include "numeric_cast.h"

#include "archive.h"

using namespace rat;


void eigvals(char*);

void selectev(const vector<double>& a, int n, int& ngrp, vector<double>& eigvl, vector<double>& eigvh, rat::vector<int>& mult, double& am);
void nmhes(rat::matrix<rational>& a, int n, rat::vector<int>& ith, int& nmat);
void cpsubmat(rat::matrix<rational>& a, int n, rat::matrix<rational>& b, int i1, int i2);

int setecanint(const rat::vector<rational>& eigvr, const vector<interval>& eab, const int ncan, const int included[], rat::vector<rational>& ercan,  rat::vector<interval>& ercanint, vector<double>& dfcand, int usev[]); 

int chkonce(const int n, const rat::vector<rational>& p, const rational a, const rational b, rational& fa, rational& fb);
int chkinc(const int n, const rat::vector<rational>& p, const double e, const double h, const int mult, const double am, rational& a, rational& b);
int countinc(int n, const rat::vector<rational>& p, rational& a, rational& b, const int mult,  const double eps, const int maxitarg, vector<interval>& abtemp);
int bisect2(const rat::vector<rational>& p, const int n, const int maxit, rational& a, rational& fa, rational& fda, rational& b, rational& fb, rational& fdb);
int bisect3(const rat::vector<rational>& p, const int n, const int maxit, rational& a, rational& fa, rational& fda, rational& fdda, rational& b, rational& fb, rational& fdb, rational& fddb);
int vietatermschk(const vector<interval>& e, const int n, rat::vector<rational>& cor);
int mulvieta(rat::vector<int>& num,const interval& sp,const int n,const int r,const vector<interval>& e, vector<interval>& p,rat::vector<int>& last,int dir); 

const double eps=1e-9;
// *******************************************************************
// *******************************************************************
static double *aa, *ma, *ka;
static int chkres;
static int mattype;     // 0:A, 1:K, 2:MinvK
// *******************************************************************
std::string sub3(rational const& a)
{
	std::stringstream ss; // needs #include <sstream>
	rough b = numeric_cast<rough>(a);
//	int default_precision = std::cout.precision(17);
	ss << " " <<  b << " ";
	return ss.str();
}

int main(int argc, char *argv[]) 
{
	extern int totalit;  // total number of iterations, bisect(), bisectrf() in ratutil.cpp
	char matname[1024];
	long long tstart0=gettime();
	sscanf(argv[1],"%d\n",&chkres);
	std::cout << "start  dvsrch2 of file-reading argc=" << argc << " argv[1]=" << argv[1] << " argv[2]=" << argv[2] <<  " argv[3]=" << argv[3] << std::endl;
	std::cout << "        chkres =" << chkres;
	sscanf(argv[2],"%s\n",matname);
	std::cout << " matname =" << matname;
	sscanf(argv[3],"%d\n",&mattype);
	std::cout << " mattype =" << mattype << std::endl;

	eigvals(matname);

	std::cout << "dvsrch2 finish. total time=" << (double)(gettime()-tstart0)/1000000.0 << "(sec)" << std::endl;
	std::cout << "totalit=" << totalit << std::endl;
	return 0;
}

void eigvals(char matname[]){
	char matfile[1024];
	double tmp;
	
std::cout << std::endl;
	std::cout << "%dvsrch2 start.";
#ifdef FACTORIZEONLY
	std::cout << " FACTORIZEONLY,";
#endif
#ifdef  FACTORCOMP
	std::cout << " FACTORCOMP,";
#endif
#ifdef  EPROF2
	std::cout << " EPROF2,";
#endif
#ifdef  BISECT
	std::cout << " BISECT,";
#endif
std::cout << std::endl;

// *******   Declare scalar variables *********************************************
	int i, j, k, n, i1, i2, itr[2], nexist, ok, nok=0, rank;
	int ngrp, nmat, nsize, ngrp1, ncandi, d, noff, reset, isit, reqndeig;
	double epsint=1e-5, eveps, accdouble=45, log2ev, evlarge, am;
	long long nn, ii,tstart0,tstart1,tstart2=0,tstarti,tstartr,ttp;// nn used for number of combination
	double dlcm=1;
	rational ur, ura, urb, Clcm=rational::ONE, faeig, fbeig, ceig;
// ********************************************************************
	std::ifstream fin;
	std::ofstream ofs;
	std::fstream ifs;
	iarchive ia(ifs);         // #include "archive.h"
	oarchive oa(ofs);         // #include "archive.h"
	strcpy(matfile,matname);  // required #include <string.h>
	strcat(matfile, ".txt");
	std::cout << "Reading CT2D output matfile =" << matfile << std::endl;
	fin.open(matfile);
	if(!fin){
			std::cout << "cannot open input file\n";
			exit(EXIT_FAILURE);
	}

	fin >> n;
	std::cout << "dvsrch2 n=" << n << std::endl;
// *******************************************************************
// *******************************************************************
// *******************************************************************
// *******   Declare array type variables ****************************
	int usev[n+1], included[n+1], idx[n+1];
	rat::vector<rational>  ercan(n+1), cor(n+1), figvr(n+1);
	vector<interval> figv(n+1), ercanint(n+1);
	rat::vector<rational>  figvaf(n+1), figvbf(n+1), efcana(n+1), efcanb(n+1);
	vector<double> dfcand(n+1), figdaf(n+1);
	rat::matrix<double> a(n,n), ev(n,n);
	vector<double> v(n+1), est(n+1), eigvl(n+1), eigvh(n+1);
	rat::vector<int> ith(n), mult(n+1);
	rat::matrix<rational> ar(n+1,n+1),b(n+1,n+1),as(n+1,n+1),bs(n+1,n+1),eigenvector(n+1,n+1);
	rat::matrix<rational> bb(n+1,n+1);
	rat::vector<rational> p(n+1), urat(n+1), vrat(n+1), qrat(n+1), rrat(n+1);
	rat::vector<rational> eigvr(n+1), eiglow(n+1), eigup(n+1);   
	rat::matrix<rational> h(n+1,n+1);      
	rat::vector<rational> atemp(n+1), btemp(n+1); 
	rat::vector<interval> abtemp(n+1), eigint(n+1); 
	// ******* get, put polynomials   *********************************
	int npolynomial=0;                // number of polynomials found  
	rat::vector<int> ppolynomial(n+2);     // pointer to  polynomials      
	rat::vector<rational> cpolynomial(n*2); // coefficients of polynomials  
	// ****************************************************************
	ppolynomial[0]=0;
	// *******   Matrix generation  ***************************************
	// ***************  read matrix from file made by CT2D program ********
	rat::matrix<double> ak(n,n); 
	rat::matrix<rational> kar(n+1,n+1); 
	ka = new double[n*n];
	ma = new double[n];
	aa = new double[n*n];
	for (j=0; j<n; j++){
		for (i=0; i<n; i++){ aa[j*n+i]  = 0; ka[j*n+i]  = 0; }
		ma[j]  = 0;
	}
	std::cout << " Now read file=" << matfile << std::endl;
	for (;;){
		fin >> i;
		fin >> j;
		fin >> tmp;
		ka[n*j+i]=tmp;
		if(i==n-1 && j==n-1) break;
	}
	for (j=0; j<n; j++){
		fin >> tmp;
		ma[j]=tmp;
	}
	for (;;){
		fin >> i;
		fin >> j;
		fin >> tmp;
		aa[n*j+i]=tmp;
//		std::cout << " read i=" << i << " j=" << j << " val=" << tmp << std::endl;
		if(i==n-1 && j==n-1) break;
	}
	fin.close();
	std::cout << "dvsrch2 read file end" << " n*n=" << n*n << std::endl;

	std::cout << "Input matrix print" <<  std::endl;
// ******** format conversion from C type one dimensional array to matrix format *********
	for (j=0; j < n; ++j) {
		for (i=0; i < n; ++i){ a[i][j] = aa[j*n+i]; }
	}
	std::cout << "Converted matrix" <<  std::endl;
	for (j=0; j<n; j++){
		for (i=0; i<n; i++){ 
			std::cout << " " << a[i][j];
		}
		std::cout << std::endl;
	}

// *********  convert matrix from double to rational ****************
	std::cout << "mattype=" << mattype <<  std::endl;
	if(mattype == 2){          //MinvK
		std::cout << " MinvK selected for A matrix" << std::endl;
		for (j=0; j < n; ++j) {
			for (i=0; i < n; ++i){ ak[i][j] = ka[j*n+i]; }
		}
		cnvmat(ak,kar,n,n);    // kar(n,n) <-- K matrix rational
		for (i=0; i < n; ++i) {
			rational mforinv = Rdset(&ma[i]);
			for (j=0; j < n; ++j){  h[i][j] = kar[i][j] / mforinv; }
		}                        // h(n,n) <-- M^{-1}K matrix rational
	}else if(mattype == 1){
		std::cout << " K selected for A matrix" << std::endl;
		for (j=0; j < n; ++j) {
			for (i=0; i < n; ++i){ a[i][j] = ka[j*n+i]; }
		}
		cnvmat(a,h,n,n);     // h(n,n) <-- K matrix rational
	}else{
		std::cout << " SinvKSinv selected for A matrix" << std::endl;
		cnvmat(a,h,n,n);    // h(n,n) <-- S^{-1}KS^{-1} matrix rational
	}
	std::cout << n << "x" << n << " A matrix" << std::endl;
// ********  h changed to LCM mul matrix in ar   **************
	matprn(h,n,n); MatDgtCount(h,n,n);
	Clcm=rat2int(h,n,ar);
	dlcm=Clcm;
	std::cout << "%LCM of denominators of matrix A =" << Clcm  << " dlcm=" << dlcm << std::endl;
	longint mgcd=matngcd(ar,n);
	std::cout << "%GCD of numerators of integer matrix=" << mgcd  << std::endl;
	copymat(ar,n,n,as);    // matrix save in as for eigenvector computation
// ********* matprn(ar,n,n); MatDgtCount(ar,n,n);
	MatDgtCountPrt(ar,n,n);
	if(chkres==0){
		// Convert symmetric matrix to Hessenberg form
		// by using elementary transformation is similarity transformation form.
		tstart0=gettime();          // ***************************************
		elmhes(ar,n);               // *********   ELMHES  *******************
		std::cout << std::endl;     // ***************************************
		std::cout << " ELMHES complete, time=" << (double)(gettime()-tstart0)/1000000.0 << "(sec)" << std::endl;
	        // ----- checkpoint H 
		std::stringstream ofn;
		std::stringstream ofn2;
		if(mattype==0){
			ofn << matname << "A" << ".chk";
			ofn2<< matname << "A2" << ".chk";
		}else if(mattype==1){
			ofn << matname << "K" << ".chk";
			ofn2<< matname << "K2" << ".chk";
		}else{
			ofn << matname << "MinvK" << ".chk";
			ofn2<< matname << "MinvK2" << ".chk";
		}
		ofs.open(ofn.str().c_str(),std::fstream::trunc);
		if (!ofs) {
			throw std::runtime_error("Write open failed for " + ofn.str());
		}
		oa << ar;
		ofs.close();
		ofs.open(ofn2.str().c_str(),std::fstream::trunc);  //  archive file for Frobenius matrix
		if (!ofs) {
			throw std::runtime_error("Write open failed for " + ofn2.str());
		}
	}else if(chkres==1){              //   read H from checkpoint file
		std::stringstream ifn;
		if(mattype==0){
			ifn << matname << "A" << ".chk";
		}else if(mattype==1){
			ifn << matname << "K" << ".chk";
		}else{
			ifn << matname << "MinvK" << ".chk";
		}
		ifs.open(ifn.str().c_str(),std::ofstream::in);
		if (!ifs) {
			throw std::runtime_error("Read open failed for " + ifn.str());
		}
		ia >> ar;
		std::cout << "Read Hessenberg matrix from checkpoint file" << std::endl;
		ifs.close();
	}
// *******************************************************************
	MatDgtCountPrt(ar,n,n);
	std::cout << std::endl;
// *******************************************************************
//	matprn(ar,n,n);
	int maxit=n*4; double tol=1e-10;
	int irtn=jacobev(a,ev,tol,n,maxit);  // compute all eigenvalues and eigenvectors in double precision
	if(irtn<0){
		std::cout << " jacobev NOT converged stop" << std::endl;
		exit(3);
	}
	//*********************************************************************
	//  eigenvalues are multiplied by LCM of denominators of A.
	for (i=0; i<n; i++){   
		if(dlcm > 0){  //  m * A * x = m * \lambda * x   
			v[i] = a[i][i] * dlcm; 
		} else{ 
			v[i] = a[i][i];
		}      
	}   
	//************************************************
	bsort(v,n);   //  sort (bubble sort) eigenvalues
	//************************************************

	std::cout << n << " eigenvalues obtained by using double precision floating-point arithmetic" << std::endl;
	for (i=0; i<n; i++){
		est[i] = v[i];
		std::cout << i+1 << "-th estimate eigval = " << std::setprecision(16) << est[i] << std::endl;
	}

	//********************************************************************
	selectev(est,n,ngrp,eigvl,eigvh,mult,am);  //  grouping eigenvalues 
	//********************************************************************
		
	ngrp1=ngrp+1;
	std::cout << ngrp1 << " eigenvalues selected" << std::endl;
	std::cout << "  group          eivenvalue "<< std::endl;
	int ismal=0;
	for (i=1; i<=ngrp1; i++){
		if(i==1){
			eveps=epsint;
		}else{
			if(fabs(eigvl[i]-eigvl[i-1]) < eveps){
				eveps = fabs(eigvl[i]-eigvl[i-1]);
				ismal=i;
			}
		}
		std::cout << "  "  << i << "   " << eigvl[i]  << "   " << eigvh[i] << " multiplicity " << mult[i]+1 <<  std::endl;
	}
	if(epsint > eveps) epsint=eveps;
	std::cout << " Smallest eigenvalue gap="  << eveps << "  epsint=" << epsint << " @ i=" << ismal << std::endl;
	std::cout << std::endl;
	//*************************************************************************
	nmhes(ar,n,ith,nmat);	// count number of submatrices of Hessenberg form
	//*************************************************************************
	if(chkres==0){
		oa << nmat;
		oa << ith;
	}else if(chkres>=2){
		std::stringstream ifn;
		if(mattype==0){
			ifn << matname << "A2" << ".chk";
		}else if(mattype==1){
			ifn << matname << "K2" << ".chk";
		}else{
			ifn << matname << "MinvK2" << ".chk";
		}
		ifs.open(ifn.str().c_str(),std::ofstream::in);
		if (!ifs) {
			throw std::runtime_error("Read open failed for " + ifn.str());
		}
		ia >> nmat;
		ia >> ith;
		std::cout << "Read checkpoint file2 nmat=" << nmat << " ith=" << ith << std::endl;
	}
	//*************************************************************************
	std::cout << "Hessenberg matrix size of " << n << " is divided into " << nmat+1 << " submatrices" << std::endl;
	i1=1;
	for (i=0; i<=nmat; i++){
		std::cout << i+1 << "-th submatrix = " << i1 << " to " << ith[i]+1 << std::endl;
		i1=ith[i]+2;
	}
	std::cout << std::endl;
	std::cout << "Eigenvalue computaion for " << nmat+1 << " submatrices" << std::endl;
	// *******************************************************************
	// ***** Loop over each Hessenberg submatrix  ************************
	// *******************************************************************
	for (k=nmat; k>=0; k--){
		i2=ith[k];
		std::cout << std::endl;
		std::cout << std::endl;
		std::cout << "K=" << k << " Ith=" << ith[k] << std::endl;
		if(k>0) i1=ith[k-1]+1;  else  i1=0; 
		nsize = i2-i1+1;
		std::cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%"  << std::endl;
		std::cout << "%Sub Matrix  " << k+1 << " size =" << nsize << " i1=" << i1+1 << " i2=" << i2+1 << std::endl;
		std::cout << "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%"  << std::endl;
		//********************************************************************************
		cpsubmat(ar,n,b,i1,i2);      // ----- copy submatrix from ar to b 
		//********************************************************************************
		if(chkres<=1){
			tstart0=gettime();
		        // ****************************************************************************
			hesfrb(b,nsize,p);  // compute characteristic polynomial of Hessenberg submatrix
		        // ****************************************************************************
			std::cout << std::endl;
			std::cout << "hesfrb complete, time=" << (double)(gettime()-tstart0)/1000000.0 << "(sec)" << std::endl;
			std::cout << std::endl;

			oa << p;
			std::flush(ofs);
			std::cout << " wrote checkpoint file" << std::endl;
			std::cout << std::endl;
			if(k==0) ofs.close();
		}else{
			ia >> p;
			std::cout << "Read characterestic polynomial from checkpoint file" << std::endl;
		}
		std::cout << " " << std::endl;
		std::cout << "NumDgts of coefficients of the characterestis polynomial " << std::endl;
		for (i=0; i<=nsize; i++){
			longint xd = p[i].denominator();
 			longint xn = p[i].numerator();
 			std::cout << "i=" << i << " " << numdgts(xd)+numdgts(xn) << std::endl;
		}
		std::cout << "%Characteristic polynomial"<< std::endl; polytexformintfmt(p,nsize);
		// *************************************************************************************
		// (1)  divide by the known (found in the previous loop iterations) divisors  **********
		// *************************************************************************************
		std::cout << "divide by the known (found in previous iterations) divisors=" << npolynomial << std::endl;
		for(int iod=1; iod<=npolynomial; iod++){
			int ksz=getpolynomial(iod,vrat,npolynomial,ppolynomial,cpolynomial);
			polytexformintr(vrat,ksz);
			for (i=0; i<=nsize; i++)  urat[nsize-i] =p[i];
			if(polydivchk(urat,nsize,vrat,ksz,qrat,rrat)){
				for (i=0; i<=nsize-ksz; i++) p[nsize-ksz-i]=qrat[i];   //  replace P by Q 
				nsize = nsize-ksz; 
				std::cout << "Divided up by the known divisor = " << std::endl; polytexformintr(vrat,ksz);
				std::cout << " Replaced characteristic polynomial new polynomial follows;" << std::endl;
				polytexform(p,nsize);
				npolynomial=putpolynomial(ksz,vrat,npolynomial,ppolynomial,cpolynomial);
			}
		}
		std::cout << "divide by the known polynomials END" << std::endl << std::endl;
		// *******************************************************************
		// ********** (2)  inclusion and refine accuracy *********************
		// *******************************************************************
		nexist=0;
		tstarti=gettime();
		for(i=1; i<=ngrp1;i++){
			std::cout << std::endl << " inclusion loop i=" << i  <<  std::endl;
			rational tempa,tempb;
			int idummy, nmult, maxit = 53;
			double eigvli=eigvl[i], eigvhi=eigvh[i];
			if(fabs((eigvli+eigvhi)/am) < eps){  //   eigenvalue close to zero
				if(eigvli*eigvhi <= 0){
					eigvli=eigvli*2; eigvhi=eigvhi*2; // multiply 2 in case of this interval includes zero.
				}else{
					eigvli=eigvli-fabs(eigvli)*2; eigvhi=eigvhi+fabs(eigvhi)*2;
				}
				std::cout << " Close to zero eigval, replaced eigvl=" << eigvl[i]  << " to " << eigvli << " eigvh=" <<  eigvh[i]  << " to " << eigvhi <<  std::endl;
			}
			idummy = chkinc(nsize,p,eigvli,eigvhi,mult[i],am,tempa,tempb);
			std::cout << " chkinc return=" << idummy  <<  std::endl;
			nmult = countinc(nsize,p,tempa,tempb,mult[i],epsint,maxit,abtemp);
			std::cout << std::endl << " Retern val countinc=" << nmult  << " for candidate, eigvl=" << eigvl[i] << std::endl;
			for(j=0; j<nmult; j++){
				nexist ++; eigint[nexist]=abtemp[j]; included[nexist]=1;
				eigvr[nexist]=(abtemp[j].lower()+abtemp[j].upper())/rational::TWO;
				std::cout << "       j=" << j << " nexist=" << nexist << " eigval=" << (double)eigvr[nexist] << " a=" << abtemp[j].lower() << " b=" << abtemp[j].upper() <<  " Width of interval=" << (double)(abtemp[j].upper()-abtemp[j].lower()) << std::endl; 
			}
		}
		std::cout << std::endl << "%FOUND " << nexist << " intervals.  nsize=" << nsize << std::endl;
		ttp=gettime()-tstarti;
		std::cout << " inclusion and Refine time=" << (double)(ttp)/1000000.0 << "(sec)" << std::endl;
		tstart2+=ttp;
		if(nexist < nsize){
			std::cout <<  " Less inclusion STOP" << std::endl; exit(3);
		}
	// ***********************************************************************************************
		if(nsize == nexist){
			std::cout << " in each of intervals one root exists. For the first order factors," << std::endl;
			for(i=1; i<=nexist;i++){
				std::cout << std::endl << " refine accuracy i=" << i << " for eigval=" << (double)eigvr[i] <<  " eiglow=" << (double)eigint[i].lower() <<   " eigup=" << (double)eigint[i].upper() <<  std::endl;
				// ******** Preprocess to call bisect, *********************
				Horner(nsize,p,eigint[i],faeig,fbeig);
				if(faeig*fbeig > rational::ZERO){
					std::cout << " ERROR check Horner .....faeig=" << faeig << " fbeig=" << fbeig << std::endl;
					formatprn(eigint[i]); polytexform(p,nsize); exit(3);
				}
				// ******** bisect to refine eigenvalue *********************
#ifdef BISECT
				int irtn = bisect(p,nsize,eps,100,eigint[i],faeig,fbeig,ceig);
#else
				int irtn = bisectrf(p,nsize,eps,100,eigint[i],faeig,fbeig,ceig);
#endif
				if(irtn < 0){
					std::cout << " bisect not converged STOP" << std::endl; exit(3);
				}
				if(irtn==0){
					irtn=100; // eigvr[i] is solution, i.e., eigenvalue is rational number
					std::cout << " bisect encountered rational number solution" << std::endl;
				}
				std::cout << "    "; 
				formatprn(eigint[i]);
				// **********************************************************
				std::cout << "  Return from bisect irtn=" <<  irtn << std::endl; 
				included[i] = abs(irtn);
				eigvr[i]=ceig; evlarge=(double)ceig; // save the largest eivenvalue
				if(irtn==0) included[i]=1; 
				formatprn(eigint[i]);
				longint eln=eigint[i].lower().numerator();
				longint eld=eigint[i].lower().denominator();
				longint eun=eigint[i].upper().numerator();
				longint eud=eigint[i].upper().denominator();
				std::cout << " eigint_l_n=" << numdgts(eln) << " eigint_l_d=" << numdgts(eld) << " eigint_u_n=" << numdgts(eun)  << " eigint_u_d=" << numdgts(eud) << std::endl; 
			}
		}
		// *******************************************************************
		// ************* (3)  divide by the first order polynomials **********
		// *******************************************************************
		std::cout << std::endl << "divide by the first order polynomial ngrp1=" << ngrp1 << std::endl;
		for(i=1; i<=nexist;i++){
			std::cout << "Check first order polynomial (double)eigvr[" << i << "]=" << (double)eigvr[i] << std::endl;
			if(eigint[i].upper().floor() == eigint[i].lower().ceil()){
				std::cout << "this interval includes integer" << std::endl;
				for (j=0; j<=nsize; j++) urat[nsize-j]=p[j];
				longint tempden=eigvr[i].denominator();
				if(tempden!=longint::ZERO){
					vrat[0]=-eigint[i].upper().floor(); vrat[1]=rational::ONE;
				}else{
					std::cout << "eigvr denominator zero " << tempden << std::endl;
				}
				longint tempnum=vrat[0].numerator();
				std::cout << "longint tempnum=" << tempnum << std::endl;
				if(tempnum==longint::ZERO){
					if(urat[0]!=rational::ZERO){
						isit=0;
					}else{
						isit=1;
					}
					std::cout << "Zero vrat[0] isit=" << isit << std::endl;
				}else{
					isit=polydivchk(urat,nsize,vrat,1,qrat,rrat);
				}
				if(isit){	//****** trial division by polydivchk (rational.cpp) ***
					std::cout << "%OK Divided up by the first order polynomial = " << std::endl; polytexformintr(vrat,1);
					included[i]=0; nsize --;
					for (j=0; j<=nsize; j++) p[nsize-j]=qrat[j];   //  replace P by Q 
					std::cout << " Replaced characteristic polynomial new polynomial is;" << std::endl;
					polytexform(p,nsize);
					npolynomial=putpolynomial(1,vrat,npolynomial,ppolynomial,cpolynomial);
					std::cout << " Eigenvector computation for lambda=" << -(double)vrat[0] << std::endl;
					copymat(as,n,n,bs);
					for (int l=0; l<n; l++) bs[l][l]=bs[l][l]+vrat[0];
					solhmg(bs,n,rank,eigenvector);   //  solve homogenious equation
				}  // if(isit)
			}         //  if(eigint[i].upper().floor() == eigint[i].lower().ceil())
		}                //   for(i=1; i<=nexist;i++){
		if(nsize <= 0){
			if(nsize>0){
				for (i=0; i<=nsize; i++)  urat[nsize-i] =p[i];
				npolynomial=putpolynomial(nsize,urat,npolynomial,ppolynomial,cpolynomial);
				std::cout << "added p() and replace npolynomial=" << npolynomial << std::endl;
			}
			continue;
		}
		// ***********************************************************************
		// ************ (4) compose divisor candidates and make trial division ***
		// ***********************************************************************
		std::cout << "Vieta's formula check  ngrp1=" << ngrp1 <<  std::endl;
		for(i=1; i<=nexist; i++){
			if(included[i] >= 1) std::cout << "included[" << i << "]=" << included[i] << " eigvr[i]=" << (double)eigvr[i] << std::endl;
		}
		d=2; noff=0; reset=0;
#ifdef FACTORIZEONLY
		while(d <= nsize/2){                // search for polynomial of degree 2 to nsize/2
#else
		while(d <= nsize){                  // search for polynomial of degree 2 to nsize
#endif
			if(d>nsize/2 && d != nsize){ d++; continue;}
			tstart0=gettime();
			std::cout << std::endl << "Divisor search for degree " << d << " starts." << std::endl;
			if(reset==0){
			//*****************************************************************************
			//(*) refine accuracy of eigenvalues according to d, i.e., degree of polynomial
			//*****************************************************************************
				tstartr=gettime();
				for(i=1; i<=nexist; i++){
					if(included[i]>=1){
						log2ev=log2(evlarge)+1;
						reqndeig=((int)log2ev+2)*d+accdouble; // <==== Required number of digits for the eigenvalue
						double stmp=pow(2,log2ev*d);
						double epstmp=1/stmp;
						std::cout << std::endl << "required accuracy i=" << i << " for eigval=" << eigvl[i] << " log2ev=" << log2ev << " d=" << d << " epstmp=" << epstmp << " reqndeig=" << reqndeig << " current interval=" << (double)(eigint[i].upper()-eigint[i].lower()) << std::endl;
						int iwas = chkonce(nsize,p,eigint[i].lower(),eigint[i].upper(),faeig,fbeig);
						if(iwas==0){
							std::cout << "chkonce ERROR then STOP" << std::endl;
							std::cout << "  Refine accuracy of eigval" <<  " Width of interval=" << (double)(eigint[i].upper()-eigint[i].lower()) << " faeig=" << (double)faeig << " fbeig=" << (double)fbeig << std::endl; 
							exit(3);
						}
						std::cout << " calling bisect with epstmp=" << epstmp << " reqndeig=" << reqndeig  <<  std::endl; 
#ifdef BISECT
						int irtn=bisect(p,nsize,epstmp,reqndeig,eigint[i],faeig,fbeig,eigvr[i]);
#else
						std::cout << "  CALL bisectrf reqndeig=" << reqndeig << std::endl; 
						int irtn=bisectrf(p,nsize,epstmp,reqndeig,eigint[i],faeig,fbeig,eigvr[i]);
#endif
						if(irtn < 0){
							std::cout << " bisect not converged STOP" << std::endl; exit(3);
						}
						if(irtn==0){
//							irtn=reqndeig;
							std::cout << " bisect encountered rational number solution" << std::endl;
						}
						std::cout << " Refined accuracy of eigval eigvr=" << (double)eigvr[i] << " Width of interval=" << (double)(eigint[i].upper()-eigint[i].lower()) << std::endl; 
						formatprn(eigint[i]);
					}    // if(included[i]>=1){
				}	    //  for(i=1; i<=nexist; i++){
			}	           //   if(reset==0){
			ttp=gettime()-tstartr;
			std::cout << " Refine time=" << (double)(ttp)/1000000.0 << "(sec)" << std::endl;
			tstart2+=ttp;
			// *****  set candidates of eigenvalues in ercan from eigvr  ************* 
			ncandi=setecanint(eigvr,eigint,nexist,included,ercan,ercanint,dfcand,usev);
			std::cout << "combination loop for " << d << " refine a, time=" << (double)(gettime()-tstart0)/1000000.0 << "(sec)" << std::endl;
			for(i=1; i<d;i++) idx[i]=i+noff; idx[d]=d+noff-1;        // initialize for nxtcmb
			nn=comb(ncandi,d); reset=0;   // since nn is type "long long", ngrp1-nuse should be "< 62"
			std::cout <<  std::endl  << "comb(" << ncandi << "," << d << ")=" << nn << std::endl << std::endl;
			if(nok != 0) std::cout <<  std::endl  << "reset comb(" << ncandi << "," << d << ")=" << nn << std::endl << std::endl;
			std::cout << "Vieta's formula check for d=" << d << " nexist=" << nexist << " ncandi=" << ncandi << std::endl;
			for(ii=1; ii<=nn;ii++){             // loop over nn combinations of candidates
				nxtcmb(ncandi,d,idx);       // select one of _(ncan-nused) C_d candidates
				for(j=1; j<=d; j++) figdaf[j]=dfcand[idx[j]]; 
				double ud=figdaf[1];
				for(j=2; j<=d; j++) ud=ud+figdaf[j];
				if(askinty(ud,epsint)){;
					for(j=1; j<=d; j++){ 
						figvr[j]=ercan[idx[j]]; figv[j]=ercanint[idx[j]];
#ifdef FACTORCOMP
						longint ad=figv[j].lower().denominator(); longint an=figv[j].lower().numerator();//interval
						longint bd=figv[j].upper().denominator(); longint bn=figv[j].upper().numerator();//interval
						std::cout << "system'factor " << ad  << "';" << std::endl;
						std::cout << "system'factor " << an  <<  "';" << std::endl;
						std::cout << "system'factor " << bd  <<  "';" << std::endl;
						std::cout << "system'factor " << bn  <<  "';" << std::endl;
#endif
					}
					ok=1; nok++;   // increment number of ok
					// *************************************
					tstart1=gettime();
					ok=vietatermschk(figv, d, cor);
					double vtime= (double)(gettime()-tstart1)/1000000.0;
					if(vtime>1) std::cout << "return from vietatermschk time=" <<  (double)(gettime()-tstart1)/1000000.0 << "(sec)" << std::endl;
					if(ok){        // OK, try trial division
						std::cout << " polynomial of degree " << d << " all coefficients seem to be integer" << std::endl;
						for(int r=1; r<=d; r++) vrat[d-r]=cor[r];
						vrat[d]=rational::ONE;
						std::cout << " Trial division"  << std::endl;
						for (j=0; j<=nsize; j++) urat[nsize-j]=p[j];
						polytexformintr(urat,nsize);
						isit=polydivchk(urat,nsize,vrat,d,qrat,rrat);  // trial division
						std::cout << " Polydivchk rtnval=" << isit << std::endl;
						if(isit==0) std::cout << std::endl;
						if(isit){;                            // trial division divided up
							std::cout << "%polynomial degree " << d << " divided up" << std::endl;
							for (j=0; j<=nsize-d; j++) p[nsize-d-j]=qrat[j]; // set quatient for next p(\lambda)
							nsize = nsize-d;                                 // and decrease nsize
							std::cout << " ii=" << ii << " Divisor = " << std::endl; polytexformr(vrat,d);
							for (j=1; j<=d; j++) std::cout << " group is " << usev[idx[j]] << ", eigval=" << (double)figvr[j] << std::endl; 
							npolynomial=putpolynomial(d,vrat,npolynomial,ppolynomial,cpolynomial);
							std::cout << "replaced. npolynomial=" << npolynomial << std::endl << std::endl;
							for (j=1; j<=d; j++)  included[usev[idx[j]]]=0;
							noff=idx[1]-1;  // number of OFFed indecies
							if(d+noff>ncandi) break;
							reset=1; break;
//							/********************/
						}    // if(isit) 
					}           //  if(ok){
				}	           //   if(askinty(u,eps,dummy))
			}                         //    for(ii=1; ii<=nn;ii++){
			if(reset==0){
				std::cout << "%combination loop for " << d << " complete, time=" << (double)(gettime()-tstart0)/1000000.0 << "(sec)" << std::endl;
				d++; noff=0;
			}
		}                               //      while(d<=nsize){
		if(nsize>0){
			if(p[0]==rational::ONE){
				for (i=0; i<=nsize; i++)  urat[nsize-i] =p[i];
			}else{
				for (i=0; i<=nsize; i++)  urat[nsize-i] =-p[i];
			}
			npolynomial=putpolynomial(nsize,urat,npolynomial,ppolynomial,cpolynomial);
			std::cout << "added p() and replace npolynomial=" << npolynomial << std::endl;
		}
		std::cout << std::endl;
		std::cout << "END of k =" << k << std::endl;
		std::cout << "PRINT current divisors found npolynomial=" << npolynomial << std::endl;
		for(int iod=1; iod<=npolynomial; iod++){
			int ksz=getpolynomial(iod,vrat,npolynomial,ppolynomial,cpolynomial);
			std::cout << "%" << std::endl;
			if(k==0){
				polytexformr2(vrat,ksz,rational::ONE);
			}else{
				polytexformintr(vrat,ksz);
			}
		}
		std::cout << "PRINT current divisors found END" << std::endl;
	}         //                                    for(k=nmat; k>=0; k--){
}

int setecanint(const rat::vector<rational>& eigvr, const vector<interval>& eab, const int ncan, const int included[],
		rat::vector<rational>& ercan, rat::vector<interval>& ercanab, vector<double>& dfcand, int usev[])
{ 
	int j=0;
	for(int i=1; i<=ncan;i++){     
		if(included[i]){                           // exclude eigvl[i] according to included[i]
			j++; 
			usev[j]=i; 
			ercan[j]=eigvr[i];
			ercanab[j]=eab[i]; 
			rational tmp=eab[i].upper();
			dfcand[j]=(double)(tmp-tmp.floor()); // extract 53 bits under the point of eab[i].upper()
		};
	}
	return j;    // return number of candidates set in array evcan[1] to [j]
}

void selectev(const vector<double>& a, int n, int& ngrp, vector<double>& eigvl, vector<double>& eigvh, rat::vector<int>& mult, double& am) // am in dummy arguments
{
	int i,ii,j=0;
	double x;
	am=fabs((a[n-1]-a[0])/((double)n));  // abs(mean) 
	std::cout << "selectev Amean=" << am  << std::endl;
	eigvl[j+1]=a[0]; eigvh[j+1]=0; x=a[0]/am; ii=0; mult[j+1]=0;
	for(i=1; i < n; i++){
		if(fabs(a[i]/am-x) > eps){
			j++; x = a[i]/am; eigvl[j+1]=a[i]; ii=i; mult[j+1]=0;
		}
		if(ii != i){
			eigvh[j+1]=a[i];     // higher eigval of jth group 
			mult[j+1]++;
		}
	}
	ngrp = j;
}

void nmhes(rat::matrix<rational>& a, int n, rat::vector<int>& ith, int& nmat)
{
	rational x, y;
	int j=1;
	std::cout << "nmhes  start n=" << n  << std::endl;
	for (int i=2; i <= n; ++i) {
		if (a[i-1][i-2] == rational::ZERO) {
			ith[j-1] = i-2;
			j++;
		}
	}
	ith[j-1] = n-1;
	nmat = j-1;
}

void cpsubmat(rat::matrix<rational>& a, int n, rat::matrix<rational>& b, int i1, int i2)
{
	int nsize = i2-i1+1;
	for (int j=0; j < nsize; ++j) {
		for (int i=0; i < nsize; ++i) b[i][j] = a[i+i1][j+i1];
	}
}

int chkonce(const int n, const rat::vector<rational>& p, const rational a, const rational b, rational& fa, rational& fb)
{
	int rtnval=1;
	fa=Horner(n,p,a); fb=Horner(n,p,b);
	if(fa*fb > rational::ZERO) rtnval=0;
	return rtnval;
}

int chkinc(const int n, const rat::vector<rational>& p, const double e, const double h, const int mult, const double am, rational& a, rational& b)
{
	double ta,tb;
	int rtnval=0;
	rational fa,fb;
	
	if(fabs(e/am) > eps){
		ta=e-fabs(e)*eps; tb=h+fabs(h)*eps;
		if(mult==0) tb=e+fabs(e)*eps;
		a = Rdset(&ta); fa=Horner(n,p,a);
		b = Rdset(&tb); fb=Horner(n,p,b);
		if(fa*fb<rational::ZERO) rtnval=1;
	}else{
		ta=e-eps; tb=h+eps;   // consider eigenvalues close to zero 
		if(mult==0) tb=e+eps;
		a = Rdset(&ta); fa=Horner(n,p,a);
		b = Rdset(&tb); fb=Horner(n,p,b);
		if(fa*fb<rational::ZERO){ 
			rtnval=1;
		}else{
			if(mult>0){
				ta=2*ta; tb=2*tb;
				a = Rdset(&ta); fa=Horner(n,p,a);
				b = Rdset(&tb); fb=Horner(n,p,b);
				std::cout << "chkink interval inflation ta=" << ta << " fa=" << (double)fa << " tb=" << tb << " fb=" << (double)fb << std::endl;
				if(fa*fb<rational::ZERO) rtnval=1;
			}
		}
	}
	return rtnval;
}

int srchroot(int n, const rat::vector<rational>& p, rational& a, rational& fa, rational& b, rational& fb, const double eps, const int maxit, vector<interval>& abtemp, int& ifound)
{
	rational c;
	std::cout << "srchroot called ifound=" << ifound << std::endl;
	int rtnbisect1 = bisectrf(p,n,eps,maxit,a,fa,b,fb,c);
	if(rtnbisect1 != 0){
		abtemp[ifound]=interval(a,b);
		std::cout << " rtn bisect1=" << rtnbisect1 << " c=" << (double)c << " the first lambda=" << std::endl;
		std::cout << "    "; formatprn(abtemp[ifound]);
		ifound++;
	}else{ std::cout << " srchroot rtnbisect1=" << rtnbisect1 << " ifound=" << ifound << " STOP" << std::endl; exit(3); }
	return rtnbisect1;
}

int countinc(int n, const rat::vector<rational>& p, rational& a, rational& b, const int mult,  const double eps, const int maxitarg, vector<interval>& abtemp)
{
	rational fa,fb,fda,fdb,fdda,fddb,c,fc,fdc,fddc,a0,b0,fa0,fb0,fda0,fdb0,a1,b1,fa1,fb1,fda1,fdb1,c1,c2;
	rational a2,fa2,b2,fb2,fda2,fdb2; 
	int rtnval=0, rtnbisect1, rtnbisect2, ifound=0;
	double dtol, dzero=0;
	int maxit=maxitarg;
	if(maxit <= 0) maxit=2;
	std::cout << " countinc mult=" << mult <<  " a=" << a << " b=" << b << " maxit=" << maxit << " eps=" << eps << std::endl;
	Horner3(n,p,a,fa,fda,fdda); 
	Horner3(n,p,b,fb,fdb,fddb);
	a0=a; fa0=fa; fda0=fda; b0=b; fb0=fb; fdb0=fdb; // <---------- save original interval
	// ***************************
	// ***** mult = 0  ***********
	// ***************************
	if(mult == 0){
		rtnval=0;
		if(fa*fb <= rational::ZERO){
			int maxitarg=maxit;
			if(log2((double)(b-a))/eps > maxit) maxitarg=log2((double)(b-a))/eps; 
			rtnbisect1 = bisectrf(p,n,eps,maxitarg,a,fa,b,fb,c);
			if(rtnbisect1>=0){
				abtemp[0]=interval(a,b); rtnval=1; //  <--------  find first root and continue 
				std::cout << "mult=0 case, The first lambda=" << std::endl;
				std::cout << "    "; 
				formatprn(abtemp[0]);
			}else{ std::cout << "Fail in bisect, STOP" << std::endl; exit(3); }
		}else{
			std::cout << "countinc inclusion does not exist" << std::endl;
		}
		return rtnval;
	}  // if(mult == 0){
	/**********************************************************/
	if(mult>=1){
		std::cout << " After first Horner3 a=" << std::setprecision(16) << (double)a << " b=" << (double)b << std::endl;
		std::cout << " it=0 fa=" << sub3(fa) << " fb=" << sub3(fb) << std::endl;
		std::cout << "     fda=" << sub3(fda) << " fdb=" << sub3(fdb) << std::endl;
		std::cout << "    fdda=" << sub3(fdda) << " fddb=" << sub3(fddb) << std::endl;
	}
	/*****************************/
	/******* mult = 1  ***********/
	/*****************************/
	if(mult == 1){
		if( fa*fb > rational::ZERO &&  fda*fdb > rational::ZERO ){  /* no root exists */
			std::cout << "countinc no root for mult=1"  << std::endl;
			return 0;
		}
		/************  searching only one root  *******************/
		if( fa*fb < rational::ZERO  && fda*fdb > rational::ZERO){
			dtol=(double)(b-a)/(pow(2,maxit));
			rtnbisect1=bisectrf(p,n,dtol,maxit,a,fa,b,fb,c);
			if(rtnbisect1 != 0){
				abtemp[0]=interval(a,b); ifound=1; //  <----------  find first root and return 
				std::cout << "countinc return from bisect rtnval=" << ifound << " c=" << (double)c  << " The first lambda=" << std::endl;
				std::cout << "    "; formatprn(abtemp[0]);
				return 1;    //   <---------- find only one root then return
			}else{ std::cout << "countinc return from bisect rtnval=" << ifound << " STOP" << std::endl; exit(3); }
		}
		/************  start searching two roots  *******************/
		if(fa*fb > rational::ZERO && fda*fdb <= rational::ZERO && fa*fda < rational::ZERO){
			int maxitarg=maxit;
			if(log2((double)(b-a))/eps > maxit) maxitarg=log2((double)(b-a))/eps; 
			rtnbisect2 = bisect2(p,n,maxitarg,a,fa,fda,b,fb,fdb);
			if(rtnbisect2<0) {
				std::cout << " rtnbisect2=" << rtnbisect2 << std::endl;
			}else{
				a1=a; b1=b; fa1=fa; fb1=fb; fda1=fda; fdb1=fdb;  // <----------  save interval after bisect2
				rtnbisect1 = srchroot(n,p,a,fa,b,fb,dzero,2,abtemp,ifound);  // abtemp[0]=root
				if(fa0*fa1<rational::ZERO){ a=a0;b=a1;fa=fa0;fb=fa1; } //  set interval(a0,a1)
				if(fb1*fb0<rational::ZERO){ a=b1;b=b0;fa=fb1;fb=fb0; } //  set interval(b1,b0)
				rtnbisect1 = srchroot(n,p,a,fa,b,fb,dzero,2,abtemp,ifound);  // abtemp[1]=root
			}
			return 2;
		}
	}  // if(mult == 1){
	/**********************************************************/
	/******* mult = 2, i.e., three roots may exist  ***********/
	/**********************************************************/
	if(mult == 2){
		std::cout << " mult==2"  << std::endl;
		if(fa*fb > rational::ZERO){  // three roots do not exist, but two may exist.
			if(fda*fdb < rational::ZERO && fa*fda < rational::ZERO){ //  two roots may exist.
				std::cout << " calls bisect2 a=" << (double)a << " b=" << (double)b << std::endl;
				rtnbisect2 = bisect2(p,n,maxit+10,a,fa,fda,b,fb,fdb);
				if(rtnbisect2<0) {
					std::cout << " bisect2 does not converge rtnval=" << rtnbisect2 << std::endl; exit(3);
				}
				std::cout << " save interval a1=" << (double)a << " b1=" << (double)b <<  " fa1=" << (double)fa << " fb1=" << (double)fb << std::endl;
				a1=a; fa1=fa; b1=b; fb1=fb;  //  <-----------  save interval after bisect2
				rtnbisect1 = srchroot(n,p,a,fa,b,fb,dzero,2,abtemp,ifound);  // abtemp[0]=root
				if(fa0*fa1<rational::ZERO){ a=a0;b=a1;fa=fa0;fb=fa1; } // set a,fa,b,fb
				if(fb1*fb0<rational::ZERO){ a=b1;b=b0;fa=fb1;fb=fb0; }
				rtnbisect1 = srchroot(n,p,a,fa,b,fb,dzero,2,abtemp,ifound);  // abtemp[1]=second root
			}
			return ifound;
		}else{
		// ********************************************************
//		CASE of (fa*fb < 0) // one or three roots may exist.
		// **********  searching only one root  *******************
			std::cout << " fa="  << sub3(fa) << " fb=" << sub3(fb) << std::endl;
			if( fdda*fddb > rational::ZERO){  // cannot call bisect3
				rtnbisect1 = srchroot(n,p,a,fa,b,fb,dzero,2,abtemp,ifound);  // abtemp[0]=root
				return ifound;   //   <---------- find only one root then return
			}else{
		// **********  searching three roots   *******************
				int rtnbisect3 = bisect3(p,n,maxit,a,fa,fda,fdda,b,fb,fdb,fddb);
				if(rtnbisect3<0) { std::cout << " bisect3 NOT coverged rtnval=" << rtnval << " STOP" << std::endl; exit(3); }
				a1=a; b1=b; fa1=fa; fb1=fb; fda1=fda; fdb1=fdb;  // <------  save interval after bisect3
				std::cout << " return bisect3 fa="  << sub3(fa) << " fb=" << sub3(fb) <<  " fda="  << sub3(fda) << " fdb=" << sub3(fdb) << std::endl;
				////////// sign(fa0,fa1,fb1,fb0)=(+ - + -) ||  sign(fa0,fa1,fb1,fb0)=(- + - +) ///////////////////////////
				if((fa0>rational::ZERO && fa1<rational::ZERO && fb1>rational::ZERO) || (fa0<rational::ZERO && fa1>rational::ZERO && fb1<rational::ZERO)){
					std::cout << "  sign(fa0,fa1,fb1,fb0)=(+ - + -) or (- + - +)" << std::endl;
					a=a0;b=a1;fa=fa0;fb=fa1;   // set (a0,a1)
					rtnbisect1 = srchroot(n,p,a,fa,b,fb,dzero,2,abtemp,ifound);  // abtemp[0]=root
					a=a1;b=b1;fa=fa1;fb=fb1;   // set (a1,b1)
					rtnbisect1 = srchroot(n,p,a,fa,b,fb,dzero,2,abtemp,ifound);  // abtemp[1]=root
					a=b1;b=b0;fa=fb1;fb=fb0;   // set (b1,b0)
					rtnbisect1 = srchroot(n,p,a,fa,b,fb,dzero,2,abtemp,ifound);  // abtemp[2]=root
				////////// sign(fa0,fa1,fb1,fb0)=(+ + - -) ||  sign(fa0,fa1,fb1,fb0)=(- - + +) ///////////////////////////
				}else if((fa0>rational::ZERO && fa1>rational::ZERO && fb1<rational::ZERO) || (fa0<rational::ZERO && fa1<rational::ZERO && fb1>rational::ZERO)){ 
					std::cout << "  sign(fa0,fa1,fb1,fb0)=(+ + - -) or (- - + +)" << std::endl;
					a=a1;b=b1;fa=fa1;fb=fb1;
					// ************************************************************************
					rtnbisect1 = srchroot(n,p,a,fa,b,fb,dzero,2,abtemp,ifound);  // abtemp[0]=root
					if(fdb1*fdb0>rational::ZERO){
						std::cout << "no additional roots" << std::endl;
						return ifound;
					}
					a=b1;b=b0;fa=fb1;fb=fb0;fda=fdb1;fdb=fdb0;  // search (b1,b0) for two roots
					rtnbisect2 = bisect2(p,n,maxit+10,a,fa,fda,b,fb,fdb);
					if(rtnbisect2<0) {
						std::cout << " rtnbisect2=" << rtnbisect2 << " ?? " << std::endl; // ?
					}else{
						a2=a; b2=b; fa2=fa; fb2=fb; fda2=fda; fdb2=fdb;  // <----------  save interval after bisect2
						rtnbisect1 = srchroot(n,p,a,fa,b,fb,dzero,2,abtemp,ifound);  // abtemp[1]=root
						if(fb2*fb0<rational::ZERO){ a=b2;b=b0;fa=fb2;fb=fb0; } //  search interval(b2,b0)
						if(fb1*fa2<rational::ZERO){ a=b1;b=a2;fa=fb1;fb=fa2; } //  search interval(b1,a2)
						rtnbisect1 = srchroot(n,p,a,fa,b,fb,dzero,2,abtemp,ifound);  // abtemp[2]=second root
					}
				////////// sign(fa0,fa1,fb1,fb0)=(+ - - -) ||  sign(fa0,fa1,fb1,fb0)=(- + + +),  two roots may exist in (a1,b1)
				}else if((fa0>rational::ZERO && fa1<rational::ZERO &&  fb1<rational::ZERO) || (fa0<rational::ZERO && fa1>rational::ZERO &&  fb1>rational::ZERO)){
					std::cout << "  sign(fa0,fa1,fb1,fb0)=(+ - - -) or (- + + +)" << std::endl;
					if((fa0>rational::ZERO && fdb1<rational::ZERO)||(fa0<rational::ZERO && fdb1>rational::ZERO)){ // two roots ?
						a=a1;b=b1;fa=fa1;fb=fb1;fda=fda1;fdb=fdb1;  // search (a1,b1) for two roots
						rtnbisect2 = bisect2(p,n,maxit+10,a,fa,fda,b,fb,fdb);
						if(rtnbisect2<0) {
							std::cout << " rtnbisect2=" << rtnbisect2 << " ?? " << std::endl;
						}else{
							a2=a; b2=b; fa2=fa; fb2=fb; fda2=fda; fdb2=fdb;  // <-----------  save interval after bisect2
							rtnbisect1 = srchroot(n,p,a,fa,b,fb,dzero,2,abtemp,ifound);  // abtemp[0]=root
							if(fa1*fa2<rational::ZERO){ a=a1;b=a2;fa=fa1;fb=fa2;} //  set interval(a1,a2)
							if(fb2*fb1<rational::ZERO){ a=b2;b=b1;fa=fb2;fb=fb1;} //  set interval(b2,b1)
							rtnbisect1 = srchroot(n,p,a,fa,b,fb,dzero,2,abtemp,ifound);  // abtemp[1]=root
						}
					} else { //  fb1>0 && fdb1<0  means two roots may exist in (b1,b0)
						a=b1;b=b0;fa=fb1;fb=fb0;fda=fdb1;fdb=fdb0;  // search (b1,b0) for two roots
						rtnbisect2 = bisect2(p,n,maxit+10,a,fa,fda,b,fb,fdb);
						if(rtnbisect2<0) {
							std::cout << " rtnbisect2=" << rtnbisect2 << " ?? " << std::endl;
						}else{
							a2=a; b2=b; fa2=fa; fb2=fb; fda2=fda; fdb2=fdb; // <-----  save interval after bisect2
							rtnbisect1 = srchroot(n,p,a,fa,b,fb,dzero,2,abtemp,ifound);  // abtemp[0]=root
							if(fa1*fa2<rational::ZERO){ a=a1;b=a2;fa=fa1;fb=fa2; } //  set interval(a1,a2)
							if(fb2*fb0<rational::ZERO){ a=b2;b=b0;fa=fb2;fb=fb0; } //  set interval(b1,b0)
							rtnbisect1 = srchroot(n,p,a,fa,b,fb,dzero,2,abtemp,ifound);  // abtemp[1]=root
						}
					}
					a=a0;b=a1;fa=fa0;fb=fa1;
					rtnbisect1 = srchroot(n,p,a,fa,b,fb,dzero,2,abtemp,ifound);  // abtemp[2]=root
				////////// sign(fa0,fa1,fb1,fb0)=(+ + + -) ||  sign(fa0,fa1,fb1,fb0)=(- - - +) //////////////////////
				}else{
					std::cout << "  sign(fa0,fa1,fb1,fb0)=(+ + + -) or (- - - +)" << std::endl;
					if((fa0>rational::ZERO && fdb1>rational::ZERO)||(fa0<rational::ZERO && fdb1<rational::ZERO)){ // two roots exist in (a1,b1)
						a=a1;b=b1;fa=fa1;fb=fb1;fda=fda1;fdb=fdb1;  // search (a1,b1) for two roots
						rtnbisect2 = bisect2(p,n,maxit+10,a,fa,fda,b,fb,fdb);
						if(rtnbisect2<0) {
							std::cout << " rtnbisect2=" << rtnbisect2 << " ?? " << std::endl;
						}else{
							a2=a; b2=b; fa2=fa; fb2=fb; fda2=fda; fdb2=fdb;  // <-----  save interval after bisect2
							rtnbisect1 = srchroot(n,p,a,fa,b,fb,dzero,2,abtemp,ifound);  // abtemp[0]=root
							if(fa1*fa2<rational::ZERO){ a=a1;b=a2;fa=fa1;fb=fa2;} //  set interval(a1,a2)
							if(fb2*fb1<rational::ZERO){ a=b2;b=b1;fa=fb2;fb=fb1;} //  set interval(b2,b1)
							rtnbisect1 = srchroot(n,p,a,fa,b,fb,dzero,2,abtemp,ifound);  // abtemp[1]=root
						}
					} else { //  fb1>0 && fdb1<0  means two roots may exist in (a0,a1)
						a=a0;b=a1;fa=fa0;fb=fa1;fda=fda0;fdb=fda1;  // search (a0,a1) for two roots
						rtnbisect2 = bisect2(p,n,maxit+10,a,fa,fda,b,fb,fdb);
						if(rtnbisect2<0) {
							std::cout << " rtnbisect2=" << rtnbisect2 << " ?? " << std::endl;
						}else{
							a2=a; b2=b; fa2=fa; fb2=fb; fda2=fda; fdb2=fdb; // <------  save interval after bisect2
							rtnbisect1 = srchroot(n,p,a,fa,b,fb,dzero,2,abtemp,ifound);  // abtemp[0]=root
							if(fa0*fa2<rational::ZERO){ a=a0;b=a2;fa=fa0;fb=fa2; } //  set interval(a0,a2)
							if(fb2*fa1<rational::ZERO){ a=b2;b=a1;fa=fb2;fb=fa1; } //  set interval(b2,a1)
							rtnbisect1 = srchroot(n,p,a,fa,b,fb,dzero,2,abtemp,ifound);  // abtemp[1]=root
						}
					}
					a=b1;b=b0;fa=fb1;fb=fb0; //  set interval(b1,b0)
					rtnbisect1 = srchroot(n,p,a,fa,b,fb,dzero,2,abtemp,ifound);  // abtemp[2]=root
				} //if((fa0 > rational::ZERO &&  )||(fda{     //   fb0<0, fda1*fdb1<0, fdda1*fddb1<0 are assumed
			} // if( fdda*fddb > rational::ZERO){ .. }else{
		} // if( fa*fb > rational::ZERO){ .. }else{ i.e., fa*fb<0
		return ifound;
	} // if(mult == 2){
}


int bisect2(const rat::vector<rational>& p, const int n, const int maxit, rational& a, rational& fa, rational& fda, rational& b, rational& fb, rational& fdb)
{
	rational c, fc, fdc, fddc;
	int it=0, rtnval=0;

	std::cout << " Enter bisect2 a=" << sub3(a) << " b=" << sub3(b) << std::endl;
	while( it < maxit+2){
		if(fda*fdb > rational::ZERO){
			std::cout << " bisect2 error  a=" << sub3(a) << " b=" << sub3(b) << " fda=" << sub3(fda) <<  " fdb=" << sub3(fdb) <<  std::endl;
		}
		it++;
		if(it >= maxit){
			std::cout << " bisect2 MAXIT reached it=" << it << std::endl;
			rtnval=-it; break;
		}
		c = (a + b) / rational::TWO; Horner3(n,p,c,fc,fdc,fddc);
		if(fda*fdc > rational::ZERO){
			a=c; fa=fc; fda=fdc;
		}else{
			b=c; fb=fc; fdb=fdc;
		}
		if(fa*fb <=  rational::ZERO){
			std::cout << " bisect2 converged  a=" << sub3(a) << " b=" << sub3(b) << " interval=" << sub3(b-a) << " fa=" << sub3(fa) << " fb=" << sub3(fb) << " fda=" << sub3(fda) <<  " fdb=" << sub3(fdb) <<  std::endl;
			rtnval=it; break;
		}
	}
	std::cout << " bisect2 rtnval=" << rtnval << std::endl;
	return rtnval;
}

int bisect3(const rat::vector<rational>& p, const int n, const int maxit, rational& a, rational& fa, rational& fda, rational& fdda, rational& b, rational& fb, rational& fdb, rational& fddb)
{
	rational c, fc, fdc, fddc;
	int it=0, rtnval;

	std::cout << " Enter bisect3 a=" << std::setprecision(16) << (double)a << " b=" << (double)b << std::endl;
	while( it < maxit){
		if(fdda*fddb > rational::ZERO){
			std::cout << " bisect3 error  a=" << sub3(a) << " b=" << sub3(b) << " fdda=" << sub3(fdda) <<  " fddb=" << sub3(fddb) <<  std::endl;
		}
		it++;
		if(it >= maxit){
			std::cout << " bisect3 MAXIT reached it=" << it << std::endl;
			rtnval=-it; break;
		}
		c = (a + b) / rational::TWO; Horner3(n,p,c,fc,fdc,fddc);
		if(fdda*fddc > rational::ZERO){
			a=c; fa=fc; fda=fdc; fdda=fddc;
		}else{
			b=c; fb=fc; fdb=fdc; fddb=fddc;
		}
		if(it%10==0) std::cout << "bisecct3 it=" << it << " a=" <<  sub3(a) << " b=" << sub3(b) << " c=" << sub3(c) <<  std::endl;
		if(fda*fdb <=  rational::ZERO){
			std::cout << " bisect3 converged  a=" << sub3(a) << " b=" << sub3(b) <<  " fa=" << sub3(fa) << " fb=" << sub3(fb) << " fda=" << sub3(fda) <<  " fdb=" << sub3(fdb) << " fdda=" << sub3(fdda) <<  " fddb=" << sub3(fddb) <<  std::endl;
			rtnval=it; break;
		}
	}
	return rtnval;
}


int vietatermschk(const vector<interval>& e, const int n, rat::vector<rational>& cor)
{
	int i,r,inty,ok=1;
	rat::vector<int> last(n+1);
	rat::vector<int> num(n+2);
	vector<interval> p(n+1);               //interval
	/*******************************************************************
	std::cout << "vietatermschk called n=" << n << std::endl;
	for(int j=1; j<=n; j++){ 
		rational ta=e[j].lower(); rational tb=e[j].upper();	
		std::cout << "e[" << j <<"]=(" << ta << ","  << tb << ")" << std::endl;
	}
	*******************************************************************/
	for(i=0; i<n+1; i++){ p[i]=interval(rational::ZERO,rational::ZERO);}     //interval 2015may18
	for(i=1; i<n; i++){
		long long tstart0=gettime();
		num[2]=i;
		for(int j=1; j<n; j++) last[j]=0;
		inty=mulvieta(num,e[i],n,2,e,p,last,1);
		if(inty==0){ ok=0; break;}
		std::cout << "loop in vietatermschk i=" << i << " time=" << (double)(gettime()-tstart0)/1000000.0 << "(sec)" << std::endl;
	}
	if(ok==1){
		interval t=e[1];	
		for(r=2; r<=n; r++){ 
			interval er=(e[r]);	
			t=t+er;	
		}
		if((t.upper()).floor() == (t.lower()).ceil()){
			cor[1]=-(t.upper().floor());
		}else{
			ok=0;
		}
//		std::cout << "first order term=" << (double)cor[1] << std::endl;
	}
	if(ok==1){
		for(r=2; r<=n; r++){
			if(p[r].upper().floor() == p[r].lower().ceil()){
				if(r%2 == 1){
					cor[r]=-p[r].upper().floor();
				}else{
					cor[r]=p[r].upper().floor();
				}
			}else{
				ok=0; break; 
			}
//			std::cout << "r order term=" << (double)cor[r] << std::endl;
		}
	}
	return ok;
}

int mulvieta(rat::vector<int>& num,const interval& sp,const int n,const int r,const vector<interval>& e, vector<interval>& p,rat::vector<int>& last,int dir) 
{
	int dgt, dgt1;
	interval t;
	int rtnval=1;
	dgt=num[r];
	if(dir == 1){
		dgt1=dgt+1;
	}else{
		dgt1=last[r]+1;
	}
	t=sp*e[dgt1];      
	p[r]=p[r]+t;
	last[r]=dgt1;

	if(r == n){	             // check if the final term 123...n case
		if(p[r].upper().floor() != p[r].lower().ceil()){   //interval
			rtnval=0;
		}
	}
	if(dgt1 < n){
		num[r+1]=dgt1;
		int inty=mulvieta(num,t,n,r+1,e,p,last,1); 
		if(inty == 1){
			if(dgt1 <n){
				mulvieta(num,sp,n,r,e,p,last,0);
			}
		}else{
			rtnval=0;
		}
	}
	return rtnval;
}
