/*
 * interval.cpp
 *
 *  Created on: 2015/06/16
 *      Author: Osamu
 *  Modified on: 2015/06/30 by Samukawa
 *  Modified on: 2015/08/18 by Samukawa  (interval)*(rational), (interval)+(rational), inv
 *  Modified on: 2016/05/10 by Samukawa  format, formatprn 
 *  Modified on: 2016/05/13 by Samukawa  log10 approxmation
 *  Modified on: 2016/06/15 by Samukawa  log10int
 *  Modified on: 2016/06/24 by Samukawa  operator-
 *  Modified on: 2016/07/11 by Samukawa  formatprn
 *  Modified on: 2016/09/03 by Samukawa  rough (log10int)
 */
//#include <iostream>
#include <iomanip> // setprecision
#include <math.h>
#include "rmatrix.h"
#include "rvector.h"
#include "longint.h"
#include "rough.h"
#include "numeric_cast.h"
//#include "interval.h"
//#include "defferedmultiply.h"

//#define OLDFORMATPRN
//#define BYLOG10
//namespace rat {

using namespace rat;

interval::interval(const interval& b) {  //constructor
	this->l_=b.l_; this->u_=b.u_;
//	std::cout << "constructor this=(" << this->l_ << " " << this->u_ << ")" << std::endl;
}

interval::~interval() { //destructor
}

interval& interval::operator=(interval b) {
	std::swap(l_,b.l_);
	std::swap(u_,b.u_);
	return *this;
}
interval interval::operator+(interval const& b) const {
	interval res(this->l_ + b.l_, this->u_ + b.u_);
	return res;
}
interval interval::operator+(rational const& b) const {
	interval res(this->l_ + b, this->u_ + b);
	return res;
}

interval interval::operator-(interval const& b) const {
	interval res(this->l_ - b.u_, this->u_ - b.l_);
	return res;
}

interval interval::operator*(rational const& y) const {
	rational za,zb;
	rational xa=this->l_; rational xb=this->u_;
	if(xa.s != xb.s){
		std::cout << "interval::operator* this sign ERROR STOP" << std::endl;
		exit(3);
	}
	if(y > rational::ZERO){
		za = xa * y;
		zb = xb * y;
	} else {
		zb = xa * y;
		za = xb * y;
	}
	interval res(za,zb);
	return res;
}

interval interval::operator*(interval const& b) const {
	rational ua,ub,za,zb;
	rational xa=this->l_; rational xb=this->u_;
	rational ya=b.l_;     rational yb=b.u_;
	int xsgn, ysgn;
	if(xa.s != xb.s){
		std::cout << "interval::operator* this sign ERROR STOP" << std::endl;
		exit(3);
	}
	if(ya.s != yb.s){
		std::cout << "interval::operator* b sign ERROR STOP" << std::endl;
		exit(3);
	}
	if(xa > xb){
		std::cout << "interval::operator* this->l_ larger than u_ ERROR STOP" << std::endl;
		exit(3);
	}
	if(ya > yb){
		std::cout << "interval::operator* b.l_ larger than b.u_ ERROR STOP" << std::endl;
		exit(3);
	}

	xsgn=0;
	if(xa > rational::ZERO){
		xsgn=1;
	}else if(xa < rational::ZERO){
		xsgn=-1;
	}
	if(xsgn == 0){
		if(xb > rational::ZERO) xsgn=1;
	}         // i.e., xsgn==0 means (Xa,Xb)=(0,0)

	ysgn=0;
	if(ya > rational::ZERO){
		ysgn=1;
	}else if(ya < rational::ZERO){
		ysgn=-1;
	}
	if(ysgn == 0){
		if(yb > rational::ZERO) ysgn=1;
	}         // i.e., ysgn==0 means (Ya,Yb)=(0,0)

	if(xsgn*ysgn > 0){  // result +
		ua=xa*ya; ub=xb*yb;
		if( xsgn > 0){
			za=ua; zb=ub;
		}else{
			za=ub; zb=ua;
		}
	}else if(xsgn*ysgn < 0){ //  result -
		ub=xb*ya; ua=xa*yb;
		if( xsgn > 0){
			za=ub; zb=ua;
		}else{
			za=ua; zb=ub;
		}
	}else if(xsgn*ysgn == 0){ //  result 0
		za=rational::ZERO; zb=rational::ZERO;
	}
	interval res(za,zb);
	return res;
}

interval interval::neg(){
	interval res=interval(-this->u_,-this->l_);
	return res;
}

interval interval::inv(){
	longint ln=this->l_.numerator(); longint ld=this->l_.denominator();
	longint un=this->u_.numerator(); longint ud=this->u_.denominator();
	rational b=RRset(ld,ln);  // use RRset to skip GCD calculation
	rational a=RRset(ud,un);
	interval res=interval(a,b);
	return res;
}

bool interval::operator==(interval const& b) const {
	return this->l_ == b.l_ && this->u_ == b.u_;
}

std::ostream& operator<<(std::ostream& o, interval const& a) {
	o << "[";
	o << a.lower();
	o << ":";
	o << a.upper();
	o << "]";
	return o;
}


double log10int(const interval& a)
{
	rational t=a.upper()-a.lower();
//	int nbits; double f=ndpw2(t, nbits);
	rough rt=numeric_cast<rough>(t);
	int nbits=rt.exponent(); double f=rt.significand();
	double dbllgt=log2(f)+(double)nbits;
	return 0.301029995663981195*dbllgt;
}

rational shl10(const rational& x, const int n)
{               // shift left n decimal digits
	rational y;
	if(x.s==0) y.s=0;
	longint xn=x.N;
	longint xd=x.D;
	longint t=pow(longint::TEN,n);
	xn=xn*t;
	y=RRset(xn,xd);
	if(x.s<0) y.s=-1;
	return y;
}
rational shr10(const rational& x, const int n)
{               // shift right n decimal digits
	rational y;
	if(x.s==0) y.s=0;
	longint xn=x.N;
	longint xd=x.D;
	longint t=pow(longint::TEN,n);
	xd=xd*t;
	y=RRset(xn,xd);
	if(x.s<0) y.s=-1;
	return y;
}

rational roundoff(const rational& x, const int n)
{   // shift left n digits, floor(), shift right n digits
	rational y,z,w;
	if(x.s==0) y.s=0;
	longint xn=x.N, xd=x.D;
	longint t=pow(longint::TEN,n); //  t=10^n
	xn=xn*t; y=RRset(xn,xd);       //  shift left
	z=y.rational::floor();         //  floor()
	y=RRset(z.N,z.D*t);            //  shift right
	if(x.s<0) y.s=-1;
	return y;
}

std::string interval::format(const int w, const int d)const {
	std::stringstream ss;
	int lz;
// 
//      xxxx.xxxxxxxxxxxxxxxxxyyyyyyyyyyyyyyyy
//      xxxx.xxxxxxxxxxxxxxxxxzzzzzzzzzzzzzzzz
//          <--- lz --------->
// 
	rational diff = this->upper()-this->lower();  // width of this interval
	if (diff != rational::ZERO) {
		rational t = diff;
		double dbllgten=log10int(*this);
		lz=-(int)dbllgten;  // count number of leading zero digits after point.
//		std::cout << " debug lz=" <<  lz << " w=" << w  << " d=" << d << std::endl;
	}else{
		ss << "\n";
//		ss << this->lower().format(w,d) << ": (a=b)\n" << this->upper().format(w,d);
		ss << this->lower().format(w,d) << ": (a=b)";
		return ss.str();
	}
	if (lz < d) {
		ss << "\n";
		ss << this->lower().format(w,d) << ":\n" << this->upper().format(w,d);
		return ss.str();
	} else {
		int nplaceup=w-6-d; // recover the number of digits upper than point.
		int dt = lz + 9;    // use (leading zero + 9) for under the point
		int wt = w+lz;
		int k = ndd(dt) + 11; // ndd(dt) is number of digits of XX, 11 is ( XX digits )
		if( d < k ) k = d;
		int w2 = (d - k)/2;   // w2 is digits before ( XX digits )
		int w3 = (d - k - w2);// w3 is digits after ( XX digits )
		if (ndd(dt-w2-w3)+11 < k) ++w2;
		ss << "\n";
		//////////////////////////////////////////////////////////////
 		rational tt0=this->lower();             // get lower rational number of this interval
//		std::cout << " debug tt0.N=" <<  tt0.N << " tt0.D=" << tt0.D << " w3=" << w3 <<  std::endl;
		rational tt3=roundoff(tt0,w2);          // shift left w2 digits, floor(), shift right w2 digits
		ss << tt3.format(nplaceup+6+w2,w2);     // print upper portion
		ss << " (" << dt-w2-w3 << " digits) " ;	// digits to skip are ( xxx digits)  
		tt3=roundoff(tt0,dt-w3);       // shift left dt-w3 digits, floor(), shift right dt-w3 digits
		rational tt1=tt0-tt3;                   // subtract off to get lower w3 digits
		rational tt2=shl10(tt1,dt);             // shift left dt digits
//		tt3=tt2.rational::floor();              // floor()
		longint qt3=tt2.N / tt2.D;              // no rounding-up, i.e., always round-off 
//		ss <<  tt3.N;                           // print lower portion
		ss <<  qt3;                             // print lower portion
		ss << "\n";                             // new line
		//////////////////////////////////////////////////////////////
 		tt0=this->upper();                      // get upper rational number of this interval
		tt3=roundoff(tt0,w2);                   // shift left w2 digits, floor(), shift right w2 digits
//		std::cout << " Debug tt3.N=" <<  tt3.N << " tt3.D=" << tt3.D <<  " w2=" << w2 <<  std::endl;
		ss << tt3.format(nplaceup+6+w2,w2);     // print upper portion
		ss << " (" << dt-w2-w3 << " digits) " ;	// digits to skip are ( xxx digits)  
		tt3=roundoff(tt0,dt-w3);       // shift left dt-w3 digits, floor(), shift right dt-w3 digits
		tt1=tt0-tt3;                            // subtract off to get lower w3 digits
		tt2=shl10(tt1,dt);                      // shift left dt digits
//		tt3=tt2.rational::floor();              // floor()
		qt3=tt2.N / tt2.D;
		ss <<  qt3;                             // print lower portion
		return ss.str();
	}
//	return ss.str();
}

void formatprn(const interval& x){
	int nplaceup,nplacef,aapw2,aapw10,nplace;
	rational a=x.lower(), b=x.upper();
	rational aa=a.abs();
	double ba=(double)(b-a);  //////////////////////////////////////
//	double daa=ndpw2(aa,aapw2); // 2016spt03
	rough raa=numeric_cast<rough>(aa);
	aapw2=raa.exponent(); double daa=raa.significand();
//	double daa10=roughdec(daa,aapw2,aapw10);  // 2016spt03
	double daa10=roughdec(raa,aapw10); // number of decinal digits upper than decimal point 
	if(a != b){
		double dba=log10int(x);
		int nbadgt=-(int)dba;  // decimal digit where difference begins
//		nplacef=nbadgt;
		nplacef=nbadgt+5;   // 2016aug28
//		std::cout << "formatprn nplace=" << " nplacef=" << nplacef << " (doouble)a=" << (double)a << " (doouble)b=" << (double)b << " (double)(b-a)=" << ba << " log10=" << log10(ba)  << std::endl;
		if(nplacef > 50 || nplacef < 0 ) nplacef=50;
	}else{
		nplacef=50;
//		std::cout << "formatprn nplace=" << " nplacef=" << nplacef << " (doouble)a=" << (double)a << " (doouble)b=" << (double)b << " (double)(b-a)=" << ba << std::endl;
	}
	nplaceup=aapw10;
//	std::cout << "Formatprn nplaceup=" << nplaceup << std::endl;
	if(nplaceup > 30 || nplaceup < 0 ) nplaceup=30;
//	std::cout << "Formatprn nplaceup=" << nplaceup << " old up=" << log10((double)b) << " ba=" << ba << std::endl;
	nplace=10+nplaceup+nplacef;
	std::cout << x.format(nplace,nplacef) << std::endl;
	/************************************
	if(nplace<80){
		int margin=80-nplace;
		nplace+=margin;
		nplacef+=margin;
	}
	std::cout << "formatprn nplace=" << nplace << " nplacef=" << nplacef << std::endl;
	std::cout << x.format(nplace,nplacef) << std::endl;
	************************************/
}
//} /* namespace rat */
