/*
 * longint.cpp
 *
 *  Created on: 2011/07/12
 *      Author: goda
 *  Modified on: 2012/12/24
 *  Modified on: 2013/jan/06  ldivide
 *  Modified on: 2013/aug/21  ldivide, switch to robust (bisection) algorithm
 *  Modified on: 2013/oct/27  lgcd2 by O. Gohda
 */
#include "longint.h"
#include <stdio.h>
#include <stdlib.h>
#include <iostream>
#include <sstream>
#include <algorithm>
#include <stdexcept>
#include <cassert>
#include <cstdlib>
#include "mempool.h"
#ifdef TLS
#include "pthread.h"
#endif	// TLS

#define USESHL

long long gettime();


#ifdef TLS
#ifdef MacOS
pthread_key_t longint::_mempool;
#else
__thread mem_pool* longint::_mempool;
#endif
bool longint::pool_initialized = false;
#else	// TLS
mem_pool* longint::_mempool = NULL;
#endif	// TLS
uint64_t longint::BASE = (uint64_t)65536u*(uint64_t)65536u;


long long longint::ldividetime = 0;	/* 2013 Nov 06 */
long longint::ldividecount = 0;     /* 2013 Nov 06 */
long long longint::lgcd2time = 0;   /* 2013 Oct 01 */
long longint::lgcd2count = 0;
long long longint::lgcdtime = 0;
long longint::lgcdcount = 0;        /* 2013 Oct 01 */
long long longint::opsubeqtime = 0; /* 2013 Nov 30 */
long longint::opsubeqcount = 0;     /* 2013 Nov 30 */
long long longint::shrtime = 0;     /* 2013 Nov 30 */
long longint::shrcount = 0;         /* 2013 Nov 30 */
long long longint::lmultime = 0;	/* 2013 Nov 30 */
long longint::lmulcount = 0;		/* 2013 Nov 30 */
long long longint::laddtime = 0;	/* 2013 Nov 30 */
long longint::laddcount = 0;		/* 2013 Nov 30 */

long unsigned longint::ldiv2calls = 0;
long unsigned longint::ldiv2qdiff0 = 0;
long unsigned longint::ldiv2qdiff1 = 0;
long unsigned longint::ldiv2qdiff2 = 0;
#ifdef EPROF
long long longint::lgcdtime = 0;
long long longint::absadtime = 0;
long long longint::absubtime = 0;
long long longint::lcmptime = 0;
long long longint::ldiv2ptime = 0;
long long longint::lmultime = 0;
long long longint::ldividetime = 0;
long long longint::ldivnbittime = 0;
long long longint::lmulnbittime = 0;
#endif	// EPROF

bool longint::_init = init();

const longint longint::ZERO = (uint32_t)0u;
const longint longint::ONE = (uint32_t)1u;
const longint longint::TWO = (uint32_t)2u;
const longint longint::THREE = (uint32_t)3u;
const longint longint::FOUR = (uint32_t)4u;
const longint longint::FIVE = (uint32_t)5u;
const longint longint::SIX = (uint32_t)6u;
const longint longint::TEN = (uint32_t)10u;
const longint longint::SIXTEEN = (uint32_t)16u;
/*********************************************************
const longint longint::M = (uint32_t)1000000u;
const longint longint::MM = M*M;
const longint longint::MMM = MM*MM;
const longint longint::MMMM = MMM*MMM;
const longint longint::MMMMM = MMMM*MMMM;
const longint longint::MMMMMM = MMMMM*MMMMM;
const longint longint::MMMMMMM = MMMMMM*MMMMMM;
const longint longint::MMMMMMMM = MMMMMMM*MMMMMMM;
const longint longint::MMMMMMMMM = MMMMMMMM*MMMMMMMM;
const longint longint::MMMMMMMMMM = MMMMMMMMM*MMMMMMMMM;
const longint longint::MMMMMMMMMMM = MMMMMMMMMM*MMMMMMMMMM;
const longint longint::MMMMMMMMMMMM = MMMMMMMMMMM*MMMMMMMMMMM;
*********************************************************/
const longint longint::TWOGIGA =  (uint32_t)2147483648u;
const longint longint::FOURGIGA = (uint64_t)4294967296u;
const longint longint::FOURPETA = (uint64_t)4503599627370496u;

bool longint::init() {
#ifdef TLS
	if (!pool_initialized) {
#ifdef MacOS
		if (pthread_key_create(&_mempool,0) != 0) {
			std::cerr << "pthread_key_create failed." << std::endl;
			exit(1);
		}
#endif	/* MacOS */
		pool_initialized = true;
	}
#endif	// TLS
	return true;
}

uint32_t* longint::lalloc(int size, int& allocated_size) {
	uint32_t* p;
	try {
		if (size > maxdigits) {
			p = new uint32_t[size];
			allocated_size = size;
		} else {
			mem_pool* mp;
#ifdef TLS
			if (!pool_initialized) {
				longint::init();
			}
#ifdef MacOS
			if ((mp = (mem_pool*)pthread_getspecific(_mempool)) == NULL) {
				mp = new mem_pool();
				pthread_setspecific(_mempool,(void*)mp);
			}
#else
			if ((mp = _mempool) == 0) {
				_mempool = new mem_pool();
				mp = _mempool;
			}
#endif	/* MacOS */
#else	// TLS
			if ((mp = _mempool) == 0) {
				_mempool = new mem_pool();
				mp = _mempool;
			}
#endif	// TLS
			assert(mp != NULL);
			int idx = mp->getpoolindex(size,150);
			p = mp->dalloc(idx);
			allocated_size = mp->psize(idx);
		}
	} catch(std::bad_alloc& e) {
		std::cerr << "bad_alloc in lalloc: size=" << size << std::endl;
#ifdef MEMSTAT
		printmemstat(std::cerr);
#endif
		exit(99);
	}
	return p;
}

void longint::lfree(uint32_t* p, int size) {
	if (size > maxdigits) {
		delete[] p;
	} else {
#ifdef TLS
#ifdef MacOS
		mem_pool* mp = (mem_pool*)pthread_getspecific(_mempool);
#else
		mem_pool* mp = _mempool;
#endif	/* MacOS */
#else	/* TLS */
		mem_pool* mp = _mempool;
#endif	/* TLS */
		assert(mp != 0);
		int idx = mp->getpoolindex(size,151);
		mp->dfree(p,idx);
	}
}

void longint::realloc(int size, int id) {
	int new_size = 0;
	uint32_t* p = lalloc(size, new_size);
	for (int i=0; i <= this->l; ++i) {
		p[i] = this->d[i];
	}
	lfree(this->d,this->capacity);
	this->d = p;
	this->capacity = new_size;
}

longint (*longint::lgcd_p)(longint, longint) = longint::lgcd2;  // ftH[glgcd2

/**
 * Add a n digit longint number (RHS) to m digit longint number (LHS)
 * The result replaces LHS.
 * Returns carry (0u or 1u).
 * Assume m >= n, otherwise the result is undefined.
 */
uint32_t longint::addll(uint32_t* a, int m, uint32_t* b, int n)
{
	uint32_t carry;
	int i;
	assert(m >= n);
	carry = 0;
	for (i=0; i < n; ++i) {
		uint32_t t, u;
		t = a[i] + carry;
		if (t < a[i]) {	// overflow
			a[i] = t + b[i];
			carry = 1u;
		} else {
			assert(carry==0u || a[i]!=0xffffffff);
			u = t + b[i];
			if (u < t) carry = 1u; else carry = 0u;
			a[i] = u;
		}
	}
	for (; i < m && carry != 0; ++i) {
		uint32_t v = a[i] + carry;
		if (v < a[i]) carry = 1u; else carry = 0u;
   		a[i] = v;
	}
	return carry;
}

/**
 * Subtract a n digit longint number (RHS) from a m digit longint number (LHS).
 * The result replaces LHS.
 * Returns borrow (0u  or 1u).
 * The resulting LHS is not zero suppressed, may contain leading zero digits.
 * Assume m >= n, otherwise the result is undefined.
 */
uint32_t longint::subll(uint32_t* a, int m, uint32_t* b, int n)
{
	uint32_t borrow;
	int i;
	assert(m>=n);
	borrow = 0;
   	for (i=0; i < n; ++i) {
		uint32_t t, u;
		t = a[i] - borrow;
       	if (t > a[i]) {	// underflow
       		assert(a[i]==0u && borrow==1u);
			a[i] = t - b[i];
		} else {
			assert(borrow==0 || a[i]!=0);
			u = t - b[i];
 			if (u > t) borrow = 1u; else borrow = 0u;
        	a[i] = u;
		}
  	}
    for (; i < m && borrow != 0u; ++i) {
		uint32_t v = a[i] - borrow;
		if (v > a[i]) borrow = 1u; else borrow = 0u;
   		a[i] = v;
   	}
	return borrow;
}

/**
 * Multiply m digit longint number (LHS) by a digit (RHS).
 * Result replaces LHS.
 * Returns carry (0u <= carry < BASE).
 */
uint32_t longint::mullu(uint32_t* a, int m, uint32_t s)
{
	uint32_t carry;
    int i;
	carry = 0u;
	for (i=0; i < m; ++i) {
		uint64_t t = static_cast<uint64_t>(a[i])*s + carry;
		carry = (t >> 32);
		a[i] = static_cast<uint32_t>(t);
	}
	return carry;
}

/**
 * Divide m digit longint number (LHS) by a digit (RHS).
 * Result, the quotient,  replaces LHS.
 * Returns the remainder .
 */
uint32_t longint::divlu(uint32_t* a, int m, uint32_t s)
{
	uint32_t r;
	int i;
	i = m;
	r = 0u;
	while (i > 0) {
		uint64_t t;
		--i;
		t = (static_cast<uint64_t>(r)<<32) + a[i];
		a[i] = t / s;
		r = t % s;
	}
	return r;
}

/*
 * Compute an estimate of quotient from 3-digit dividend and 2-digit divisor.
 * Based on Section 4.3.1, The Art of Computer Programming, Volume 2, by D.E.Knuth.
 */
uint32_t longint::estimate_quotient(uint32_t* u, uint32_t* v)
{
	uint64_t t, q, r;

	t = (static_cast<uint64_t>(u[2]) << 32) + u[1];
	if (u[2] == v[1]) {
		q = BASE - 1u;
//		r = u[1] + v[1];
	} else {
//		t = (static_cast<uint64_t>(u[2]) << 32) + u[1];
		q = t/v[1];
	}
	r=t-q*v[1];//r=t%v[1];
	while (r < BASE and q*v[0] > (r << 32) + u[0]) {
		q = q - 1;
		r = r + v[1];
	}
	return static_cast<uint32_t>(q);
}

// Default constructor
longint::longint() {
	try {
		this->d = lalloc(mindigits, this->capacity);
	} catch(std::runtime_error& e) {
		std::cerr << "longint::longint(): lalloc failed: mindigits=" << mindigits << "std::endl";
		exit(99);
	}
	this->l = 0;
}

// Copy constructor
longint::longint(const longint& rhs) {
	try {
		this->d = lalloc(rhs.capacity, this->capacity);
	} catch(std::runtime_error& e) {
		std::cerr << "longint::longint(const longint&): lalloc failed: rhs.capacity=" <<
			rhs.capacity << std::endl;
		assert(false);
		exit(99);
	}
	this->l = rhs.l;
	for (int i=0; i < rhs.l+1; ++i) {
#ifdef CHECKOVERFLOW
		this->checkoverflow(i,"A");
#endif
#ifdef CHECKBOUND
		this->checkbound(i,3001);
#endif
		this->d[i] = rhs.d[i];
	}
}

longint::longint(uint32_t n) {
	this->d = lalloc(mindigits, this->capacity);
	this->l = 0;
	if(n != 0)	{
		int i = 1;
		do {
#ifdef CHECKOVERFLOW
			this->checkoverflow(i,"longint(uint32_t n)");
#endif
			this->d[i] = n % BASE;
			n = n / BASE;
			i = i + 1;
		} while (n > 0);
		this->l = i - 1;
	}
}

longint::longint(uint64_t n) {
	this->d = lalloc(mindigits, this->capacity);
	this->l = 0;
	if(n != 0)	{
		int i = 1;
		do{
#ifdef CHECKOVERFLOW
			this->checkoverflow(i,"longint(uint64_t");
#endif
#ifdef CHECKBOUND
			this->checkbound(i,3003);
#endif
			this->d[i] = n % BASE;
			n = n / BASE;
			i = i + 1;
		} while (n > 0);
		this->l = i - 1;
	}
//	if(this->l > longintlmax) longintlmax=this->l;
}

longint::longint(const std::string& s) {
	uint64_t dd;
	this->d = lalloc(mindigits, this->capacity);
	this->l = 0;
	if (s.size()==0 || s[0]=='0') return;
	this->l = 1;
	this->d[1] = 0;
	for (unsigned int i=0; i < s.size(); ++i) {
		// Multiply d by 10
		dd = 0llu;
		for (int j=1; j <= this->l; ++j) {
			dd = (uint64_t)this->d[j]*(uint64_t)10u + dd;
			if (this->capacity < this->l+2) this->realloc(this->l+2,1002);
#ifdef CHECKOVERFLOW
			this->checkoverflow(j,"longint(const string-1");
#endif
			this->d[j] = dd % BASE;
			dd = dd / BASE;
		}
		if (dd != 0) {
#ifdef CHECKOVERFLOW
			this->checkoverflow(this->l+1,"longint(const string-2");
#endif
			this->d[++this->l] = dd;
		}
		// Add a digit
		dd = (uint64_t)(s[i] - '0');
		for (int j=1; j < this->l+1 && dd != 0llu; ++j) {
			dd = (uint64_t)this->d[j] + dd;
#ifdef CHECKOVERFLOW
			this->checkoverflow(j,"longint(const string-3");
#endif
			this->d[j] = dd % BASE;
			dd = dd / BASE;
		}
		if (dd != 0llu) {
			if (this->capacity < this->l+2) this->realloc(this->l+2,1002);
#ifdef CHECKOVERFLOW
			this->checkoverflow(this->l+1,"longint(const string-4");
#endif
			this->d[++this->l] = dd;
		}
	}
}

longint::longint(uint32_t* d, int l)
{
	this->d = lalloc(l+1 >= mindigits ? l+1 : mindigits , this->capacity);
	this->l = l;
	for (int i=0; i <l; ++i) { this->d[i+1] = d[i]; }
}

longint::~longint() {
	lfree(this->d,this->capacity);
}

std::string longint::hexstr() const {
	std::stringstream result;
	result << "{";
	for (int i=this->l; i >= 1; --i) {
		result << " " << std::hex << this->d[i];
	}
	result << "}";
	return result.str();
}

std::string longint::str() const {
	std::string result;
	char s[1000];
	sprintf(s, "{ capacity: %d, l: %d, d=[", this->capacity, this->l);
	result += s;
	result += this->hexstr();
	result += "]}";
	return result;
}

#ifdef CHECKOVERFLOW
void longint::checkoverflow(int index, const std::string& tag) {
	if (this->capacity < index+1) {
		std::cerr << "Index out of bound at " << tag << ": index=" << index << ", capacity=" << this->capacity << std::endl;
		assert(false);
	}
}
#endif

#ifdef CHECKBOUND
void longint::checkbound(int idx, int id) {
	if (idx < 0 || idx > this->l) {
		std::cerr << "checkbound: index out of bound: id=" << id << ", l=" << this->l << ", index=" << idx << std::endl;
		exit(99);
	}
}
#endif

longint& longint::operator =(uint32_t n) {
	int i;
	this->l = 0;
	if(n != 0)	{
		i = 1;
		do{
#ifdef CHECKOVERFLOW
			this->checkoverflow(i,"operator=");
#endif
			this->l = i;
#ifdef CHECKBOUND
			this->checkbound(i,3004);
#endif
			this->d[i] = n % longint::BASE;
			n = n / longint::BASE;
			i = i + 1;
		} while (n > 0);
		if (this->l != i-1) {
			std::cerr << "longint::operator=(long): Assertion failed 9001" << std::endl;
		}
		this->l = i - 1;
//		if(this->l > longint::longintlmax) longint::longintlmax=this->l;
	}
	return *this;
}

longint& longint::operator =(uint64_t rhs) {
//	longint A; int i;
	int i;

	this->l = 0;
	if(rhs == 0)	return *this;
	i = 1;
	do{
#ifdef CHECKOVERFLOW
		this->checkoverflow(i,"operator=(long long unsigned");
#endif
//#ifdef CHECKBOUND
//		this->checkbound(i,3005);
//#endif
		this->d[i] = rhs % longint::BASE;
		rhs = rhs / longint::BASE;
		i = i + 1;
	} while (rhs > 0);
	this->l = i - 1;
//	if(this->l > longint::longintlmax) longint::longintlmax=this->l;
	return *this;
}

longint& longint::operator =(const longint& rhs) {
	int i;
	if (this == &rhs) return *this;	// self assignment!
	if (this->capacity < rhs.capacity) {
		this->l = 0;	// to avoid copying
		this->realloc(rhs.capacity,1004);
	}
	this->l = rhs.l;
	for (i=0; i < rhs.l+1; ++i) {
#ifdef CHECKOVERFLOW
		this->checkoverflow(i,"operator=(const longint&");
#endif
#ifdef CHECKBOUND
		this->checkbound(i,3006);
#endif
		this->d[i] = rhs.d[i];
	}
	return *this;
}

longint& longint::operator+=(const longint& rhs) {
	uint64_t t;
	int i;
	if (this->l >= rhs.l) {
		i = 1;
		t = 0llu;
		while (i <= rhs.l) {
			t = t + (uint64_t)this->d[i] + (uint64_t)rhs.d[i];
			if (t < longint::BASE) {
#ifdef CHECKBOUND
				this->checkbound(i,3008);
#endif
				this->d[i] = t;
				t = 0llu;
			} else {
#ifdef CHECKBOUND
				this->checkbound(i,3009);
#endif
				this->d[i] = t - longint::BASE;
				t = 1llu;
			}
			++i;
		}
		while (i <= this->l && t != 0llu) {
			t = t + (uint64_t)this->d[i];
			if (t < longint::BASE) {
#ifdef CHECKBOUND
				this->checkbound(i,3010);
#endif
				this->d[i] = t;
				t = 0llu;
			} else {
#ifdef CHECKBOUND
				this->checkbound(i,3011);
#endif
				this->d[i] = t - longint::BASE;
				t = 1llu;
			}
			++i;
		}
		if (t != 0llu) {
#ifdef CHECKOVERFLOW
			this->checkoverflow(this->l+1,"operator+=");
#endif
			this->d[++this->l] = t;
		}
	} else {
		if (this->capacity < rhs.l+2) {                  //20150323
			this->realloc(rhs.l+2,2002);             //20150323
		}                                                //20150323
		i = 1;
		t = 0llu;
		while (i <= this->l) {
			t = t + (uint64_t)this->d[i] + (uint64_t)rhs.d[i];
#ifdef CHECKOVERFLOW
			this->checkoverflow(i,"operator+=1");
#endif
			if (t < longint::BASE) {
#ifdef CHECKBOUND
				this->checkbound(i,3012);
#endif
				this->d[i] = t;
				t = 0llu;
			} else {
#ifdef CHECKBOUND
				this->checkbound(i,3013);
#endif
				this->d[i] = t - longint::BASE;
				t = 1llu;
			}
			++i;
		}
		this->l = rhs.l;
		while (i <= rhs.l) {
			t = t + (uint64_t)rhs.d[i];
#ifdef CHECKOVERFLOW
			this->checkoverflow(i,"operator+=2");
#endif
			if (t < longint::BASE) {
#ifdef CHECKBOUND
				this->checkbound(i,3014);
#endif
				this->d[i] = t;
				t = 0llu;
			} else {
#ifdef CHECKBOUND
				this->checkbound(i,3015);
#endif
				this->d[i] = t - longint::BASE;
				t = 1llu;
			}
			++i;
		}
		if (t != 0llu) {
#ifdef CEHECKOVERFLOW
			this->checkoverflow(this->l+1,"operator+=3");
#endif
			this->d[++this->l] = t;
		}
	}
	return *this;
}

longint& longint::operator-=(const longint& rhs)
{
#ifdef EPROF2
	eprof(longint::opsubeqtime,longint::opsubeqcount);
#endif
	assert(this->l >= rhs.l);
	int m = this->l;
	int n = rhs.l;
	uint32_t borrow;
	int i;
	borrow = 0;
   	for (i=1; i <= n; ++i) {
		uint32_t t = this->d[i] - borrow;
       	uint32_t u;
       	if (t > this->d[i]) {	// underflow
			this->d[i] = t - rhs.d[i];
		} else {
			u = t - rhs.d[i];
 			if (u > t) {
				borrow = 1u;
			} else {
				borrow = 0u;
			}
        	this->d[i] = u;
		}
  	}
    for (; i <= m && borrow != 0u; ++i) {
		uint32_t v;
		v = this->d[i] - borrow;
		if (v > this->d[i]) {
			borrow = 1u;
		} else {
			borrow = 0u;
		}
   		this->d[i] = v;
   	}
	assert(borrow == 0);
   	this->slz();
	return *this;
}

longint& longint::operator *=(const longint& rhs) {
    if (this->l == 0 || rhs.l == 0) {
        this->l = 0;
    } else {
        longint res;
        int m = this->l;
        int n = rhs.l;
        int i, j;
        
        if (res.capacity < m+n+1) res.realloc(m+n+1, 0);
        res.l = m+n;
        for (i=0; i < res.l; ++i) res.d[i+1] = 0u;
        
        uint32_t carry;
        res.d[1] = 0;
        for (i=0; i < n; ++i) {
            carry = 0u;
            for (j=0; j < m; ++j) {
                uint64_t t = static_cast<uint64_t>(this->d[j+1])*rhs.d[i+1] + carry + res.d[i+j+1];
                carry = (t >> 32);
                res.d[i+j+1] = static_cast<uint32_t>(t & 0xffffffffu);
            }
            res.d[i+m+1] = carry;
        }
        res.slz();
        longint::swap(res,*this);
    }
	return *this;
}

longint& longint::operator /=(const longint& rhs) {
	assert(false || "operator /= not implemented.");
	return *this;
}

longint& longint::operator>>=(int n) {
	uint32_t c;
	for (int j=0; j < n; ++j) {
		c = 0;
		for (int i=this->l; i > 0; --i) {
			uint32_t lsb;
			lsb = (this->d[i] << 31);
#ifdef CHECKOVERFLOW
			this->checkoverflow(i,"operator>>=");
#endif
#ifdef CHECKBOUND
			this->checkbound(i,3016);
#endif
			this->d[i] = ((this->d[i] >> 1) | c);
			c = lsb;
		}
		if (this->d[l] == 0) --this->l;
	}
	return *this;
}

longint& longint::operator<<=(int n) {
#ifdef USESHL
    this->shl(n);
#else
	uint32_t c;
	for (int j=0; j < n; ++j) {
		c = 0;
		for (int i=1; i <= this->l; ++i) {
			uint32_t msb;
			msb = this->d[i] >> 31;
#ifdef CHECKOVERFLOW
			this->checkoverflow(i,"operator<<=");
#endif
#ifdef CHECKBOUND
			this->checkbound(i,3017);
#endif
			this->d[i] = ((this->d[i] << 1) | c);
			c = msb;
		}
		if (c != 0) {
			if (this->capacity < this->l+2) {         // 2016may30
				this->realloc(this->l+2,1007);    // 2016may30
			}                                         // 2016may30
#ifdef CHECKOVERFLOW
			this->checkoverflow(this->l+1,"operator<<=");
#endif
			this->d[++this->l] = c;
		}
	}
#endif
	return *this;
}

longint& longint::operator |=(const longint& r) {
	if (this->capacity < r.capacity) {
		this->realloc(r.capacity,1004);
	}
	int ll = this->l;
	int lr = r.l;
	int lmin = std::min(ll,lr);
	int lmax = std::max(ll,lr);
	for (int i=1; i <= lmin; ++i) this->d[i] |= r.d[i];
	for (int i=lmin+1; i <= lr; ++i) this->d[i] = r.d[i];
	this->l = lmax;
	return *this;
}

longint longint::operator>>(int n) const {
	longint result = *this;
	result >>= n;
	return result;
}

longint longint::operator<<(int n) const {
	longint result = *this;
	result <<= n;
	return result;
}

longint longint::operator |(const longint& r) const {
	longint result = *this;
	result |= r;
	return result;
}

bool longint::operator ==(const longint& rhs) const {
	return lcmp(*this,rhs) == 0;
}

bool longint::operator !=(const longint& rhs) const {
	return lcmp(*this,rhs) != 0;
}

bool longint::operator >(const longint& rhs) const {
	return lcmp(*this,rhs) > 0;
}

bool longint::operator >=(const longint& rhs) const {
	return lcmp(*this,rhs) >= 0;
}

bool longint::operator <=(const longint& rhs) const {
	return lcmp(*this,rhs) <= 0;
}

bool longint::operator <(const longint& rhs) const {
	return lcmp(*this,rhs) < 0;
}

longint::operator double() {
	double result;
	result=0.0;
	for (int i=this->l; i > 0; --i) {
		result = result*(double)longint::BASE + (double)this->d[i];
	}
	return result;
}

longint longint::isqrt() const {
	longint res = (uint32_t)0u;
    longint add;
//    int n;
    add = (uint32_t)1u;
    add <<= ((this->l+1)/2*32);
//    n = (this->l+1)/2*16;
    while (add != longint::ZERO)
    {
        longint temp = res | add;
        longint g2 = temp*temp;
        if (*this >= g2)
        {
            res = temp;
        }
        add >>= 1;
    }
    return res;
}

uint32_t longint::ctz() const
{
    uint32_t di, bi;
    uint32_t t;
    if (this->l == 0) return 0;
    for(di=0; this->d[di+1] == 0; ++di) ;
    for(t=this->d[di+1], bi=0; (t & 1)==0; t >>= 1, ++bi);
    return di*32+bi;
}

void longint::shl(int32_t n)
{
    int32_t w = n / 32;
    int32_t b = n % 32;
    if (l != 0 && n != 0) {
        uint32_t i;
        if (b == 0) {
            if (capacity < l+w+1) realloc(l+w+1, 3);
            for(i = l; i > 0; --i) {
                d[i+w] = d[i];
            }
            for (i = w; i > 0; --i) {
                d[i] = 0;
            }
            this->l += w;
        } else {
            uint32_t u, v;
            if (capacity < l+w+2) realloc(l+w+2, 3);
            u = d[l];
            d[l+1+w] = (u >> (32-b));
            for (i = l; i > 1; --i) {
                v = d[i-1];
                d[i+w] = (u << b | v >> (32-b));
                u = v;
            }
            d[w+1] = u << b;
            for (i = w; i > 0; --i) {
                d[i] = 0;
            }
            this->l += (w+1);
            this->slz();
        }
    }
}

void longint::shr(int32_t n)
{
#ifdef EPROF2
	eprof e(longint::shrtime,longint::shrcount);
#endif
    int32_t nw = n/32;
    int32_t nb = n%32;
    if (l <= nw) {
        this->l = 0;
    } else if (nb == 0) {
        uint32_t m = this->l-nw;
        uint32_t n = 0;
        if (m >= 4) {
            n = m & 0xfffffffc;
            for (uint32_t i=0; i < n; i+=4) {
                this->d[i+1] = this->d[i+nw+1];
                this->d[i+2] = this->d[i+nw+2];
                this->d[i+3] = this->d[i+nw+3];
                this->d[i+4] = this->d[i+nw+4];
            }
        }
        for (uint32_t i=n; i < m; ++i) {
            this->d[i+1] = this->d[i+nw+1];
        }
        this->l = this->l - nw;
    } else {
        uint32_t m = this->l-nw-1;
        uint32_t n = 0;
        uint32_t nbl = 32 - nb;
        if (m >= 4) {
            n = m & 0xfffffffc;
            for (uint32_t i=0; i < n; i+=4) {
                this->d[i+1] = (this->d[i+nw+1] >> nb | this->d[i+nw+2] << nbl);
                this->d[i+2] = (this->d[i+nw+2] >> nb | this->d[i+nw+3] << nbl);
                this->d[i+3] = (this->d[i+nw+3] >> nb | this->d[i+nw+4] << nbl);
                this->d[i+4] = (this->d[i+nw+4] >> nb | this->d[i+nw+5] << nbl);
            }
        }
        for (uint32_t i=n; i < m; ++i) {
            this->d[i+1] = (this->d[i+nw+1] >> nb | this->d[i+nw+2] << nbl);
        }
        this->d[l-nw] = (this->d[l] >> nb);
        this->l -= nw;
    }
    this->slz();
}

int numdgts(const longint& X){
    return X.l;
}

uint32_t numbits(const longint& A)    // Number of Bits of longint number
{
	uint32_t dl=A.d[A.l];
	uint32_t nba=0;
	if(dl != 0){  
		uint32_t nbdl=32-longint::clzu(dl);  // number of bits in A.d[A.l]
		nba=32*(numdgts(A)-1)+nbdl;          // number of bits of A
	}
	return nba;
}

longint longint::lset(uint32_t n){  /* setting longint by long int */
	longint A; int i;

	A.l = 0;
	if(n == 0)	return A;
	i = 1;
	do{
#ifdef CHECKOVERFLOW
		A.checkoverflow(i,"lset");
#endif
		A.l = i;
#ifdef CHECKBOUND
		A.checkbound(i,3018);
#endif
		A.d[i] = n % longint::BASE;
		n = n / longint::BASE;
		i = i + 1;
	} while (n > 0);
	A.l = i - 1;
//2015dec12	if(A.l > longint::longintlmax) longint::longintlmax=A.l;
	return A;
}

longint longint::llset(uint64_t n){	/* setting longint by unsigned long long int */
	longint A; int i;

	A.l = 0;
	if(n == 0)	return A;
	i = 1;
//	if (A.capacity < 3) A.realloc(3,1008);
	do{
#ifdef CHECKOVERFLOW
		A.checkoverflow(i,"llset");
#endif
		A.l = i;
#ifdef CHECKBOUND
		A.checkbound(i,3019);
#endif
		A.d[i] = n % longint::BASE;
		n = n / longint::BASE;
		i = i + 1;
	} while (n > 0);
	A.l = i - 1;
//	if(A.l > longint::longintlmax) longint::longintlmax=A.l;
	return A;
}
longint longint::smul(const longint& A, uint32_t n){	/* multiplication longint * Ulongint */
	longint X; uint64_t t; int i;

	X.l = 0;
	if(A.l == 0 || n == 0)	return X;
//	if(abs(n) >= longint::BASE){               /*   2011 May 06   */
	if(n >= longint::BASE){                    /*   2011 May 06   */
		// printf("FATAL ERROR in smul %lu X ",n);
		std::cout << "FATAL ERROR in smul " << n << " X ";
		decprt("X\0",A);
assert(A.l != 0 && n !=0);
assert(false);
		exit(3);
	}
	if (X.capacity < A.l+1) X.realloc(A.l+1,1009);
	t = 0;
	for(i = 1; i <= A.l; i++){
		t = t + (uint64_t)A.d[i] * n;
#ifdef CHECKOVERFLOW
		X.checkoverflow(i,"smul-1");
#endif
		X.l = i;
#ifdef CHECKBOUND
		X.checkbound(i,3020);
#endif
#ifdef TUNE
        X.d[i] = static_cast<uint32_t>(t);
        t = (t >> 32);
#else
		X.d[i] = t % longint::BASE;
		t = t / longint::BASE;
#endif
	}
	X.l = A.l;
	while(t != 0llu){
		if(X.capacity < X.l+2) {
			X.realloc(X.l+2,1010);
		}
		X.l = X.l + 1;
#ifdef CHECKOVERFLOW
		X.checkoverflow(X.l,"smul-2");
#endif
#ifdef CHECKBOUND
		X.checkbound(X.l,3020);
#endif
#ifdef TUNE
        X.d[X.l] = static_cast<uint32_t>(t);
        t = (t >> 32);
#else
		X.d[X.l] = t % longint::BASE;
		t = t / longint::BASE;
#endif
	}
//	if(X.l > longint::longintlmax) longint::longintlmax=X.l;
	return X;
}

void longint::slmul(const longint& A, uint64_t n, longint& X){
	uint64_t t; int i; longint Y,Z,R,ZERO;

	X.l = 0;
	if(A.l == 0 || n == 0)	return;

	if( n > longint::BASE){
		Y = llset(n);
		X = lmul(A,Y);
	} else {
		if (X.capacity < A.l+1) X.realloc(A.l+1,1011);
		t = 0;
		for(i = 1; i <= A.l; i++){
			t = t + (uint64_t)A.d[i] * n;
#ifdef CHECKOVERFLOW
			X.checkoverflow(i,"slmul-1");
#endif
			X.l = i;
#ifdef CHECKBOUND
			X.checkbound(i,3021);
#endif
			X.d[i] = t % longint::BASE;
			t = t / longint::BASE;
		}
		X.l = A.l;

		while(t != 0){
			if (X.capacity < X.l+2) {
				X.realloc(X.l+2,1012);
			}
			X.l = X.l + 1;
#ifdef CHECKOVERFLOW
			X.checkoverflow(X.l,"slmul-2");
#endif
			X.d[X.l] = t % longint::BASE;
			t = t / longint::BASE;
		}

	}
//	if(X.l > longint::longintlmax) longint::longintlmax=X.l;
	return;
}

longint longint::sdiv(const longint& A, uint32_t n, uint64_t* r){  /* division longint / int */
	longint X; uint64_t t; int i;
	X.l = *r = 0;
	if(A.l == 0)	return X;
	if(     n  >= longint::BASE ){               /*   2011 May 6  */
		std::cout << "Error(sdiv) : " << n << ", BASE=" << longint::BASE << std::endl;
		decprt("Error A=\0",A);
//		std::cout << "<sdiv: return " << X << std::endl;
		return X;
	}
	if (X.capacity < A.l+1) X.realloc(A.l+1,1013);	// the result has at most A.l+1 digits.
	t = 0;
	X.l = A.l;
	for(i = A.l; i >= 1; i--){
		t = t * longint::BASE + (uint64_t)A.d[i];	// 2011/08/29 Goda
#ifdef CHECKOVERFLOW
		X.checkoverflow(i,"sdiv-1");
#endif
#ifdef CHECKBOUND
		X.checkbound(i,3022);
#endif
		X.d[i] = t / (uint64_t)n;	// 2011/08/29 Goda
		t = t % n;	// 2011/08/29 Goda
	}
	*r = t;
	X.l = A.l;
#ifdef CHECKOVERFLOW
	X.checkoverflow(A.l,"sdiv-2");
#endif
	if (X.d[A.l] == 0)	X.l = X.l - 1;
	return X;
}
void longint::decprt(std::string s, longint A){   /* decimal print out longint */
	std::cout << "decprt " << s << ":" << A << std::endl;
	return;
}

int longint::lcmp(const longint& A, const longint& B){   /* compare longint vs longint */
	int i;
	if(A.l > B.l)	return 1;
	if(B.l > A.l)	return -1;
#ifdef EPROF
	long long tstart = gettime();
#endif
	for (i=A.l; i > 0; --i) {
		if (A.d[i] != B.d[i]) break;
	}
#ifdef EPROF
	longint::lcmptime += (gettime() - tstart);
#endif
	return (i == 0 ? 0 : (A.d[i] > B.d[i]) ? 1 : -1);
}

longint longint::absub(const longint& A, const longint& B, int id) {	/* absolute subtraction |longint A| - |longint B| at |A| >= |B| */
	longint X;
	int t = lcmp(A, B);
	if(t < 0){
		std::cerr << "Error : A < B  in absub()" << std::endl;
		exit(3);
	}
	X=A;
	X-=B;
	return X;
}
longint longint::ladd(const longint& A, const longint& B) {    /* addition  longint + longint */
#ifdef EPROF2
	eprof e(longint::laddtime,longint::laddcount);
#endif
	longint X;
	
	X = A;
	X += B;
	return X;
}
longint longint::lsub(const longint& A, const longint& B){   /* subtraction longint - longint */
	longint X,Y; int s;

	X.l = 0;
	s = lcmp(A, B);
	if(s == 0)     return X;
	if(A.l == 0) return B;
	if(B.l == 0) return A;
	s = (s >= 0);
	if(s){
		X=A-B;
	}else{
		X=B-A;
	}
	return X;
}

int longint::lsubv(const longint& A, const longint& B, longint& X, int id) {   /* subtraction longint - longint */
	longint Y; int s;

	X.l = 0;
	s = lcmp(A, B);
	if(s == 0)     return s;
	if(A.l == 0) X = B;
	if(B.l == 0) X = A;
	if(s == 1){
		X = absub(A, B, 3);
	}else{
		X = absub(B, A, 4);
	}
	return s;
}

longint longint::lmul(const longint& A, const longint& B){   /* multiplication longint * longint */
	longint X,Y,Z;
#ifdef EPROF2
	eprof e(longint::lmultime,longint::lmulcount);
#endif
    X = A;
    X *= B;
    return X;
}

longint longint::ldivide(const longint& a, const longint& b, longint *r)
{
#ifdef EPROF2
	eprof e(longint::ldividetime,longint::ldividecount);
#endif
	assert(b.l != 0);
	if (b.l == 0) {
		throw std::runtime_error("longint: divide by zero");
	}
	longint u(a);
	if (a < b) {
		if (r != 0) *r = a;
		u.l = 0;
		return u;
	}
	longint v(b);
	longint qv;
	longint q;
	if (b.l == 1) {
		uint32_t rr = divlu(&u.d[1],u.l,v.d[1]);
		u.slz();
		if (r != 0) {
			if (rr == 0u) {
				r->l = 0;
			} else {
				r->d[1] = rr;
				r->l = 1;
			}
		}
		return u;
	}
	int f, k, m, n;
	m = a.l;
	n = b.l;
	if (qv.capacity < n+2) qv.realloc(n+2,0);
	assert(qv.capacity>=(n+2));
	if (q.capacity < (m-n+2)) q.realloc(m-n+2,0);
	q.l = m-n+1;
	if (u.capacity < u.l+2) u.realloc(u.l+2,0);
	u.d[m+1] = 0u;
	++u.l;
	f = clzu(b.d[n]);
	if (f != 0) {
		u <<= f;
		v <<= f;
	}
	k = m - n;
	while (k >= 0) {
		uint32_t qh;
		uint32_t borrow, carry;
		qh = estimate_quotient(&u.d[n+k-2+1],&v.d[n-1]);
		if (qh != 0) {
			copy_digits(&qv.d[1],&v.d[1],n);
			qv.d[n+1] = 0u;
			mullu(&qv.d[1],n+1,qh);
			borrow = subll(&u.d[k+1],n+1,&qv.d[1],n+1);
			if (borrow != 0) {
				--qh;
				carry = addll(&u.d[k+1],n+1,&v.d[1],n);
				while (carry == 0) {
					--qh;
					carry = addll(&u.d[k+1],n+1,&v.d[1],n);
				}
			}
		}
		q.d[k+1] = qh;
		k = k - 1;
	}
	if (r != 0) {
		u.slz();
		if (f != 0) u.shr(f);
		swap(u,*r);
	}
	q.slz();
#ifdef LDIV2_VERIFY
	{
		longint w;
		w = q*b + *r;
		if (a != w) {
			std::cerr << "ldivide: incorrect a=" << a.str() << ", b=" << b.str() << std::endl;
			std::cerr << "  q=" << q.str() << ", r=" << r->str() << std::endl;
			assert(0);
		}
	}
#endif
	return q;
}

longint longint::lgcd1(longint x, longint y){    // 2015.aug.19 Samukawa
	return longint::ONE;
}

longint longint::lgcd2(longint x, longint y){    // 2013.oct.27 (nov.04) Gohda
//	long long tstart = gettime();
#ifdef EPROF2
	eprof e(longint::lgcd2time,longint::lgcd2count);
#endif
	if(lcmp(x,longint::ZERO) == 0 ) {
		std::cout << "FATAL ERROR in lgcd2 x zero" << std::endl;
		assert(false);
	}
	assert(y != longint::ZERO);
	if(lcmp(y,longint::ZERO) == 0 ) {
		std::cout << "FATAL ERROR in lgcd2 y zero" << std::endl;
		assert(false);
	}
	uint32_t cf2 = std::min(x.ctz(),y.ctz());
	
    x.shr(x.ctz());
//std::cout << "lgcd2.1: x=" << x.str() << " y=" << y.str() << " cf2=" << cf2 << std::endl;

	for (;;) {
        y.shr(y.ctz());
//std::cout << "lgcd2.2: x=" << x.str() << " y=" << y.str() << std::endl;
		int s = lcmp(x,y);
		if (s == 0) break;
		if (s > 0) {	// swap
//std::cout << "lgcd2.3: swqp" << std::endl;
			swap(x,y);
		}
//std::cout << "lgcd2.4: x=" << x.str() << " y=" << y.str() << std::endl;
		if (lcmp(x,longint::ONE) == 0) break;
		y -= x;
//std::cout << "lgcd2.5: x=" << x.str() << " y=" << y.str() << std::endl;
	}
    	x.shl((int)cf2);
	return x;
}

longint longint::lgcd(longint X, longint Y){  // lgcd  from 2011 version
	longint Q,R; int i;

#ifdef EPROF2
	eprof e(longint::lgcdtime,longint::lgcdcount);
#endif
	i = lcmp(X,Y);
	if( i < 0 ){
		swap(X,Y);
	}
	while(lcmp(Y,longint::ZERO) != 0){                /*  2011JUN03  */
        /*		printf(" lgcd loop i=%d",i); decprt(" X\0",X); decprt(" Y\0",Y); */
		Q = ldivide(X, Y, &R);
        //		X = Y; Y = R;    /*  2013Nov06  */
		swap(X,Y); swap(Y,R);
	}
	return X;
}

longint longint::operator+(const longint& rhs) const {
	longint result = *this;
	result += rhs;
	return result;
}

longint longint::operator-(const longint& rhs) const {
	longint result = *this;
	result -= rhs;
	return result;
}

longint longint::operator*(const longint& rhs) const {
	return lmul(*this,rhs);
}

longint longint::operator/(const longint& rhs) const {
	longint r;
	return ldivide(*this, rhs, &r);
}

int longint::getdigits(uint32_t* d, int l, longint& a)
{
	int n = std::min(l,a.l);
	for (int i=0; i < n; ++i) {
		d[i] = a.d[i+1];
	}
	return a.l;
}
uint32_t longint::test_addll(longint& u, longint& v)
{
	return addll(&u.d[1],u.l, &v.d[1],v.l);
}
uint32_t longint::test_subll(longint& u, longint& v)
{
	return subll(&u.d[1],u.l,&v.d[1],v.l);
}
uint32_t longint::test_mullu(longint& u, uint32_t v)
{
	return mullu(&u.d[1],u.l,v);
}
uint32_t longint::test_divlu(longint& u, uint32_t v)
{
	return divlu(&u.d[1],u.l,v);
}
uint32_t longint::test_estimate_quotient(longint& u, longint& v)
{
	return estimate_quotient(&u.d[1],&v.d[1]);
}

std::ostream& operator <<(std::ostream& stream, const longint& A) {
//	cout << "ostream& operator<<" << endl;
	longint tmp = A;
	std::string s;
	uint32_t r;
	uint64_t t, q;
	if (tmp.l==0) {
		s += "0";
	} else {
		while (tmp.l != 0) {
			r = 0llu;
			for (int i=tmp.l; i >= 1; --i) {
				t = (uint64_t)r*longint::BASE + tmp.d[i];
				r = t % 10llu;
				q = t / 10llu;
#ifdef CHECKOVERFLOW
				tmp.checkoverflow(i,"operator<<");
#endif
#ifdef CHECKBOUND
				tmp.checkbound(i,3040);
#endif
				tmp.d[i] = q;
				if (tmp.d[tmp.l] == 0lu) --tmp.l;
			}
			std::stringstream out;
			out << r;
			s += out.str();
		}
	}
	stream << std::string(s.rbegin(),s.rend());
	return stream;
}

longint const& min(longint const& x, longint const& y)
{
	if (x < y) return x; else return y;
}

longint const& max(longint const& x, longint const& y)
{
	if (x > y) return x; else return y;
}

longint pow(longint const& x, int n)
{
	longint result = longint::ONE;
	longint p = x;
	while (n != 0) {
		if ((n & 1) != 0) {
			result *= p;
		}
		p *= p;
		n >>= 1;
	}
	return result;
}

oarchive& operator<<(oarchive& oa, const longint& x)
{
    oa.getOs() << " " << x.l;
    for (int i=0; i < x.l; ++i) oa.getOs() << " " << x.d[i+1];
    return oa;
}

iarchive& operator>>(iarchive& ia, longint& x)
{
	int l;
    ia.getIs() >> l;
	if (l+1 >= x.capacity) x.realloc(l+1,3001);
    for (int i=0; i < l; ++i) {
		ia.getIs()  >> x.d[i+1];
	}
	x.l = l;
    return ia;
}

std::ostream& operator<<(std::ostream& o, digits const& s)
{
	o << "{";
	if (s.len_ > 0) {
		o << std::hex << s.d_[s.len_-1];
		for (int i=0; i < s.len_-1; ++i) {
			o << " " << std::hex << s.d_[s.len_-2-i];
		}
	}
	o << "}";
	return o;
}

static inline int clzu(uint32_t a)
{
	return __builtin_clz(a);
}

static void shru(uint32_t* a, int len, int n)
{
	if (len > 0) {
		for (int i=0; i < len-1; ++i) {
			a[i] = (a[i] >> n) | a[i+1] << (32-n);
		}
		a[len-1] = (a[len-1] >> n);
	}
}

static void shlu(uint32_t* a, int len, int n)
{
	if (len > 0) {
		for (int i=len; i > 1; --i) {
			a[i-1] = (a[i-1] << n) | (a[i-2] >> (32-n));
		}
		a[0] = (a[0] << n);
	}
}

/**
 * Convert a longint value to double value between 0.0 and 2^52-1 and
 * exponet
 */
double longint2double(longint const& n, int& exp)
{
	uint64_t res;
	uint32_t m[3];
	int lz;
	switch(n.l) {
	case 0:
		res = 0;
		break;
	case 1:
		res = static_cast<uint64_t>(n.d[1]);
		break;
	case 2:
		m[1] = n.d[2];
		m[0] = n.d[1];
		lz = clzu(m[1]);
		if (lz < 11) {
			shru(m,2,11-lz);
			exp += (11 - lz);
		}
		res = (static_cast<uint64_t>(m[1])<<32) + m[0];
		break;
	default:
		m[2] = n.d[n.l];
		m[1] = n.d[n.l-1];
		m[0] = n.d[n.l-2];
		lz = clzu(m[2]);
		exp += ((n.l-2)*32);
		if (lz < 11) {
			shru(m,3,11-lz);
			exp += (11-lz);
		} else if (lz > 11) {
			shlu(m,3,lz-11);
			exp -= (lz-11);
		}
		res = (static_cast<uint64_t>(m[2])<<32) + m[1];
	}
	return static_cast<double>(res);
}
