/*
 * longint.h
 *
 *  Created on: 2011/07/12
 *      Author: goda
 *  Modified on: 2013/Jan/06  add "friend int numdgts(const longint);"
 */

#ifndef LONGINT_H_
#define LONGINT_H_
#include <string>
#include <iostream>
//#include "rvector.h"
//#include "rmatrix.h"
#define BINARY_GCD
#define USE_COMPOUND_SHIFT

#include "archive.h"
#include "mempool.h"
#ifdef TLS
 #include <pthread.h>
// #include "mpfft1.h"
#endif	// TLS

long long gettime();

class eprof {
	long long& acc_;
	long long t;
public:
	eprof(long long& a, long& cnt) : acc_(a)
	{
		t = gettime();
		++cnt;
	}
	~eprof()
	{
		acc_ += (gettime() - t);
	}
};

class digits {
	uint32_t* d_;
	int len_;
public:
	digits(uint32_t* d, int len) : d_(d), len_(len) {}
	friend std::ostream& operator<<(std::ostream&, digits const&);
};

class longint {
public:
	int capacity;
	int l;
	uint32_t *d;
#ifdef TLS
	static bool pool_initialized;
#endif	// TLS

	static uint32_t addll(uint32_t*, int, uint32_t*, int);
	static uint32_t subll(uint32_t*, int, uint32_t*, int);
	static uint32_t mullu(uint32_t*, int, uint32_t);
	static uint32_t divlu(uint32_t*, int, uint32_t);
	static uint32_t estimate_quotient(uint32_t* u, uint32_t* v);
	static uint32_t divll1(uint32_t*, int, uint32_t*, int, uint32_t*);
	static uint32_t msubllu(uint32_t*, int, uint32_t*, int, uint32_t);

public:
	static longint (*lgcd_p)(longint, longint); // ֐ւ̃|C^
	static const int mindigits = MINDIGITS;	// should be power of 2
	static const int maxdigits = MAXDIGITS;
	static uint64_t BASE;
	static int longintlmax;
	static int longintverif;
#ifdef MEMSTAT
	static long num_alloc;
	static long num_realloc;
	static long num_free;
	static long num_hit;
#endif
	static long long ldividetime;
	static long ldividecount;
	static long long lgcdtime;
	static long lgcdcount;
	static long long lgcd2time;
	static long lgcd2count;
	static long long opsubeqtime;
	static long opsubeqcount;
	static long long shrtime;
	static long shrcount;
	static long long lmultime;
	static long lmulcount;
	static long long laddtime;
	static long laddcount;

	static long unsigned ldiv2calls;
	static long unsigned ldiv2qdiff0;
	static long unsigned ldiv2qdiff1;
	static long unsigned ldiv2qdiff2;
#ifdef EPROF
	static long long lgcdtime;
	static long long absadtime;
	static long long absubtime;
	static long long lcmptime;
	static long long ldiv2ptime;
	static long long lmultime;
	static long long ldividetime;
	static long long ldivnbittime;
	static long long lmulnbittime;
#endif

	static bool _init;
	static const longint ZERO;
	static const longint ONE;
	static const longint TWO;
	static const longint THREE;
	static const longint FOUR;
	static const longint FIVE;
	static const longint SIX;
	static const longint TEN;
	static const longint SIXTEEN;
	/************************************************
	static const longint M;   // 1,000,000 = 10^6
	static const longint MM;   // 10^12
	static const longint MMM;   // 10^24
	static const longint MMMM;   // 10^48
	static const longint MMMMM;   // 10^96
	static const longint MMMMMM;   // 10^192
	static const longint MMMMMMM;  // 10^384
	static const longint MMMMMMMM;  // 10^768
	static const longint MMMMMMMMM;  // 10^1536
	static const longint MMMMMMMMMM;  // 10^3072
	static const longint MMMMMMMMMMM;  // 10^6144
	static const longint MMMMMMMMMMMM;  // 10^12288
	************************************************/
	static const longint TWOGIGA;
	static const longint FOURGIGA;
	static const longint FOURPETA;
	
	static long unsigned* _numreallocv;
#ifdef MEMSTAT
	static long unsigned* _numallocv;
	static long unsigned* _numfreev;
	static long unsigned* _peekmemv;
#endif
	static void printmemstat(std::ostream&);

#ifdef TLS
#ifdef MacOS
	static pthread_key_t _mempool;
	static pthread_key_t _mpffttls;
#else	/* MacOS */
	static __thread mem_pool* _mempool;
//	static __thread mpfft1tls_t* _mpffttls;
#endif	/* MacOS */
#else
	static mem_pool* _mempool;
#endif
	static bool init();
	static uint32_t* lalloc(int, int&);
	static void lfree(uint32_t*, int);

	longint();
	longint(const longint&);
	longint(uint32_t);
	longint(uint64_t);
	longint(const std::string&);
	longint(uint32_t*,int);
	virtual ~longint();

	void realloc(int,int);
	int getPrecision() { return l; }
#ifdef CHECKOVERFLOW
	void checkoverflow(int,const std::string&);
#endif
#ifdef CHECKBOUND
	void checkbound(int,int);
#endif
	std::string hexstr() const;
	std::string str() const;

	// overloaded operators
	longint& operator =(const longint&);
	longint& operator =(uint32_t);
	longint& operator =(uint64_t);
	longint& operator +=(const longint&);
	longint& operator -=(const longint&);
	longint& operator *=(const longint&);
	longint& operator /=(const longint&);
	longint& operator <<=(int);
	longint& operator >>=(int);
	longint& operator |=(const longint&);
	longint operator +(const longint&) const;
	longint operator -(const longint&) const;
	longint operator *(uint32_t) const;
	longint operator *(const longint&) const;
	longint operator /(const longint&) const;
	longint operator <<(int) const;
	longint operator >>(int) const;
	longint operator |(const longint&) const;
	bool operator ==(const longint&) const;
	bool operator !=(const longint&) const;
	bool operator >(const longint&) const;
	bool operator >=(const longint&) const;
	bool operator <=(const longint&) const;
	bool operator <(const longint&) const;
	operator double();

	// member functions
	longint isqrt() const;
	uint32_t ctz() const;
	void shl(int32_t);
	void shr(int32_t);
	void slz() {
		int i = l;
		while (i > 0 && d[i] == 0) --i;
		l = i;
	}
	double to_double(int&);

	/*	Returns the least significant 64-bits in a uint64_t value.
	 *  Added on July 30, 2016 by Goda
	 */
	uint64_t to_ullong() const {
		if (l == 0) return 0;
		if (l == 1) return static_cast<uint64_t>(d[1]);
		return (static_cast<uint64_t>(d[2]) << 32) + d[1];
	}

	static inline void swap(longint& a, longint& b) {
		std::swap(a.capacity,b.capacity);
		std::swap(a.l,b.l);
		std::swap(a.d,b.d);
	}

	static inline uint32_t clzu(uint32_t a)
	{
		uint32_t count;
		count = 0;
		while ((a & 0x80000000) == 0u) {
			a <<= 1;
			++count;
		}
		return count;
	}

	static inline void copy_digits(uint32_t* d, uint32_t* s, int len)
	{
		uint32_t* s_end = &s[len];
		while (s != s_end) {
			*d++ = *s++;
		}
	}


	// static functions
	static longint lset(uint32_t);
	static longint llset(uint64_t);
	static longint smul(const longint&, uint32_t);
	static void slmul(const longint&, uint64_t, longint&);
	static longint sdiv(const longint&, uint32_t, uint64_t*);
	static void decprt(std::string, longint);
//	static void sqrtprt(std::string, longint, int);
	static int lcmp(const longint&, const longint&);
//	static longint ldiv2p(const longint&);
//	static void ldiv2(const longint&, longint&);
//	static void ldivnbit(longint&, int);
//	static longint lmul2p(const longint&);
//	static void lmulnbit(longint&, int);
//	static longint absad(const longint&, const longint&);
	static longint absub(const longint&, const longint&, int);
	static longint ladd(const longint&, const longint&);
	static longint lsub(const longint&, const longint&);
	static int lsubv(const longint&, const longint&, longint&, int);
//	static void ldown(longint*, int);
//	static void lup(longint*, int);
	static longint lmul(const longint&, const longint&);
	static longint ldivide(const longint&, const longint&, longint*);
//	static longint ldivide2(const longint&, const longint&, longint*);
	static longint lgcd(longint, longint);
	static longint lgcd1(longint, longint); // do not compute GCD
	static longint lgcd2(longint, longint);
//	static longint lgcd2a(longint, longint);

	static int getdigits(uint32_t*,int,longint&);
	static uint32_t test_addll(longint&, longint&);
	static uint32_t test_subll(longint&, longint&);
	static uint32_t test_mullu(longint&, uint32_t);
	static uint32_t test_divlu(longint&, uint32_t);
	static uint32_t test_estimate_quotient(longint&, longint&);
	static uint32_t test_ldiv2(longint&, longint&);

	//longint operator *(const longint&, const longint&);
	friend std::ostream& operator <<(std::ostream&, const longint&);
	friend int lint2mp(const longint &, float *);
	friend int numdgts(const longint&);
	friend uint32_t numbits(const longint&);
//    friend oarchive& operator<<(oarchive&, const longint&);
//    friend iarchive& operator>>(iarchive&, longint&);
	// added, can be used by application, i.e., not member function.
	friend longint const& min(longint const&, longint const&);
	friend longint const& max(longint const&, longint const&);
	friend longint pow(longint const&, int);
	friend oarchive& operator<<(oarchive&, longint const&);
	friend iarchive& operator>>(iarchive&, longint&);
	friend double ndpw2(const longint&, int&);
	friend double longint2double(longint const&, int&);
};

#ifdef TLS
#ifdef USEFFT
inline struct _mpfft1tls* getffttlsp() {
#ifdef MacOS
	return (mpfft1tls_t*) pthread_getspecific(longint::_mpffttls);
#else	/* MacOS */
	return longint::_mpffttls;
#endif	/* MacOS */
}
#endif	/* USEFFT */
#endif	/* TLS */

//typedef vector<longint> longint_vector;
//typedef matrix<longint> longint_matrix;

#endif /* LONGINT_H_ */
