/*
*
*  Modified on: 2016/07/11 by Samukawa setpow10 MMMMMM
*/

#include <math.h>
#include <iostream>
#include <string>
#include <sstream>
#include "longint.h"
#include "rational.h"
//#include "mpow.h"               // for setpow10
#define RAT_REDADD		// Rational Add reduced operation
//#define RAT_REDUCE		// Rational Mul, Div reduced operation
#define TUNE			// ldivide(A,B,0) in RatRed2

int rational::rationalverif = 0;
long rational::ratredcalls = 0;
long long rational::ratredtime = 0;

#ifdef TLS
const bool rational::_init = init();
bool rational::init() {
	longint::init();
	return true;
}
#endif	// TLS
const rational rational::ZERO((int32_t) 0);
const rational rational::ONE((int32_t) 1);
const rational rational::TWO((int32_t) 2);
const rational rational::FOUR((int32_t) 4);
const rational rational::FIVE((int32_t) 5);
const rational rational::EIGHT((int32_t) 8);
const rational rational::TEN((int32_t) 10);
const rational rational::SIXTEEN((int32_t) 16);
const rational rational::KILO((int32_t) 1000);
const rational rational::KIL((int32_t) 1024);
const rational rational::MILLION((int32_t) 1000000);
const rational rational::MEGA((int32_t) 1048576);

/* Constructors */

rational::rational() : N(0u), D(1u), s(0)
{
}

rational::rational(int32_t val) {
	this->D = (uint32_t)1U;
	if (val > 0) {
		this->N = (uint32_t) val; this->s=1;
	} else if (val == 0) {
		this->N = 0U; this->s=0;
	} else {
		this->N = (uint32_t)(-val); this->s=-1;
	}
}

static int32_t lgcd32(int32_t m, int32_t n);

rational::rational(int32_t n, int32_t d) {
	if (n == 0) {
		this->s = 0; this->N = longint::ZERO; this->D = longint::ONE;
	} else {
	int sign = 1;
	if (n < 0) {
		sign = -sign;
		n = -n;
	}
	if (d < 0) {
		sign = -sign;
		d = -d;
	}
	int32_t g = lgcd32(n,d);
	this->s = sign;
	this->N = longint((uint32_t)(n/g)); this->D = longint((uint32_t)(d/g));
	}
}

rational::rational(const longint& n, const longint& d) {
	if (n == longint::ZERO) {
		this->s = 0;
		this->N = longint::ZERO;
		this->D = longint::ONE;
	} else {
		longint g = longint::lgcd_p(n,d);
		this->s = 1;
		this->N = n/g;
		this->D = d/g;
	}
}

rational::rational(const rational& rhs) : N(rhs.N), D(rhs.D), s(rhs.s) {
}

rational::~rational() {
}

/* Overloaded operators */

rational& rational::operator=(uint32_t rhs) {	// 2013.Sep.14 by samukawa
	this->D = (uint32_t) 1U;
	if(rhs != 0){
		this->N = (uint32_t) rhs; this->s=1;
	} else {
		this->N = (uint32_t) 0U; this->s=0;
	}
	return *this;
}

rational& rational::operator=(int32_t rhs) {	// Rlset
	this->D = (uint32_t) 1U;
	if(rhs > 0){
		this->N = (uint32_t) rhs; this->s=1;
	} else if(rhs == 0){
		this->N = (uint32_t) 0U; this->s=0;
	} else {
		this->N = (uint32_t)(-rhs); this->s=-1;
	}
	return *this;
}

rational& rational::operator=(const rational& rhs) {
	this->N = rhs.N;
	this->D = rhs.D;
	this->s = rhs.s;
 	return *this;
}

rational& rational::operator+=(const rational& rhs) {
	(*this) = RatAdd(*this,rhs);
	return *this;
}

rational rational::operator+(const rational& rhs) const {
	return RatAdd(*this,rhs);
}

rational& rational::operator-=(const rational& rhs) {
	rational res = RatSub(*this,rhs);
	swap(res,*this);
	return *this;
}
rational rational::operator-(const rational& rhs) const {
	return RatSub(*this,rhs);
}

rational& rational::operator*=(const rational& rhs) {
	rational res = RatMul(*this,rhs);
	swap(res,*this);
	return *this;
}

rational rational::operator*(const rational& rhs) const {
	rational result;
	result = RatMul(*this,rhs);
	return result;
}

rational rational::operator/(const rational& rhs) const {
	return RatDiv(*this,rhs);
}

rational rational::operator-() const {
	rational result(*this);
	if (result.s != 0) result.s = -result.s;
	return result;
}

bool rational::operator==(const rational& val) const {
	return RatCmp(*this,val) == 0;
}

bool rational::operator!=(const rational& val) const {
	return RatCmp(*this,val) != 0;
}

bool rational::operator<(const rational& val) const {
	return RatCmp(*this,val) < 0;
}

bool rational::operator>(const rational& val) const {
	return RatCmp(*this,val) > 0;
}

bool rational::operator<=(const rational& val) const {
	return RatCmp(*this,val) <= 0;
}

bool rational::operator>=(const rational& val) const {
	return RatCmp(*this,val) >= 0;
}

rational::operator double() const {
	int s = this->s;
	longint N = this->N;
	longint D = this->D;
	if (s == 0 || N.getPrecision() == 0) return 0.0;
	while (N.getPrecision() > 4 && D.getPrecision() > 4) {
		N >>= 32;
		D >>= 32;
	}
	return (s > 0 ? 1.0 : -1.0)*((double)N)/((double)D);
}

/* Member functions */

rational rational::abs() const {
	rational res(*this);
	if (res.s < 0) res.s = 1;
	return res;
}

rational rational::inv() const {
	longint ln=this->numerator(); longint ld=this->denominator();
	rational res=RRset(ld,ln);
	return res;
}

rational rational::max(const rational& val) const {
	rational res;
	if (*this > val) {
		res = *this;
	} else {
		res = val;
	}
	return res;
}

rational rational::min(const rational& val) const {
	rational res;
	if (*this < val) {
		res = *this;
	} else {
		res = val;
	}
	return res;
}


rational rational::floor() const {
	rational res = *this;
	if (res.s == 0) {
		res.N = longint::ZERO;
		res.D = longint::ONE;
	} else if (res.s > 0) {
		res.s = 1;
		res.N = this->numerator()/this->denominator();
		res.D = longint::ONE;
	} else {
		res.s = -1;
		res.N = (this->numerator()+this->denominator()-longint::ONE)/this->denominator();
		res.D = longint::ONE;
	}
	if (res.N == longint::ZERO) { res.s = 0;}
	return res;
}

rational rational::ceil() const {
	rational res = *this;
	if (res.s == 0) {
		res.N = longint::ZERO;
		res.D = longint::ONE;
	} else if (res.s > 0) {
		res.s = 1;
		res.N = (this->numerator()+this->denominator()-longint::ONE)/this->denominator();
		res.D = longint::ONE;
	} else {
		res.s = -1;
		res.N = this->numerator()/this->denominator();
		res.D = longint::ONE;
	}
	if (res.N == longint::ZERO) { res.s = 0;}
	return res;
}

void rational::set(const longint& n, const longint &d) {
	N = n;
	D = d;
	s = 1;
	if (longint::lcmp(N,longint::ZERO) == 0) s = 0;
}

void rational::sset(int sign, const longint& n, const longint &d) {
	N = n;
	D = d;
	if (sign >= 0) s = 1; else s = -1;
	if (longint::lcmp(n,longint::ZERO) == 0) s = 0;
}

void rational::RatPrt(const std::string& str) {   /* decimal print out rational */

	if(this->s > 0){
		printf("RatPrt  Num "); longint::decprt(str,this->N);
	} else if(this->s == 0){
		printf("RatPrt Zero  ");
	} else {
		printf("RatPrt -Num "); longint::decprt(str,this->N);
	}
	printf("        Den "); longint::decprt(str,this->D);
	return;
}

void rational::RatNdPrt(const std::string& str) const {   /* Fractional print out rational */
	longint B,X,Z;
	int i=0,d[1000],iexit,k;
	uint64_t r;

	if(this->s > 0){
		printf("%s:+",str.c_str());
	}else if(this->s == 0){
		printf("%s:=0\n",str.c_str()); return;
	}else{
		printf("%s:-",str.c_str());
	}

	Z = longint::lset(0L); B=this->N;
        do{
		X = longint::sdiv(B, 10LU, &r); d[i] = r; i = i + 1;
                iexit = longint::longint::lcmp(X, Z); B = longint::ladd(X, Z);
	}while(iexit != 0);
	for(k=i-1; k>=0; k--) printf("%d", d[k]);
	printf("/");

	Z = longint::lset(0L); B=this->D; i=0;
        do{
		X = longint::sdiv(B, 10LU, &r); d[i] = r; i = i + 1;
                iexit = longint::longint::lcmp(X, Z); B = longint::ladd(X, Z);
	} while(iexit != 0);
	for(k=i-1; k>=0; k--) printf("%d", d[k]);
	putchar('\n');
	return;
}

//static int ndd(int n)
int ndd(int n)
{
	int l;
	if (n == 0)
		l = 1;
	else {
		l = 0;
		while (n > 0) {
			l = l + 1;
			n = n / 10;
		}
	}
	return l;
}

std::string rational::format(int w, int d) const
{
	rational val = *this;
	std::stringstream res;
	std::stringstream ss;
	longint q, r;
	int wi = w-d-1;
	rational m;
	longint ml = longint::ONE;
	if (d > 0) {
		ml=pow(longint::TEN,d);
		m=RRset(longint::ONE,ml);
		m = m / rational::TWO;
		if (val.s < 0)
			val -= m;
		else
			val += m;
	}

	q = longint::ldivide(val.N,val.D,&r);
	ss << q;
	std::string ns = ss.str();
	int nl = ns.length();
	int sl = val.s < 0 ? 1 : 0;
	if (nl+sl <= wi) {
		if (val.s < 0) --wi;
		while (wi > ((int)ns.length())) {
			res << ' ';
			--wi;
		}
		if (sl > 0) res << '-';
		res << ns;
	} else if (wi >= 2+ndd(nl)+11) {
		int w2 = wi-sl-ndd(nl)-11;
		int w3 = w2/2;
		w2 = w2 - w3;
		for (int i=0; i < w2; ++i) res << ns[i];
		res << " (" << (nl - w2 - w3) << " digits) ";
		for (int i=nl-w3; i < nl; ++i) res << ns[i];
	} else {
		for (int i=0; i < wi; ++i) res << '*';
	}
	res << '.';    //     Here is a point.

	longint N, D;
	rational t;
	int len = 0;
	t = RRset(r,val.D);
	while (len < d)
	{
		longint tn = t.N; longint td = t.D; 
		tn = tn * longint::TEN;
		t = RRset(tn,td);
		q = longint::ldivide(t.N,t.D,&r);
		res <<  q;
		t = RRset(r,t.D);
		++len;
	}
	return res.str();
}

//----------------------------
rational Rlset(int32_t n){  /* setting rational by long int */
	rational X; // int i;

	X.D = longint::lset(1L);
	if(n > 0){
		X.N = longint::lset(n); X.s=1;
	} else if(n == 0){
		X.N = longint::lset(0L); X.s=0;
	} else {
		X.N = longint::lset(-n); X.s=-1;
	}
	return X;
}

rational Rfset(float *x){  /* setting rational by float */
	double t = (double)*x;
	rational X = Rdset(&t);
	RatRed2(X);
	return X;
}

rational Rdset(double *x){  /* setting rational by double */
	typedef	union{
		double r; uint32_t irr[2];
	} equivalenced;
	equivalenced Dfloat;

	uint32_t ir[2], mant;
	int iexp, i, maxe=1023, mine=-1022, mask=0x000fffff, ibias=1023;
	uint64_t n;
	longint T; rational X;

	Dfloat.r=*x;
// 	printf("Dfloat.r=%llf\n",Dfloat.r); //////////////
	ir[0] = Dfloat.irr[1]; ir[1] = Dfloat.irr[0]; 
    
	iexp = ((ir[0] & 0x7ff00000) >> 20) - ibias;
	mant = ( ir[0] & mask ) + 0x00100000;
	if(iexp==(maxe+1)){
		std::cout << "iexp=" << iexp << "=NaN or Inf, *x=" << *x << " stop by exit(3)" << std::endl;
		printf("ir[0]=%x ir[1]=%x\n",ir[0],ir[1]);  //////////////
		exit(3);
	}
	if(iexp==(mine-1)){
		mant = ir[0] & mask;     /*  DEN  */
		iexp = iexp+1;           /*  DEN  */
	}
	n = (uint64_t)mant*65536*65536 + ir[1];
//	printf("mant=%lu n=%llu iexp=%lu\n",mant,n,iexp);  //////////////
	if(*x > 0){
		X.N = n; X.s=1;
	} else if(*x == 0){             /* 2014mar08 */
		X.N = longint::ZERO; X.s=0;
	} else {
		X.N = n; X.s=-1;
	}
	T = longint::lset(1U);
	if(*x != 0){
		X.D = T;
		if(iexp > 52){
			T = X.N;
			for(i=1; i<=iexp-52; i++){
				X.N = T*longint::TWO;	T=X.N;
			}
		}else{
			for(i=1; i<=52-iexp; i++){
				X.D = T*longint::TWO;	T=X.D;
			}
		}
	}
	RatRed2(X);
	return X;
}

rational RRset(longint N, longint D){  /* setting rational by two rational */
	rational X; // int i;

	X.N = N; X.D = D; X.s=1;
	if (longint::lcmp(N,longint::ZERO) == 0) X.s = 0;
	return X;
}

rational RatAbs(const rational& A){
	rational X;
	
	X=A;
	if(X.s < 0) X.s = 1;
	return X;
}

rational RatAdd(const rational& X, const rational& Y){     /* addition rational + rational */
	rational Z; 
	longint XY,YX,XD,YD,E,F,G,T,R; 
	int t,g1;

	if( X.s * Y.s == 0){
		if( X.s == 0){
			Z=Y;                   /*  X=0,(Y=0,Y != 0)  */
		} else {
			Z=X;                   /*  (X=0,X != 0),Y=0, */
		}
	} else {
#ifdef RAT_REDADD
		/*******   pre-process       *************************/
		G = longint::lgcd_p(X.D, Y.D);
		g1 = longint::lcmp(G, longint::ONE);
		if( g1 > 0){
			XD = longint::ldivide(X.D, G, &R); YD = longint::ldivide(Y.D, G, &R);
			XY = longint::lmul(X.N, YD);   YX = longint::lmul(Y.N, XD);
		} else {
			XY = longint::lmul(X.N, Y.D);  YX = longint::lmul(Y.N, X.D);
		}
		/*******   end of pre-process   ***********************/

		if( X.s * Y.s == 1){            /* X,Y same sign   */
			if( g1 > 0){
				T=longint::ladd(XY,YX); E=longint::lgcd_p(T,G);
				Z.N=longint::ldivide(T,E,&R);
				F=longint::ldivide(Y.D,E,&R);
				Z.D=XD*F;
			} else {
				Z.N = longint::ladd(XY,YX); Z.D = X.D*Y.D;
			}
			if( X.s == 1) Z.s=1; else Z.s=-1;
		} else {              /* X,Y different sign OR result=0 */
			if( g1 > 0){
				t=longint::lsubv(XY,YX,T,2);
				if( t != 0) {
			 		if( X.s == 1) Z.s= t; else Z.s=-t;
					E=longint::lgcd_p(T,G);  Z.N=longint::ldivide(T,E,&R);
					F=longint::ldivide(Y.D,E,&R); Z.D=XD*F;
				} else {
					Z.N = longint::lset(0L); Z.D = longint::lset(1L); Z.s = 0;
				}
			} else {
				t = longint::lsubv(XY,YX,Z.N,3);
				if( t != 0) { 
			 		if( X.s == 1) Z.s= t; else Z.s=-t;
					Z.D=X.D*Y.D;
				} else {
					Z.N = longint::lset(0L); Z.D = longint::lset(1L); Z.s = 0;
				}
			}
		}   /******* X.s * Y.s == 1   ***********/
		if( g1 == 0) RatRed2(Z);
#else
		XY = longint::lmul(X.N, Y.D);
		YX = longint::lmul(Y.N, X.D);
		Z.D = longint::lmul(X.D, Y.D);
		if( X.s * Y.s == 1){
			if( X.s == 1){
				Z.N = longint::ladd(XY, YX); Z.s = 1;   /*  X>0,Y>0  */
			} else {
				Z.N = longint::ladd(XY, YX); Z.s =-1;   /*  X<0,Y<0  */
			}
		} else if(  X.s * Y.s == 0){
			if( X.s == 0){
				Z=Y;                   /*  X=0,(Y=0,Y != 0)  */
			} else {
				Z=X;                   /*  (X=0,X != 0),Y=0, */
			}
		} else {
			t = longint::lcmp( XY , YX);
			if( X.s == 1){                         /*  X>0, Y<0   */
				if( t > 0 ) {
				Z.N = longint::lsub(XY, YX); Z.s = 1; 
				} else if( t == 0 ) {
					Z.N = longint::lset(0L); Z.D = longint::lset(1L); Z.s = 0; 
				} else {
					Z.N = longint::lsub(YX, XY); Z.s = -1;
				}
			} else {                               /*  X<0, Y>0   */
				if( t > 0 ){
					Z.N = longint::lsub(YX, XY); Z.s = -1;
				} else if( t == 0 ) {
					Z.N = longint::lset(0L); Z.D = longint::lset(1L); Z.s = 0; 
				} else {
					Z.N = longint::lsub(XY, YX); Z.s = 1;
				}
			}
		}
		RatRed2(Z);
#endif
	}    /****  X.s * Y.s == 0   ***********/
	return Z;
}

rational RatSub(const rational& X, const rational& Y){     /* subtraction rational - rational */
	rational W, Z;

	W = Y; W.s = -Y.s;
	if(  X.s * Y.s != 0){
		Z = RatAdd ( X, W);
	}else{
		if( X.s == 0){
			Z=W;
		} else {
			Z=X;
		}
	}
	return Z;
}

rational RatMul(const rational& X, const rational& Y){     /* multiplication rational * rational */
	rational Z;
	if( X.s * Y.s == 0){
		Z = Rlset(0);
	} else {
#ifdef RAT_REDUCE
	longint E, R;
	rational XX, YY;
		XX = X;
		YY = Y;
//		E = lgcd_p(XX.N,YY.D);
		E = longint::lgcd2(XX.N,YY.D);
		if (longint::lcmp(E,longint::ONE) != 0) {
#ifdef TUNE
			XX.N = longint::ldivide(XX.N,E,0);
			YY.D = longint::ldivide(YY.D,E,0);
#else
			XX.N = longint::ldivide(XX.N,E,&R);
			YY.D = longint::ldivide(YY.D,E,&R);
#endif
		}
		E = longint::lgcd2(YY.N,XX.D);
		if (longint::lcmp(E,longint::ONE) != 0) {
#ifdef TUNE
			XX.D = longint::ldivide(XX.D,E,0);
			YY.N = longint::ldivide(YY.N,E,0);
#else
			XX.D = longint::ldivide(XX.D,E,&R);
			YY.N = longint::ldivide(YY.N,E,&R);
#endif
		}
		Z.N = longint::lmul(XX.N, YY.N);
		Z.D = longint::lmul(XX.D, YY.D);
		Z.s = X.s * Y.s;
#else
		Z.N = longint::lmul(X.N, Y.N);
		Z.D = longint::lmul(X.D, Y.D);
		Z.s = X.s * Y.s;
		RatRed2(Z);
#endif
	}
	return Z;
}

rational RatDiv(const rational& X, const rational& Y){   /* division rational / rational */
	rational Z;

	if(Y.s == 0){
		printf("RatDiv zero divide\n"); exit(3);
	}
	if(X.s == 0) return X;

#ifdef RAT_REDUCE
	longint E, R;
	rational XX, YY;
	XX = X;
	YY = Y;
	E = longint::lgcd2(XX.N,YY.N);
	if (longint::lcmp(E,longint::ONE) != 0) {
#ifdef TUNE
		XX.N = longint::ldivide(XX.N,E,0);
		YY.N = longint::ldivide(YY.N,E,0);
#else
		XX.N = longint::ldivide(XX.N,E,&R);
		YY.N = longint::ldivide(YY.N,E,&R);
#endif
	}
	E = longint::lgcd2(YY.D,XX.D);
	if (longint::lcmp(E,longint::ONE) != 0) {
#ifdef TUNE
		XX.D = longint::ldivide(XX.D,E,0);
		YY.D = longint::ldivide(YY.D,E,0);
#else
		XX.D = longint::ldivide(XX.D,E,&R);
		YY.D = longint::ldivide(YY.D,E,&R);
#endif
	}

	Z.N = XX.N * YY.D;
	Z.D = XX.D * YY.N;
	Z.s = X.s * Y.s;
#else  // RAT_REDUCE
	Z.N = X.N * Y.D;
	Z.D = X.D * Y.N;
	Z.s = X.s * Y.s;
	RatRed2(Z);
#endif
	return Z;
}

void RatRed2(rational& A){   /* in-place reduce numerator and denominator */
	longint LONE, R, Z;
	if(A.s != 0) {
		Z = longint::lgcd_p(A.N,A.D);
		if( longint::longint::lcmp(Z,longint::ONE) != 0 ) {	// GCD is not one
#ifdef TUNE
			A.N = longint::ldivide(A.N,Z,0);
			A.D = longint::ldivide(A.D,Z,0);
#else
			A.N = longint::ldivide(A.N,Z,&R);
			A.D = longint::ldivide(A.D,Z,&R);
#endif
		}
	}
}

int RatCmp(const rational& A, const rational& B){   /* Compare */
	rational C; int i;

	C = RatSub(A, B);
	if(C.s == 0) i=0;
	if(C.s <  0) i=-1;
	if(C.s >  0) i=1;
	return i;
}

void RatPrt(char *str, rational A){   /* decimal print out rational */

	if(A.s > 0){
		printf("RatPrt  Num "); longint::decprt(str,A.N);
	} else if(A.s == 0){
		printf("RatPrt Zero  ");
	} else {
		printf("RatPrt -Num "); longint::decprt(str,A.N);
	}
	printf("        Den "); longint::decprt(str,A.D);
	return;
}

std::ostream& operator<<(std::ostream& stream, const rational& rhs) {
	if (rhs.s < 0) stream << "-";
//	stream << "(" << rhs.N << "/" << rhs.D << ")";
	stream << rhs.N << "/" << rhs.D;
	return stream;
}

static int32_t lgcd32(int32_t m, int32_t n) {
	int32_t t;
	if (m < n) { t = m; m = n; n = t; }
	while (n != 0) {
		t = m % n; m = n; n = t;
	}
	return m;
}

double ndpw2(const longint& a, int& pw2)// Normalized Double number with Power of 2,  
{ // extract 53 bits from a, normalize 1 <= f < 2, convert f to double, a = f * 2 ** pw2.
	longint f;  // fraction part converted to double precision number
	pw2=numbits(a)-1;
	if(numbits(a) < 53){
		f=a;
		for(unsigned int k=0; k<53-numbits(a); k++) f *= longint::TWO;
	}else{
//		f = a >> (numbits(a) - 53);
		if(a.l > 4){                            // 2016oct02
			longint T1(a.d[a.l]);           // 2016oct02
			longint T2(a.d[a.l-1]);         // 2016oct02
			longint T3(a.d[a.l-2]);         // 2016oct02
			longint T=(T1*longint::FOURGIGA+T2)*longint::FOURGIGA+T3; // 2016oct02
			f = T >> (numbits(T) - 53);     // 2016oct02
		}else{                                  // 2016oct02
			f = a >> (numbits(a) - 53);
		}                                       // 2016oct02
	}
	rational na(f,longint::FOURPETA);
	return (double)na;
}

double ndpw2(const rational& a, int& pw2)// Normalized Double number with Power of 2,  
{
	int anp2,adp2;
	longint an=a.N;
	longint ad=a.D;
	double dan=ndpw2(an,anp2);
	double dad=ndpw2(ad,adp2);
	double da=dan/dad;
	pw2=anp2-adp2;
	if(da >= 2){ da=da/2; pw2++; }
	if(da < 1){ da=da*2; pw2--; }
	if(a.s < 0) da=-da;
	return da;
}

double roughdec(const double& f, const int& n, int& m)
{
	int q = n / 10L; int r = n % 10L;
	double g = f * pow(2.0,static_cast<double>(r)) * pow(1.024,static_cast<double>(q));
	m = 3*q;
	return g;
}

std::istream& operator>>(std::istream& is, rational& rhs)
{
	char c;
	longint inum;
	int iexp;
	inum = (uint32_t)0u; iexp = 0;	// inum*10^iexp
	is.get(c);
	while (c == ' ') {	// BUG: should handle tab, as well
		is.get(c);
	}
	if (c >= '0' && c <= '9') {
		inum = inum*longint::TEN + longint((uint32_t)(c - '0'));
		is.get(c);
		while (c >= '0' && c <= '9') {
			inum = inum*longint::TEN + longint((uint32_t)(c - '0'));
			is.get(c);
		}
		if (c == '.') {
			is.get(c);
			if (c >= '0' && c <= '9') {
				inum = inum*longint::TEN + longint((uint32_t)(c - '0'));
				iexp += 1;
				is.get(c);
				while (c >= '0' && c <='9') {
					inum = inum*longint::TEN + longint((uint32_t)(c - '0'));
					iexp += 1;
					is.get(c);
				}
			}
		}
	}
	is.putback(c);
	rhs.s = 1;
	rhs.N = inum;
	rhs.D = (uint32_t)1u; while (iexp-- > 0) rhs.D = rhs.D*longint::TEN;
	RatRed2(rhs);
	return is;
}

rational const& min(rational const& x, rational const& y)
{
	if (x < y) return x; else return y;
}

rational const& max(rational const& x, rational const& y)
{
	if (x > y) return x; else return y;
}

iarchive& operator>>(iarchive& ia, rational& r)
{
    ia.getIs() >> r.s;
    ia >> r.N >> r.D;
    return ia;
}

oarchive& operator<<(oarchive& oa, const rational& r)
{
    oa.getOs() << " " << r.s;
    oa << r.N << r.D;
    return oa;
}
