#ifndef RATIONAL_H
#define RATIONAL_H

#include <stdio.h>
#include <stdlib.h>
#include <string>
#include "longint.h"
#include "archive.h"
//#include "rvector.h"
//#include "rmatrix.h"

class rational {
public:
	longint N, D;
	int s;
	static int rationalverif;
	static long ratredcalls;
	static long long ratredtime;

	// Constructors
	rational();
	rational(int32_t);
	rational(int32_t,int32_t);
	rational(const longint&, const longint&);
	rational(const rational&);
	~rational();

#ifdef TLS
	static const bool _init;
	static bool init();
#endif
	// Constants
	static const rational ZERO;
	static const rational ONE;
	static const rational TWO;
	static const rational FOUR;
	static const rational FIVE;
	static const rational EIGHT;
	static const rational TEN;
	static const rational SIXTEEN;
	static const rational KILO;
	static const rational KIL;
	static const rational MILLION;
	static const rational MEGA;
	
	// Overloaded operators
	rational& operator=(const rational&);		// copy constructor
	rational& operator=(uint32_t);			// 2013.Sep.14 samukawa
	rational& operator=(int32_t);					// Rlset
	rational& operator+=(const rational&);
	rational& operator-=(const rational&);
	rational& operator*=(const rational&);
	rational operator+(const rational&) const;	// RatAdd
	rational operator-(const rational&) const;	// RatSub
	rational operator*(const rational&) const;	// RatMul
	rational operator/(const rational&) const;	// RatDiv
	rational operator-() const;					// unary minus
	bool operator==(const rational&) const;
	bool operator!=(const rational&) const;
	bool operator<(const rational&) const;
	bool operator>(const rational&) const;
	bool operator<=(const rational&) const;
	bool operator>=(const rational&) const;
	operator double() const;

	// Member functions
	longint numerator() const { return N; }
	longint denominator() const { return D; }
	rational abs() const;
	rational inv() const;
	rational max(const rational&) const;
	rational min(const rational&) const;
	rational floor() const;
	rational ceil() const;
	void set(const longint& n, const longint& d);
	void sset(int sign, const longint& n, const longint& d);
	void RatPrt(const std::string&);
	void RatNdPrt(const std::string&) const;
	int sign() { return s; }
	std::string format(int w, int d) const;
	static void swap(rational& x, rational& y)
	{
		std::swap(x.s,y.s);
		longint::swap(x.N,y.N);
		longint::swap(x.D,y.D);
	}
	friend rational const& min(rational const&, rational const&);
	friend rational const& max(rational const&, rational const&);
	friend iarchive& operator>>(iarchive& ia, rational& r);
	friend oarchive& operator<<(oarchive& oa, rational const& r);
};

rational Rlset(int32_t);
rational Rfset(float*);
rational Rdset(double*);
rational RRset(longint, longint);
rational RatAbs(const rational&);
rational RatAdd(const rational&, const rational&);
rational RatSub(const rational&, const rational&);
rational RatMul(const rational&, const rational&);
rational RatDiv(const rational&, const rational&);
void RatRed2(rational&);
int RatCmp(const rational&, const rational&);
void RatPrt(char*, rational);

int ndd(int n);
//longint setpow10(const int n);
double ndpw2(const longint& a, int& pw2);
double ndpw2(const rational& a, int& pw2);
double roughdec(const double& f, const int& n, int& m);

std::istream& operator>>(std::istream&, rational&);
std::ostream& operator<<(std::ostream&, const rational&);

//typedef vector<rational> rational_vector;
//typedef matrix<rational> rational_matrix;
//

#endif
