#ifndef _INTERFACE4_H
#define _INTERFACE4_H

#include <iostream>
#include <cstdlib>
#include <cassert>
#include <typeinfo>
#include "rvector.h"
#define PLACEMENT_NEW

namespace rat {

//template<typename> class matrix_section;

template<typename DerivedT>
class imatrix
{
	DerivedT& true_this()
	{
		return static_cast<DerivedT&>(*this);
	}

	DerivedT const& true_this() const
	{
		return static_cast<DerivedT const&>(*this);
	}
public:
	typename matrix_traits<DerivedT>::RowT const& operator[] (int idx) const
	{
		return true_this()[idx];
	}

	typename matrix_traits<DerivedT>::RowT& operator[] (int idx)
	{
		return true_this()[idx];
	}

	int rows() const
	{
		return true_this().rows();
	}

	int columns() const
	{
		return true_this().columns();
	}
};

template<typename T>
class matrix : public imatrix<matrix<T> >
{
	int rows_;
	int columns_;
	vector<T>* m_;
public:
//	typedef T value_type;
//	typedef vector<T> row_type;
	matrix(int rows, int columns) : rows_(rows), columns_(columns), m_(0) {
		m_ = new vector<T>[rows_];
		for (int i=0; i < rows_; ++i) {
//			m_[i] = *(new vector<T>(columns_));
			new (m_+i) vector<T>(columns_);
	    }
	}
	matrix(const matrix<T>& rhs) : rows_(rhs.rows_), columns_(rhs.columns_), m_(0) {
		m_ = new vector<T>[rows_];
		for (int i=0; i < rows_; ++i) {
//			m_[i] = *(new vector<T>(columns_));
			new (m_+i) vector<T>(columns_);
	    }
		*this = rhs;
	}

	~matrix() {
//		for (int i=0; i < rows_; ++i) {
//			delete &m_[i];
//		}
		delete[] m_;
	}

	matrix<T>& operator=(const matrix<T>& rhs) {
		if (this != &rhs) {
			assert(rows_==rhs.rows_ && columns_==rhs.columns_);
			for (int i=0; i < rows_; ++i) {
				m_[i] = rhs.m_[i];
			}
		}
		return *this;
	}
	const vector<T>& operator [](int idx) const { return m_[idx]; }
	vector<T>& operator [](int idx) { return m_[idx]; }
	int rows() const { return rows_; }
	int columns() const { return columns_; }

	matrix_column<T>& column(int index) const {
		matrix_column<T>* res = new matrix_column<T>(*this,index);
		return *res;
	}

	matrix_column<T>& column(int index, int row_begin, int row_end, int row_stride) const {
		matrix_column<T>* res = new matrix_column<T>(*this,index,row_begin,row_end,row_stride);
		return *res;
	}

	lmatrix_column<T>& lcolumn(int col) {
		lmatrix_column<T>* res = new lmatrix_column<T>(*this,col);
		return *res;
	}

	lmatrix_column<T>& lcolumn(int col, int row_begin, int row_end, int row_stride) {
		lmatrix_column<T>* res = new lmatrix_column<T>(*this,col,row_begin,row_end,row_stride);
		return *res;
	}

	matrix_row<T>& row(int index) const {
		matrix_row<T>* res = new matrix_row<T>(*this,index);
		return *res;
	}

	matrix_row<T>& row(int index, int col_begin, int col_end, int col_stride) const {
		matrix_row<T>* res = new matrix_row<T>(*this,index,col_begin,col_end,col_stride);
		return *res;
	}

	lmatrix_row<T>& lrow(int index) {
		lmatrix_row<T>* res = new lmatrix_row<T>(*this,index);
		return *res;
	}

	lmatrix_row<T>& lrow(int index, int col_begin, int col_end, int col_stride) {
		lmatrix_row<T>* res = new lmatrix_row<T>(*this,index,col_begin,col_end,col_stride);
		return *res;
	}

	matrix_section<T> const& section(int row_begin, int row_end, int row_stride,
									int col_begin, int col_end, int col_stride);

	lmatrix_section<T>& lsection(int row_begin, int row_end, int row_stride,
								int col_begin, int col_end, int col_stride);

	matrix_diagonal<T>& diagonal() const {
		matrix_diagonal<T>* res = new matrix_diagonal<T>(*this);
		return *res;
	}

	lmatrix_diagonal<T>& diagonal() {
		lmatrix_diagonal<T>* res = new lmatrix_diagonal<T>(*this);
		return *res;
	}

	lmatrix_diagonal<T>& ldiagonal() {
		lmatrix_diagonal<T>* res = new lmatrix_diagonal<T>(*this);
		return *res;
	}
};

template<typename T>
class matrix_section : public imatrix<matrix_section<T> >
{
	int rows_;
	int columns_;
	vector_section<T>* rv_;
public:
//	typedef T value_type;
//	typedef vector_section<T> row_type;
	matrix_section() : rows_(0), columns_(0), rv_(0) {}
	matrix_section(matrix<T> const& m, int row_begin, int row_end, int row_stride,
				   int col_begin, int col_end, int col_stride)
		: rows_((row_end-row_begin+row_stride)/row_stride), columns_((col_end-col_begin+col_stride)/col_stride), rv_(0)
	{
		rv_ = new vector_section<T>[rows_];
		for (int i=0; i < rows_; ++i) {
			rv_[i] = *(new vector_section<T>(m[row_begin + row_stride*i],col_begin,col_end,col_stride));
		}
	}
	~matrix_section()
	{
		delete[] rv_;
	}

	vector_section<T> const& operator[](int idx) const { return rv_[idx]; }
	int rows() const { return rows_; }
	int columns() const { return columns_; }
};

template<typename T>
class lmatrix_section : public imatrix<lmatrix_section<T> >
{
	int rows_;
	int columns_;
	lvector_section<T>* rv_;
public:
//	typedef T value_type;
//	typedef vector_section<T> row_type;
	lmatrix_section() : rows_(0), columns_(0), rv_(0) {}
	lmatrix_section(matrix<T>& m, int row_begin, int row_end, int row_stride,
				   int col_begin, int col_end, int col_stride)
		: rows_((row_end-row_begin+row_stride)/row_stride), columns_((col_end-col_begin+col_stride)/col_stride), rv_(0)
	{
		rv_ = new lvector_section<T>[rows_];
#ifdef PLACEMENT_NEW
		for (int i=0; i < rows_; ++i) {
			new (rv_+i) lvector_section<T>(m[row_begin + row_stride*i],col_begin,col_end,col_stride);
		}
#else
		for (int i=0; i < rows_; ++i) {
			rv_[i] = *(new lvector_section<T>(m[row_begin + row_stride*i],col_begin,col_end,col_stride));
		}
#endif
	}
	~lmatrix_section()
	{
		delete[] rv_;
	}

	lvector_section<T> const& operator[] (int idx) const { return rv_[idx]; }
	lvector_section<T>& operator[](int idx) { return rv_[idx]; }
	int rows() const { return rows_; }
	int columns() const { return columns_; }
};

template<typename T>
class matrix_column : public ivector<matrix_column<T> >
{
	matrix<T> const& pm_;
	int col_;
	int row_begin_;
	int row_end_;
	int row_stride_;
public:
	matrix_column(matrix<T> const& m, int col) : pm_(m), col_(col), row_begin_(0), row_end_(m.rows()-1), row_stride_(1)
	{}

	matrix_column(matrix<T> const& m, int col, int row_begin, int row_end, int row_stride) :
		pm_(m), col_(col), row_begin_(row_begin), row_end_(row_end), row_stride_(row_stride)
	{}

	const T& operator[](int idx) const { return pm_[row_begin_+row_stride_*idx][col_]; }

//	T& operator[](int idx) { return (*pm_)[idx][col_]; }

	int len() const { return (row_end_-row_begin_)/row_stride_+1; }
};

template<typename T>
class lmatrix_column : public ivector<lmatrix_column<T> >
{
	matrix<T>* pm_;
	int col_;
	int row_begin_;
	int row_end_;
	int row_stride_;
public:
	lmatrix_column(matrix<T>& m, int col) : pm_(&m), col_(col), row_begin_(0), row_end_(m.rows()-1), row_stride_(1)
	{
	}

	lmatrix_column(matrix<T>& m, int col, int row_begin, int row_end, int row_stride) :
		pm_(&m), col_(col), row_begin_(row_begin), row_end_(row_end), row_stride_(row_stride)
	{
	}

	lmatrix_column& operator=(lmatrix_column const& rhs)
	{
		int nrows = pm_->rows();
		for (int i=0; i < nrows; ++i) {
			(*pm_)[i][col_] = rhs[i];
		}
		return *this;
	}

	template<template<class> class U>
	lmatrix_column& operator=(ivector<U<T> > const& rhs)
	{
		int nrows = pm_->rows();
		for (int i=0; i < nrows; ++i) {
			(*pm_)[i][col_] = rhs[i];
		}
		return *this;
	}

	const T& operator[](int idx) const { return (*pm_)[row_begin_+row_stride_*idx][col_]; }
	T& operator[](int idx) { return (*pm_)[row_begin_+row_stride_*idx][col_]; }
	int len() const { return (row_end_-row_begin_)/row_stride_+1;}
};

template<typename T>
class matrix_row : public ivector<matrix_row<T> >
{
	matrix<T> const& pm_;
	int row_;
	int col_begin_;
	int col_end_;
	int col_stride_;
public:
	matrix_row(matrix<T> const& m, int row) : pm_(m), row_(row), col_begin_(0), col_end_(m.columns()-1), col_stride_(1)
	{}

	matrix_row(matrix<T> const& m, int row, int col_begin, int col_end, int col_stride) :
		pm_(m), row_(row), col_begin_(col_begin), col_end_(col_end), col_stride_(col_stride)
	{}

	const T& operator[](int idx) const { return pm_[row_][col_begin_+col_stride_*idx]; }

	int len() const { return (col_end_-col_begin_)/col_stride_+1; }
};

template<typename T>
class lmatrix_row : public ivector<lmatrix_row<T> >
{
	matrix<T> * pm_;
	int row_;
	int col_begin_;
	int col_end_;
	int col_stride_;
public:
	lmatrix_row(matrix<T>& m, int row) : pm_(&m), row_(row), col_begin_(0), col_end_(m.columns()-1), col_stride_(1)
	{
	}

	lmatrix_row(matrix<T>& m, int row, int col_begin, int col_end, int col_stride) :
		pm_(&m), row_(row), col_begin_(col_begin), col_end_(col_end), col_stride_(col_stride)
	{
	}

	const T& operator[](int idx) const { return (*pm_)[row_][col_begin_+col_stride_*idx]; }

	T& operator[](int idx) { return (*pm_)[row_][col_begin_+col_stride_*idx]; }

	int len() const { return (col_end_-col_begin_)/col_stride_+1; }
};

template<typename T>
class matrix_diagonal : public ivector<matrix_diagonal<T> >
{
	matrix<T> const& m_;
public:
	matrix_diagonal(matrix<T> const& m) : m_(m) {}
	const T& operator[](int idx) const { return m_[idx][idx]; }
	int len() const { return m_.rows() < m_.columns() ? m_.rows() : m_.columns(); }
};

template<typename T>
class lmatrix_diagonal : public ivector<lmatrix_diagonal<T> >
{
	matrix<T>& m_;
public:
	lmatrix_diagonal(matrix<T>& m) : m_(m) {}
	T& operator[](int idx) { return m_[idx][idx]; }
	T const& operator[](int idx) const { return m_[idx][idx]; }
	int len() const { return m_.rows() < m_.columns() ? m_.rows() : m_.columns(); }
};

//template<typename T>
//lmatrix_column<T>& matrix<T>::lcolumn(int index, int row_begin, int row_end, int row_stride) {
//	lmatrix_column<T>* res = new lmatrix_column<T>(*this,index,row_begin,row_end,row_stride);
//	return *res;
//}

//template<typename T>
//lmatrix_row<T>& matrix<T>::lrow(int index){
//	return *(new lmatrix_row<T>(*this,index));
//}

template<typename T>
matrix_section<T> const& matrix<T>::section(int row_begin, int row_end, int row_stride,
											int col_begin, int col_end, int col_stride)
{
	return *(new matrix_section<T>(*this,row_begin,row_end,row_stride,col_begin,col_end,col_stride));
}

template<typename T>
lmatrix_section<T>& matrix<T>::lsection(int row_begin, int row_end, int row_stride,
										int col_begin, int col_end, int col_stride) {
	lmatrix_section<T>* res = new lmatrix_section<T>(*this,row_begin,row_end,row_stride,col_begin,col_end,col_stride);
	return *res;
}

template<class T> iarchive& operator>>(iarchive& ia, matrix<T>& x) {
	int rows = x.rows();
	ia.getIs() >> rows;
	for (int i=0; i < rows; ++i) ia >> x[i];
	return ia;
}

template<class T> oarchive& operator<<(oarchive& oa, matrix<T> const&  x) {
	size_t rows = x.rows();
	oa.getOs() << " " << rows;
	for (size_t i=0; i < rows; ++i) oa << x[i];
	return oa;
}

template<typename T>
std::ostream& operator<<(std::ostream& os, const imatrix<T>& m)
{
	os << "{";
	if (m.rows() > 0) {
		os << m[0];
		for (int i=1; i < m.rows(); ++i) {
			os << m[i];
		}
	}
	os << "}";
	return os;
}

}
#endif
