/*
 * rtraits.h
 *
 *  Created on: 2014/08/16
 *      Author: Osamu
 */

#ifndef RVECTOR_H_
#define RVECTOR_H_
#include <sstream>
#include <stdexcept>
#include "rtraits.h"
#include "archive.h"

namespace rat {

template<typename DerivedT>
class ivector
{
	DerivedT& true_this()
	{
		return static_cast<DerivedT&>(*this);
	}
	DerivedT const& true_this() const
	{
		return static_cast<const DerivedT&>(*this);
	}
public:
	typename vector_traits<DerivedT>::ElementT const& operator[] (int idx) const
	{
		return true_this().operator[](idx);
	}
	typename vector_traits<DerivedT>::ElementT& operator[] (int idx) {
		return true_this().operator[](idx);
	}
	int len() const
	{
		return true_this().len();
	}
};

template<typename T> class vector_section;
template<typename T> class lvector_section;

template<typename T>
class vector : public ivector<vector<T> >
{
	T* vec_;
	int len_;

public:
//	typedef T value_type;

	vector() : vec_(0), len_(0) {}
	vector(int len) : vec_(new T[len]), len_(len) {}
	~vector()
	{
		if (vec_ != 0) delete[] vec_;
	}

	// Assignment operator
	vector<T>& operator =(const vector<T>& rhs) {
		if (this != &rhs) {
			assert(len_ == 0 || len_ == rhs.len_);
			if (len_ == 0) {
				vec_ = new T[rhs.len_];
				len_ = rhs.len_;
			}
			for (int i=0; i < len_; ++i) this->vec_[i] = rhs[i];
		}
		return *this;
	}

	// Copy from ivector<?<T> > to vector<T>
	template<template<typename> class U>
	vector<T>& operator =(ivector<U<T> > const& rhs) {
		assert(len_ == 0 || len_ == rhs.len());
		if (len_ == 0) {
			vec_ = new T[rhs.len()];
			len_ = rhs.len();
		}
		for (int i=0; i < len_; ++i) this->vec_[i] = rhs[i];
		return *this;
	}

	const T& operator [](int idx) const { return vec_[idx]; }
	T& operator [](int idx) { return vec_[idx]; }
	int len() const { return len_; }

	const vector_section<T>& section(int beg, int end, int stride) const {
		vector_section<T>*  res = new vector_section<T>(*this, beg, end, stride);
		return *res;
	}

	lvector_section<T>& lsection(int beg, int end, int stride) {
		lvector_section<T>* res = new lvector_section<T>(*this, beg, end, stride);
		return *res;
	}
};

template<typename T>
class vector_section : public ivector<vector_section<T> >
{
	vector<T> const* vec_;
	int begin_;
	int end_;
	int stride_;
public:
//	typedef T value_type;
	vector_section() : vec_(0), begin_(0), end_(-1), stride_(1) {}
	vector_section(vector<T> const& v, int begin, int end, int stride) :
		vec_(&v), begin_(begin), end_(end), stride_(stride)
	{}

	T const& operator[](int idx) const { return vec_->operator[](begin_+stride_*idx); }
	int len() const { return (end_-begin_+stride_)/stride_; }
};

template<typename T>
class lvector_section : public ivector<lvector_section<T> >
{
	vector<T>* vec_;
	int begin_;
	int end_;
	int stride_;
public:
	lvector_section() : vec_(0), begin_(0), end_(-1), stride_(1) {}

	lvector_section(vector<T>& v, int begin, int end, int stride) :
		vec_(&v), begin_(begin), end_(end), stride_(stride)
	{}

	lvector_section& operator=(lvector_section const& rhs)
	{
		int len = (end_-begin_+stride_)/stride_;
		assert(len == rhs.len());
		for (int i=0; i < len; ++i) {
			vec_->operator[](begin_+stride_*i) = rhs[i];
		}
		return *this;
	}

	template<template<class> class U>
	lvector_section& operator=(ivector<U<T> > const& rhs)
	{
		int len = (end_-begin_+stride_)/stride_;
		assert(len == rhs.len());
		for (int i=0; i < len; ++i) {
			vec_->operator[](begin_+stride_*i) = rhs[i];
		}
		return *this;
	}

	const T& operator[](int idx) const { return vec_->operator[](begin_+stride_*idx); }

	T& operator[](int idx) { return vec_->operator[](begin_+stride_*idx); }

	int len() const { return (end_-begin_+stride_)/stride_; }
};

//template<typename T>
//class matrix_column : public ivector<matrix_column<T> >
//{
//	matrix<T>* m_;
//	int col_;
//public:
//	matrix_column(matrix<T>& m, int col) : m_(&m), col_(col) {}
//	const T& operator[](int idx) const { return (*m_)[idx][col_]; }
//	T& operator[](int idx) { return m_[idx][col_]; }
//	int len() { return m_->columns(); }
//};

//template<typename T>
//void PrintVector(ivector<T>& b)	// note: argument by reference
//{
//	std::cout << "{";
//	if (b.len() > 0) {
//		std::cout << b[0];
//		for (int i=1; i < b.len(); ++i) {
//			std::cout << "," << b[i];
//		}
//	}
//	std::cout << "}";
//}

template<typename T>
iarchive& operator>>(iarchive& ia, vector<T>& v)
{
    int size;
    ia.getIs() >> size;
    if (size != v.len()) {
    	std::stringstream what;
    	what << "archived vector size [" << size << "] != input vector size [" << v.len() << "]";
        throw std::runtime_error(what.str());
    }
    for (int i=0; i < v.len(); ++i) ia >> v[i];
    return ia;
}

template<typename T>
oarchive& operator<<(oarchive& oa, vector<T> const& v)
{
    oa.getOs() << " " << v.len();
    for (int i=0; i < v.len(); ++i) {
        oa << v[i];
    }
    return oa;
}

template<typename T>
std::ostream& operator<<(std::ostream& o, ivector<T> const& b)
{
	o << "{";
	if (b.len() > 0) {
		o << b[0];
		for (int i=1; i < b.len(); ++i) {
			o << "," << b[i];
		}
	}
	o << "}";
	return o;
}

}
#endif /* RVECTOR_H_ */
