/*
 *
 *  g++ -O3 testroughmat.cpp longint.o mempool.o rational.o rough.o interval.o numeric_cast.o ratutil.o gettimeofday.o
 *
 *  ./a.exe 625
 *
 *   det(A)=1.09475e+322
*/
#include <iostream>
#include <stdlib.h>  // atoi
#include <stdio.h>
#include <math.h>
#include <sstream> // std::stringstream
#include <cfloat>
#include <cmath>
#include "longint.h"
#include "rational.h"
//#include "rvector.h"
#include "rmatrix.h"
#include "rough.h"
#include "numeric_cast.h"



using namespace rat;

typedef matrix<rough> rough_matrix;
typedef vector<rough> rough_vector;

rough ldl2(matrix<rough>& a){
    int i,j,k;
    rough s,t;
    int n=a.rows();
    rough d(1.0,0);
    for (j=0; j < n; ++j) {
        for (i=0; i < j; ++i) {
            s=rough(0,0);
            for (k=0; k < i; ++k)  s = s +  a[k][i] * a[k][j];
            a[i][j] = a[i][j] - s;
        }
        s=rough(0,0);
        for (k=0; k <= j-1; ++k) {
            t = a[k][j]/a[k][k];
            s = s + t * a[k][j];
            a[k][j] = t;
        }
        a[j][j] = a[j][j] - s;
	d*=a[j][j];
    }
    return d;
}

void ldlsubst(matrix<rough>& a, vector<rough>& x){
	int i,j,k;
	int n=a.rows();

	for (j=0; j < n-1; ++j) {
		for (i=j+1; i < n; ++i) x[i] = x[i] - a[j][i]*x[j];
	}

	for (k=0; k < n; ++k) x[k] = x[k] / a[k][k];

	for (i=n-1; i >= 0; --i) {
		for (k=0; k <= i-1; ++k) x[k] = x[k] - a[k][i]*x[i];
	}
} 
void dtgene2(matrix<rough>& a, vector<rough>& b){
 	int i, j, m;
 	rough adiag = rough(4.0,0);
 	rough offdg = rough(-1.0,0);
	int n=a.rows();
	m=(int)sqrt((double)n);
    	printf("Check n=%d  m=%d\n",n,m);
 
 	for (i=0; i < m*m; ++i) {
 		for (j=0; j < m*m; ++j) {
 			a[i][j] = rough();
 		}
 	}
 	for (i=0; i < m; ++i) {
 		if (i >= 1) a[i][i-1] = offdg;
 		a[i][i] = adiag;
 		if (((i+1) % m) != 0) a[i][i+1] = offdg;
 		a[i][i+m] = offdg;
 	}
 	for (i=m; i < (m*(m-1)); ++i) {
 		a[i][i-m] = offdg;
 		if (((i+1) % m) != 1) a[i][i-1] = offdg;
 		a[i][i] = adiag;
 		if (((i+1) % m) != 0) a[i][i+1] = offdg;
 		a[i][i+m] = offdg;
 	}
 	for (i=m*(m-1); i < m*m; ++i) {
 		a[i][i-m] = offdg;
 		if (((i+1) % m) != 1) a[i][i-1] = offdg;
 		a[i][i] = adiag;
 		if (((i+1) % m) != 0) a[i][i+1] = offdg;
 	}

//     RHS
 	for (i=0; i < m*m; ++i) b[i]=rough();
 	for (i=0; i < m; ++i) b[i]=rough(40.,0);    // bottom
 	for (i=m*m-m; i < m*m; ++i) b[i]=b[i]+rough(100.0,0);  // upper 
 	for (i=0; i < m*m; i+=m) b[i]=b[i]+rough(100.0,0);    // left 
 	for (i=m-1; i < m*m; i+=m) b[i]=b[i]+rough(100.0,0);   // right
 }

void matprn2(matrix<rough>& a){
  int i,j;
  int n=a.rows();
  int m=a.columns();
  for (i=0; i<m; i++){
     std::cout << i << " ";
     for (j=0; j<n; j++){
       std::cout << a[j][i] << " ";
     }
     std::cout << std::endl;
  }
}

void vecprn(vector<rough>& b){
  int i;
  int n=b.len();
  for (i=0; i<n; i++){
     std::cout << b[i] << " ";
  }
  std::cout << std::endl;
}

int main(int argc, char *argv[]){
  int n = atoi(argv[1]);
  rough_matrix a(n,n);
  rough_vector b(n);

  dtgene2(a,b);
//matprn2(a);
  vecprn(b);

  rough det=ldl2(a);
//matprn2(a);
  std::cout << "det(A)=" << det << std::endl;
  ldlsubst(a,b);
  vecprn(b);
  return 0;
}

