/*
 * thread_manager.cpp
 *
 *  Created on: 2012/12/25
 *      Author: osamu
 */
#include <pthread.h>
#ifdef MacOS
#include <dispatch/dispatch.h>
#else
#include <semaphore.h>
#endif
#include <iostream>
#include <cassert>
#include <stdexcept>
#include "longint.h"
#include "mempool.h"
//#include "mpfft1.h"
#include "thread_manager.h"

using namespace std;

//namespace thread_manager {

//	static bool initialized = init();
	pthread_t threads[thread_manager::num_threads];
#ifdef MacOS
	dispatch_semaphore_t sem_start[thread_manager::num_threads];
	dispatch_semaphore_t sem_result[thread_manager::num_threads];
#else
	sem_t sem_start[thread_manager::num_threads];
	sem_t sem_result[thread_manager::num_threads];
#endif
	int thread_num[thread_manager::num_threads];
	thread_manager::task_t tasks_[thread_manager::num_threads];

	void* ptask(void* p)
	{
		int tid = *((int*)p);
//		std::cout << "thread " << tid << " started." << std::endl;

		// Initialize memory pool for this thread.
		mem_pool* mp = new mem_pool();
#ifdef MacOS
		assert(pthread_getspecific(longint::_mempool) == NULL);
		pthread_setspecific(longint::_mempool,(void*)mp);
#else 	/* MacOS */
		longint::_mempool = mp;
#endif	/* MacOS */
//		std::cout << "mempool initialization successfull" << std::endl;

		// Initialize FFT thread local storage
//		if (pthread_key_create(&longint::_mpffttls,0) != 0) {
//            std::cerr << "pthread_key_create failed." << std::endl;
//            exit(1);
//        }
//		cout << "thread_manager: tid=" << pthread_self() << " key=" << longint::_mpffttls << " at " << &longint::_mpffttls << ")" << std::endl;
//        mpinbd();   /* initialize fft */
//        int m = 12;
//        int c__2=2;
//#ifdef MacOS
//        mpfft1tls_t* pp = (mpfft1tls_t*)pthread_getspecific(longint::_mpffttls);
//#else	/* MacOS */
//	mpfft1tls_t* pp = longint::_mpffttls;
//#endif	/* MacOS */
//std::cout << "pp=" << pp << std::endl;
//        struct _mpcom0& mpcom0 = pp->mpcom0;
//        struct _mpcom1& mpcom1 = pp->mpcom1;
//        mpcom1.nw=pow_ii(&c__2, &m);
//        cout << "  npr=" << mpcom0.npr << endl;
//        mpinix(&m);

		// Wait for parallel task
		while(true) {
#ifdef MacOS
			dispatch_semaphore_wait(sem_start[tid],DISPATCH_TIME_FOREVER);
#else
			sem_wait(&sem_start[tid]);
#endif
			if (tasks_[tid].f == 0) break;		// terminate thread
//			std::cout << "thread_manager::ptask: tid=" << tid << std::endl;
			tasks_[tid].f(tasks_[tid].params);	// execute the task
#ifdef MacOS
			dispatch_semaphore_signal(sem_result[tid]);			// post result
#else
			sem_post(&sem_result[tid]);
#endif
		}
//		std::cout << "thread " << tid << " ended." << std::endl;
		pthread_exit(0);
		return 0;
	}

	thread_manager::thread_manager()
	{
		int i, rc;

//		std::cout << "thread_manager::init()" << std::endl;

#ifdef TLS
		longint::init();
#endif	// TLS

		for (i=1; i < num_threads; ++i) {
#ifdef MacOS
			sem_start[i] = dispatch_semaphore_create(0);
			if (sem_start[i] == 0) {
				throw runtime_error("thread_manager: sem_init() failed.");
			}
#else
			if (sem_init(&sem_start[i],0,0) != 0) {
				throw runtime_error("thread_manager: sem_init() failed.");
			}
#endif
#ifdef MacOS
			sem_result[i] = dispatch_semaphore_create(0);
			if (sem_result[i] == 0) {
				throw runtime_error("thread_manager: sem_init() failed.");
			}
#else
			if (sem_init(&sem_result[i],0,0) != 0) {
				throw runtime_error("thread_manager: sem_init() failed.");
			}
#endif
		}
		for(i=1; i < num_threads; ++i) {
			thread_num[i] = i;
			if ((rc = pthread_create(&threads[i], 0, ptask, (void*)&thread_num[i])) == -1) {
				throw runtime_error("thread_manager: pthread_create() failed");
			}
		}
	}

	thread_manager::~thread_manager()
	{
//		std::cout << "thread_manager::terminate()" << std::endl;

		for(int i=1; i < num_threads; ++i) {
			tasks_[i].f = 0;
			tasks_[i].params = 0;
#ifdef MacOS
			dispatch_semaphore_signal(sem_start[i]);
#else
			sem_post(&sem_start[i]);
#endif
		}
		for (int i=1; i < num_threads; ++i) {
			pthread_join(threads[i],0);
			//dispatch_semaphore_destroy(&sem_start[i]);
			//dispatch_semaphore_destroy(&sem_result[i]);
		}
	}

	void thread_manager::execute(task_t* tasks, int num_tasks)
	{
		int i;

		// std::cout << "thread_manager::execute(): num_tasks=" << num_tasks << std::endl;

		i = 0;
		while (i < num_tasks) {
			int nt;
			if ((num_tasks - i) > num_threads) {
				nt = num_threads;
			} else {
				nt = num_tasks - i;
			}

			tasks_[0].f = tasks[i].f;
			tasks_[0].params = tasks[i].params;
			for (int j=1; j < nt; ++j) {
				tasks_[j].f = tasks[i+j].f;
				tasks_[j].params = tasks[j].params;
#ifdef MacOS
				dispatch_semaphore_signal(sem_start[j]);
#else
				sem_post(&sem_start[j]);
#endif
			}

			tasks_[0].f(tasks_[0].params);

			for (int j=1; j < nt; ++j) {
#ifdef MacOS
				dispatch_semaphore_wait(sem_result[j],DISPATCH_TIME_FOREVER);
#else
				sem_wait(&sem_result[j]);
#endif
			}
			i += nt;
		}
	}

	thread_manager _tm = thread_manager();
//}
